Package: netplot
Title: Beautiful Graph Drawing
Version: 0.4-0
Authors@R: c(
  person("George", "Vega Yon", email = "g.vegayon@gmail.com", role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-3171-0844")),
  person("Porter", "Bischoff", email = "portergbischoff@gmail.com", role = c("aut"), comment = c(ORCID = "0009-0004-6742-6281"))
  )
Description: A graph visualization engine that emphasizes on 
  aesthetics at the same time providing default parameters that yield
  out-of-the-box-nice visualizations. The package is built on top of
  'The Grid Graphics Package' and seamlessly work with 'igraph' and 
  'network' objects.
Depends: R (>= 3.4.0), grid
License: MIT + file LICENSE
Encoding: UTF-8
Imports: graphics, grDevices, igraph, network, sna, stats
Suggests: knitr, rmarkdown, markdown, igraphdata, intergraph, ggraph,
        gridGraphics, ggplot2, gridExtra, gridBase, magrittr, tinytest
VignetteBuilder: knitr
URL: https://github.com/USCCANA/netplot,
        https://usccana.github.io/netplot/
BugReports: https://github.com/USCCANA/netplot/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-23 00:31:26 UTC; runner
Author: George Vega Yon [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3171-0844>),
  Porter Bischoff [aut] (ORCID: <https://orcid.org/0009-0004-6742-6281>)
Maintainer: George Vega Yon <g.vegayon@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-23 10:50:22 UTC

# Generated by roxygen2: do not edit by hand

S3method(nplot,default)
S3method(nplot,igraph)
S3method(nplot,matrix)
S3method(nplot,network)
S3method(print,netplot)
S3method(print,netplot_legend)
export(alter)
export(colorRamp2)
export(color_formula)
export(colorkey)
export(ego)
export(get_edge_gpar)
export(get_gpar)
export(get_vertex_gpar)
export(locate_vertex)
export(make_colors)
export(make_edges_colors)
export(make_vertex_colors)
export(nplot)
export(nplot_base)
export(nplot_legend)
export(npolygon)
export(piechart)
export(segments_gradient)
export(set_edge_gpar)
export(set_gpar)
export(set_vertex_gpar)
import(grid)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,rgb)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(igraph,degree)
importFrom(igraph,ecount)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_with_fr)
importFrom(igraph,vcount)
importFrom(network,as.edgelist)
importFrom(sna,gplot.layout.kamadakawai)
importFrom(stats,quantile)
importFrom(stats,terms)

<!-- README.md is generated from README.Rmd. Please edit that file -->

[![CRAN
status](https://www.r-pkg.org/badges/version/netplot.png)](https://cran.r-project.org/package=netplot)
[![CRAN](https://cranlogs.r-pkg.org/badges/netplot.png)](https://cran.r-project.org/package=netplot)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/rgexf.png)](https://cran.r-project.org/package=rgexf)
[![R](https://github.com/USCCANA/netplot/actions/workflows/ci.yml/badge.svg)](https://github.com/USCCANA/netplot/actions/workflows/ci.yml)
[![Build
status](https://ci.appveyor.com/api/projects/status/3k2m3oq6o99qcs0r?svg=true.png)](https://ci.appveyor.com/project/gvegayon/netplot)
[![USC’s Department of Preventive
Medicine](https://raw.githubusercontent.com/USCbiostats/badges/master/tommy-uscprevmed-badge.svg)](https://keck.usc.edu/cana/)

# netplot

An alternative graph visualization tool that emphasizes aesthetics,
providing default parameters that deliver out-of-the-box lovely
visualizations.

Some features:

1.  Auto-scaling of vertices using sizes relative to the plotting
    device.
2.  Embedded edge color mixer.
3.  True curved edges drawing.
4.  User-defined edge curvature.
5.  Nicer vertex frame color.
6.  Better use of space-filling the plotting device.

The package uses the `grid` plotting system (just like `ggplot2`).

## Installation

You can install the released version of netplot from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("netplot")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("USCCANA/netplot")
```

## Example

This is a basic example which shows you how to solve a common problem:

``` r
library(igraph)
#> 
#> Attaching package: 'igraph'
#> The following objects are masked from 'package:stats':
#> 
#>     decompose, spectrum
#> The following object is masked from 'package:base':
#> 
#>     union
library(netplot)
#> Loading required package: grid
#> 
#> Attaching package: 'netplot'
#> The following object is masked from 'package:igraph':
#> 
#>     ego
set.seed(1)
data("UKfaculty", package = "igraphdata")
l <- layout_with_fr(UKfaculty)
#> This graph was created by an old(er) igraph version.
#>   Call upgrade_graph() on it to use with the current igraph version
#>   For now we convert it on the fly...

plot(UKfaculty, layout = l) # ala igraph
```

<img src="man/figures/README-example-1.png" style="width:85.0%" />

``` r

V(UKfaculty)$ss <- runif(vcount(UKfaculty))
nplot(UKfaculty, layout = l) # ala netplot
```

<img src="man/figures/README-example-2.png" style="width:85.0%" />

``` r
sna::gplot(intergraph::asNetwork(UKfaculty), coord=l)
```

<img src="man/figures/README-example-3.png" style="width:85.0%" />

### UKfaculty

``` r
# Random names
set.seed(1)
nam <- sample(babynames::babynames$name, vcount(UKfaculty))

ans <- nplot(
  UKfaculty,
  layout                = l,
  vertex.color          = ~ Group,
  vertex.nsides         = ~ Group,
  vertex.label          = nam,
  vertex.size.range     = c(.01, .03, 4),
  bg.col                = "transparent",
  vertex.label.show     = .25,
  vertex.label.range    = c(10, 25),
  edge.width.range      = c(1, 4, 5),
  vertex.label.fontfamily = "sans"
  )

# Plot it!
ans
```

<img src="man/figures/README-fig-uk-faculty-1.png" id="fig-uk-faculty"
style="width:85.0%" />

Starting version 0.2-0, we can use gradients!

``` r
ans |>
  set_vertex_gpar(
    element = "core",
    fill = lapply(get_vertex_gpar(ans, "frame", "col")$col, \(i) {
      radialGradient(c("white", i), cx1=.8, cy1=.8, r1=0)
      }))
```

<img src="man/figures/README-fig-uk-faculty-gradient-1.png"
id="fig-uk-faculty-gradient" style="width:85.0%" />

### USairports

``` r
# Loading the data
data(USairports, package="igraphdata")

# Generating a layout naively
layout   <- V(USairports)$Position
#> This graph was created by an old(er) igraph version.
#>   Call upgrade_graph() on it to use with the current igraph version
#>   For now we convert it on the fly...
layout   <- do.call(rbind, lapply(layout, function(x) strsplit(x, " ")[[1]]))
layout[] <- stringr::str_remove(layout, "^[a-zA-Z]+")
layout   <- matrix(as.numeric(layout[]), ncol=2)

# Some missingness
layout[which(!complete.cases(layout)), ] <- apply(layout, 2, mean, na.rm=TRUE)

# Have to rotate it (it doesn't matter the origin)
layout <- netplot:::rotate(layout, c(0,0), pi/2)

# Simplifying the network
net <- simplify(USairports, edge.attr.comb = list(
  weight = "sum",
  name   = "concat",
  Passengers = "sum",
  "ignore"
))

# Pretty graph
nplot(
  net,
  layout            = layout,
  edge.width        = ~ Passengers,
  edge.color        = ~
    ego(col = "white", alpha = 0) +
    alter(col = "yellow", alpha = .75),
  skip.vertex       = TRUE,
  skip.arrows       = TRUE,
  edge.width.range  = c(.75, 4, 4), 
  bg.col            = "black",
  edge.line.breaks  = 10
  )
```

<img src="man/figures/README-fig-us-airports-1.png" id="fig-us-airports"
style="width:85.0%" />

<!-- README.md is generated from README.Rmd. Please edit that file -->
[![CRAN
status](https://www.r-pkg.org/badges/version/netplot)](https://cran.r-project.org/package=netplot)
[![CRAN](https://cranlogs.r-pkg.org/badges/netplot)](https://cran.r-project.org/package=netplot)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/rgexf)](https://cran.r-project.org/package=rgexf)
[![R](https://github.com/USCCANA/netplot/actions/workflows/ci.yml/badge.svg)](https://github.com/USCCANA/netplot/actions/workflows/ci.yml)
[![Build status](https://ci.appveyor.com/api/projects/status/3k2m3oq6o99qcs0r?svg=true)](https://ci.appveyor.com/project/gvegayon/netplot)
[![USC's Department of Preventive Medicine](https://raw.githubusercontent.com/USCbiostats/badges/master/tommy-uscprevmed-badge.svg)](https://preventivemedicine.usc.edu)

# netplot <img src="man/figures/logo.png" align="right" height="200" alt="rgexf hex sticker logo"/>

**netplot** is a graph visualization engine for R that emphasizes
*aesthetics*. Its defaults are chosen so that a single call to `nplot()`
produces a publication-quality figure out of the box, while still giving
you fine-grained control when you need it. It works directly with
`igraph`, `network`, and adjacency-matrix objects.

## Why netplot?

Compared with the base `plot()` methods in `igraph` and `sna`/`network`,
netplot aims to make the *common case beautiful* and the *hard case
possible*:

-   **Beautiful defaults.** Vertices, edges, arrows, and labels are
    auto-scaled *relative to the plotting device*, so figures look right
    regardless of size or aspect ratio and fill the plotting area
    instead of floating in whitespace.
-   **Map data to aesthetics with formulas.** Color, shape, and size
    vertices (and scale edge widths) straight from graph attributes:
    `nplot(g, vertex.color = ~   group, vertex.nsides = ~ group, vertex.size = ~ degree)`.
    Categorical, numeric, and logical attributes are each handled
    sensibly, and a legend is added automatically. See
    `vignette("formulas")`.
-   **Smart edges.** True curved edges with user-defined curvature, an
    embedded edge-color mixer that blends each edge between its
    endpoints’ colors, and edge-width/arrow scaling that respects the
    layout.
-   **Built on `grid`.** Because netplot draws with the `grid` system
    (the same engine as `ggplot2`), plots are first-class grid objects:
    you can post-edit them with `set_vertex_gpar()` / `set_edge_gpar()`,
    arrange several with `gridExtra::grid.arrange()`, add gradients, and
    export cleanly.
-   **Lightweight.** Following the “tinyverse” philosophy, netplot leans
    on base R graphics facilities and keeps its dependency footprint
    small.

A quick feature checklist:

1.  Auto-scaling of vertices, edges, and labels relative to the plotting
    device.
2.  Formula interface to map colors, shapes, and sizes from graph
    attributes.
3.  Embedded edge color mixer (blends edges between endpoint colors).
4.  True curved edges with user-defined curvature.
5.  Nicer vertex frame colors and vertex shapes.
6.  Automatic legends and color keys.
7.  Gradient fills for vertices and edges.
8.  Better use of space, filling the plotting device.

The package uses the `grid` plotting system (just like `ggplot2`).

## Installation

You can install the released version of netplot from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("netplot")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("USCCANA/netplot")
```

## Example

This is a basic example which shows you how to solve a common problem:

``` r
library(igraph)
#> 
#> Attaching package: 'igraph'
#> The following objects are masked from 'package:stats':
#> 
#>     decompose, spectrum
#> The following object is masked from 'package:base':
#> 
#>     union
library(netplot)
#> Loading required package: grid
#> 
#> Attaching package: 'netplot'
#> The following object is masked from 'package:igraph':
#> 
#>     ego
set.seed(1)
data("UKfaculty", package = "igraphdata")
l <- layout_with_fr(UKfaculty)
#> This graph was created by an old(er) igraph version.
#> ℹ Call `igraph::upgrade_graph()` on it to use with the current igraph version.
#> For now we convert it on the fly...

plot(UKfaculty, layout = l) # ala igraph
```

<img src="man/figures/README-example-1.png" style="width:85.0%" />

``` r

V(UKfaculty)$ss <- runif(vcount(UKfaculty))
nplot(UKfaculty, layout = l) # ala netplot
```

<img src="man/figures/README-example-2.png" style="width:85.0%" />

``` r
sna::gplot(intergraph::asNetwork(UKfaculty), coord=l)
```

<img src="man/figures/README-example-3.png" style="width:85.0%" />

### UKfaculty

``` r
# Random names
set.seed(1)
nam <- sample(babynames::babynames$name, vcount(UKfaculty))

ans <- nplot(
  UKfaculty,
  layout                = l,
  vertex.color          = ~ Group,
  vertex.nsides         = ~ Group,
  vertex.label          = nam,
  vertex.size.range     = c(.01, .03, 4),
  bg.col                = "transparent",
  vertex.label.show     = .25,
  vertex.label.range    = c(10, 25),
  edge.width.range      = c(1, 4, 5),
  vertex.label.fontfamily = "sans"
  )

# Plot it!
ans
```

<img src="man/figures/README-fig-uk-faculty-1.png" id="fig-uk-faculty"
style="width:85.0%" />

Starting version 0.2-0, we can use gradients!

``` r
ans |>
  set_vertex_gpar(
    element = "core",
    fill = lapply(get_vertex_gpar(ans, "frame", "col")$col, \(i) {
      radialGradient(c("white", i), cx1=.8, cy1=.8, r1=0)
      }))
```

<img src="man/figures/README-fig-uk-faculty-gradient-1.png"
id="fig-uk-faculty-gradient" style="width:85.0%" />

### USairports

``` r
# Loading the data
data(USairports, package="igraphdata")

# Generating a layout naively
layout   <- V(USairports)$Position
#> This graph was created by an old(er) igraph version.
#> ℹ Call `igraph::upgrade_graph()` on it to use with the current igraph version.
#> For now we convert it on the fly...
layout   <- do.call(rbind, lapply(layout, function(x) strsplit(x, " ")[[1]]))
layout[] <- stringr::str_remove(layout, "^[a-zA-Z]+")
layout   <- matrix(as.numeric(layout[]), ncol=2)

# Some missingness
layout[which(!complete.cases(layout)), ] <- apply(layout, 2, mean, na.rm=TRUE)

# Have to rotate it (it doesn't matter the origin)
layout <- netplot:::rotate(layout, c(0,0), pi/2)

# Simplifying the network
net <- simplify(USairports, edge.attr.comb = list(
  weight = "sum",
  name   = "concat",
  Passengers = "sum",
  "ignore"
))

# Pretty graph
nplot(
  net,
  layout            = layout,
  edge.width        = ~ Passengers,
  edge.color        = ~
    ego(col = "white", alpha = 0) +
    alter(col = "yellow", alpha = .75),
  skip.vertex       = TRUE,
  skip.arrows       = TRUE,
  edge.width.range  = c(.75, 4, 4), 
  bg.col            = "black",
  edge.line.breaks  = 10
  )
```

<img src="man/figures/README-fig-us-airports-1.png" id="fig-us-airports"
style="width:85.0%" />

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----pkgs-and-data, message=FALSE, warning=FALSE------------------------------
library(netplot)
library(igraph)
library(sna)
library(ggraph)

# We will use the UKfaculty data from the igraphdata package
data("UKfaculty", package = "igraphdata")

# With a fixed layout
set.seed(225)
l_ukf <- layout_with_fr(UKfaculty)

## -----------------------------------------------------------------------------
# Function to map base to grid
map_base_to_grid <- function(fun) {
  gridGraphics::grid.echo(fun)
  grid::grid.grab()
}

## ----igraph-sna, fig.keep='none'----------------------------------------------
# Mapping base to grid
ig <- map_base_to_grid(function() plot(UKfaculty, main = "igraph"))
nw <- map_base_to_grid(function() gplot(as.matrix(as_adj(UKfaculty)), coord = l_ukf, main = "sna"))

## ----nplot, fig.keep='none'---------------------------------------------------
nplot(UKfaculty, layout = l_ukf)
grid::grid.text("netplot", x = .05, y=.9)
np <- grid::grid.grab()

## ----fig.width=7, fig.height=5, fig.keep='none'-------------------------------
ggraph::ggraph(UKfaculty, layout = l_ukf) +
  ggraph::geom_edge_arc() +
  ggraph::geom_node_point() +
  ggplot2::ggtitle("ggraph")

gg <- grid::grid.grab()

## ----fig.width=7, fig.height=7, fig.cap="Comparison of default `igraph`, `sna`, `ggraph`, and `netplot` default call. `nplot` fills completely the plotting area, and adjusts vertex size, edge width, and edge arrows' size accordingly to the plotting area and plotting device."----
# Putting all together
gridExtra::grid.arrange(ig, nw, gg, np, nrow=2, ncol=2)

## -----------------------------------------------------------------------------
x_igraph  <- sample_smallworld(1, 20, 4, .05)
x_network <- intergraph::asNetwork(x_igraph)

l <- layout_nicely(x_igraph)

# Putting two plots in the same page (one using igraph and the other network)
gridExtra::grid.arrange(
  nplot(x_igraph, layout = l),
  nplot(x_network, layout = l), ncol=2, nrow=1
)

## ----node-scaling, fig.width=7, fig.height=7, fig.cap="Modifying `vertex.size.range`: Each figure shows a different parameter for the vertex size range. From left to right, and top down: (a) Has all vertices with the same scale of 2.5%, (b) "----

gridExtra::grid.arrange(
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.025, .025)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025, 4)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.02, .05, 4)),
  ncol=2, nrow=2
)



## ----node-shapes, fig.width=4, fig.height=4, fig.cap="Number of sides for node drawing."----
nplot(make_ring(5), vertex.nsides = 2:6,
      vertex.size.range = c(.1,.1),
      edge.curvature = 0)

## ----sim-net------------------------------------------------------------------
set.seed(12233)

x <- sample_smallworld(1, size = 10, nei = 2, .1)
l <- layout_with_fr(x)

## ----edge-color, fig.width=7, fig.height=3, fig.cap="Modifying `edge.color.mix`: Each figure shows a different parameter for the edge color mixer. From left to right, (a) colors the edges as alter, (b) mixes ego and alter's colors, and (c) only uses ego"----

gridExtra::grid.arrange(
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=0, alpha = .1, col="black") + alter(mix=1),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=.5, alpha = .1, col="black") + alter(mix=.5),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=1, alpha = .1, col="black") + alter(mix=0),
    vertex.size.range = c(.05,.1)
    ),
  ncol=3, nrow=1
)


## ----edge-curvature, fig.width=7, fig.height=3, fig.cap="Modifying `edge.curvature`: Each figure shows a different parameter for the edge curvature. From left to right, (a) straight edges, (b) the edge between ego and alter is an arc that measures $\\pi/2$ radians (90 degree), and (c) the edge as an arc between ego and alter that measures $\\pi$ radians (180 degrees)."----
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.curvature = 0, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi/2, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi, vertex.size.range = c(.05,.1)),
  ncol = 3, nrow=1
)

## ----edge-line-type, fig.width=6, fig.height=6, fig.cap="Changing the number of breaks in the edge (arc) and the type of line to be drawn."----
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.line.breaks=3),
  nplot(x, layout = l, edge.line.lty = 2, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 3, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 4, edge.line.breaks=10),
  nrow=2, ncol=2
)

## -----------------------------------------------------------------------------
data(UKfaculty, package = "igraphdata")
# fakenames <- sample

## ----data-sim, warning=FALSE, message=FALSE, fig.width = 6, fig.height = 5----
library(igraph)

# Simulation parameters
set.seed(12)
n <- 100

# Points
x0  <- rnorm(n)
x1  <- x0*.4 + rnorm(n)

# network
net <- sample_smallworld(1, n, 4, .1)

## ----netplot, fig.width = 6, fig.height = 5-----------------------------------
library(netplot)
library(magrittr)

np <- nplot(net, bg.col = "gray70", vertex.nsides = "square") %>%
  set_edge_gpar("line", col = "gray50", alpha = .7) %>%
  set_vertex_gpar("frame", fill = "black", col = "black")

## ----baseplot, fig.width = 6, fig.height = 5----------------------------------
plot(x0, x1)

## ----alltogether, echo=FALSE, fig.width = 6, fig.height = 5-------------------
library(gridBase)
library(grid)

# Base plot and views
plot(x0, x1)
vp <- baseViewports() 

# Adding the plot to the current view
pushViewport(vp$plot)
pushViewport(viewport(.25, .75, width = .5, height = .5))
grid.draw(np)

## ----topright, fig.width = 6, fig.height = 5----------------------------------
plot(x0, x1)
# vp <- baseViewports() 
# pushViewport(vp$figure)
pushViewport(viewport(1, 1, just = c("right", "top"), width = .3, height = .3))
grid.draw(np)

## ----tilted, fig.width = 6, fig.height = 5------------------------------------
plot(x0, x1)
pushViewport(
  viewport(x = .8, y = .8, width = .3, height = .3, angle = -20)
  )
grid.draw(np)

#' netplot: Beautiful Graph Drawing
#'
#' netplot is a graph visualization engine that emphasizes aesthetics while
#' providing default parameters that yield out-of-the-box nice visualizations.
#' It is built on top of the **grid** graphics system (the same engine behind
#' **ggplot2**) and works seamlessly with `igraph` and `network` objects.
#'
#' The main entry point is [nplot()]. See `vignette("examples", package =
#' "netplot")` for a general overview and `vignette("formulas", package =
#' "netplot")` for mapping vertex/edge aesthetics from graph attributes.
#'
#' @keywords internal
"_PACKAGE"

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 6,
  warning = FALSE,
  message = FALSE
)

## ----pkgs---------------------------------------------------------------------
library(netplot)
library(igraph)

## ----data---------------------------------------------------------------------
data("UKfaculty", package = "igraphdata")

set.seed(225)
l <- layout_with_fr(UKfaculty)

# A couple of extra attributes to play with. We qualify igraph::degree()
# explicitly because other packages (e.g. sna) also define a degree().
V(UKfaculty)$indeg  <- igraph::degree(UKfaculty, mode = "in")
V(UKfaculty)$is_hub <- V(UKfaculty)$indeg > stats::median(V(UKfaculty)$indeg)

## ----color-factor, fig.cap="Vertices colored by the categorical `Group` attribute, with an automatic legend."----
nplot(UKfaculty, layout = l, vertex.color = ~ Group)

## ----color-numeric, fig.cap="Vertices colored by in-degree (a numeric attribute) using a continuous gradient, with a color-bar legend."----
nplot(UKfaculty, layout = l, vertex.color = ~ indeg)

## ----color-logical, fig.cap="Vertices colored by a logical attribute (hub vs. non-hub)."----
nplot(UKfaculty, layout = l, vertex.color = ~ is_hub)

## ----shape, fig.cap="Vertex color *and* shape mapped from the same `Group` attribute."----
nplot(
  UKfaculty,
  layout        = l,
  vertex.color  = ~ Group,
  vertex.nsides = ~ Group
)

## ----size, fig.cap="Vertex size mapped from in-degree."-----------------------
nplot(
  UKfaculty,
  layout            = l,
  vertex.size       = ~ indeg,
  vertex.size.range = c(.01, .04, 4)
)

## ----combined, fig.cap="Color, shape, and size all mapped from graph attributes in a single call."----
nplot(
  UKfaculty,
  layout            = l,
  vertex.color      = ~ Group,   # color by department
  vertex.nsides     = ~ Group,   # distinct shape per department
  vertex.size       = ~ indeg,   # size by popularity (in-degree)
  vertex.size.range = c(.01, .04, 4)
)

## ----edge-width, fig.cap="Edge width mapped from the numeric `weight` edge attribute."----
nplot(
  UKfaculty,
  layout           = l,
  edge.width       = ~ weight,
  edge.width.range = c(1, 4, 4),
  skip.arrows      = TRUE
)

## ----edge-color, fig.width=7, fig.height=3, fig.cap="Varying the `mix` between `ego` and `alter` in the edge color formula. Left: alter only. Middle: an even blend. Right: ego only."----
gridExtra::grid.arrange(
  nplot(UKfaculty, layout = l, vertex.color = ~ Group,
        edge.color = ~ ego(mix = 0, alpha = .1) + alter(mix = 1)),
  nplot(UKfaculty, layout = l, vertex.color = ~ Group,
        edge.color = ~ ego(mix = .5, alpha = .1) + alter(mix = .5)),
  nplot(UKfaculty, layout = l, vertex.color = ~ Group,
        edge.color = ~ ego(mix = 1, alpha = .1) + alter(mix = 0)),
  ncol = 3
)

## ----set-gpar, fig.cap="Recoloring an existing plot by attribute using `set_vertex_gpar()`."----
np <- nplot(UKfaculty, layout = l)

set_vertex_gpar(np, element = "core", fill = ~ Group)

#' For internal use
#' Manages naming convention for netplot grobs
#' @noRd
netplot_name <- (function() {
  # grid::gPath("graph", netplot_name$make(idx[[i]]), element)
  # Counter
  index <- 0L

  list(
    # Makes the name of a grob (edge/vertex/graph)
    make = function(idx, for_graph = FALSE) {

      if (for_graph)
        sprintf("graph.%i", idx[1])
      else if (length(idx) == 2)
        sprintf("edge.%i-%i", idx[1], idx[2])
      else if (length(idx) == 1L)
        sprintf("vertex.%i", idx)
      else
        stop("Invalid type.", call. = FALSE)

    },
    # Get the path
    path = function(idx, element = NULL, graph = NULL) {

      # # if no graph, by default the last one
      # if (!length(graph))
      #   graph <- netplot_name$last_graph()

      # Getting type
      type <- switch (length(idx),
        `1` = "vertex",
        `2` = "edge",
        stop("Incorrect number of indices in `idx`.")
      )

      # Validating elements
      np_validate$elements(element, type)

      do.call(
        grid::gPath,
        list(
          "graph", netplot_name$make(idx),
          element
          )
        )
    },
    # Creates a new graph increasing index to 1
    new = function() {

      index <<- index + 1L
      sprintf("graph.%i", index)

    },
    # Returns the last graph number
    last_graph = function() {
      netplot_name$make(index, for_graph = TRUE)
    }
  )

})()


# GPAR
.grid_par_names <- names(grid::get.gpar())
graphics.off() # To make sure it is closed after the call


#' Validating element types
#' @noRd
np_validate <- (function() {

  # Valid elements
  vertex <- c("core", "frame", "label")
  edge   <- c("arrow", "line")

  list(
    # Class
    is_netplot = function(x) {

      if (!inherits(x, "netplot"))
        stop("This function is only applicable for 'netplot' class objects.",
             call. = FALSE)

      invisible()

    },
    # Elements
    elements = function(elements, type) {

      # Is there anything to revise?
      if (missing(elements) || is.null(elements))
        return(invisible())

      test <- switch (
        type,
        vertex = which(!(elements %in% vertex)),
        edge   = which(!(elements %in% edge)),
        stop("The `type` should be either 'vertex' or 'edge'.", call. = FALSE)
      )

      if (length(test))
        stop("Invalid element type. The following `element`(s) for ",
             type, " are wrongly specified: '",
             paste(elements, collapse="', '"), "'. ", call.=FALSE)

      invisible()

    },
    #Type
    type = function(type, multiple = FALSE) {

      # Missing
      if (missing(type))
        stop("Unspecified (missing) `type`. You must provide either {'vertex', 'edge'}",
             call. = FALSE)

      # More than one?
      if (!multiple && length(type) != 1)
        stop("Only one type, vertex or edge, can be specified.", call. = FALSE)

      test <- which(!(type %in% c("vertex", "edge")))
      if (length(test))
        stop(
          "Invalid `type` ('", paste0(type[test], collapse="', '"),"'. ",
          "`type` should be one of {'vertex', 'edge'}."
          )

      invisible()


    },
    gpar = function(gpar) {

      if (is.list(gpar))
        gpar <- names(gpar)

      # Are the par aesthetics registered?
      test <- which(!(gpar %in% .grid_par_names))
      if (length(test))
        stop("The following parameters are not part of `grid::gpar`: '",
             paste(gpar[test], collapse="', '"), "'.",
             "Supported parameters are: '",
             paste(.grid_par_names, collapse="', '"),"'.",
             call. = FALSE)

      invisible()
    }
  )

})()

# Misc

# Wrappers for different graph methods -----------------------------------------

stop_unsopported_graph <- function(x) {
  stop("Graph of class `", class(x), "` is not supported.", call. = FALSE)
}

get_vertex_attr <- function(x, attr) {

  if (inherits(x, "igraph"))
    igraph::vertex_attr(x, attr)
  else if (inherits(x, "network"))
    network::get.vertex.attribute(x, attr)
  else
    stop_unsopported_graph(x)

}

get_edge_attr <- function(x, attr) {

  if (inherits(x, "igraph"))
    igraph::edge_attr(x, attr)
  else if (inherits(x, "network"))
    network::get.edge.attribute(x, attr)
  else
    stop_unsopported_graph(x)

}

calc_degree <- function(x, mode. = "in") {
  if (inherits(x, "igraph"))
    igraph::degree(x, mode = mode.)
  else if (inherits(x, "network"))
    sna::degree(x, cmode = match.arg(
      mode., c("indegree", "outdegree", "freeman")
    ))
  else
    stop_unsopported_graph(x)
}

#' @keywords internal
color_nodes <- function(...) UseMethod("color_nodes")

#' @keywords internal
color_nodes.formula <- function(formula, ...) {

  # Extract the LHS of the formula
  lhs <- as.character(formula[[2]])

  # Extract the RHS of the formula
  rhs <- as.character(formula[[3]])

  # Check if the lhs exists in the parent environment
  if (!(lhs %in% ls(parent.frame()))) {
    stop("LHS does not exist in the parent environment")
  }

  # Check if the rhs is a valid name
  color_nodes(get(lhs, envir = parent.frame()), rhs, ...)

}

#' @keywords internal
color_nodes.default <- function(
  graph,
  attribute,
  palette = grDevices::palette(),
  na_color = "white",
  ...
  ) {

  # Extracting the attribute from the graph
  value <- get_vertex_attribute(graph, attribute)
  attr_type <- class(value)

  # Identifying NAs
  na_idx <- which(is.na(value))

  # Handle characters, are turned into factors
  if (attr_type == "character") {

    value <- as.factor(value)
    attr_type <- "factor"

  } else if (inherits(value, "numeric")) {

    # Checking if it is numeric, but if it can be converted to int
    test_int <- abs(as.integer(value) - value) < .Machine$double.eps^.5

    if (all(test_int, na.rm = TRUE)) {

      value <- as.integer(value)
      attr_type <- "integer"

    }

  }

  

  # Saving the original
  value_orig <- value

  # Handle factors
  if (attr_type == "factor") {

    # Map levels to colors
    cpal  <- grDevices::colorRampPalette(palette)(nlevels(value))

    # Creating mapping to recover colors
    names(cpal) <- levels(value)

    value <- cpal[as.integer(value)]

  }  else if ("numeric" %in% attr_type) { # Handle numerics

    # Find min and max
    attr_min <- min(value)
    attr_max <- max(value)

    # Create color scale
    value <- grDevices::colorRamp(palette)(
      (value - attr_min)/(attr_max - attr_min)
    )

    cpal <- function(val) {grDevices::rgb(
      grDevices::colorRamp(palette)(val),
      maxColorValue = 255
    )}

    # Color nodes based on attribute value
    value <- grDevices::rgb(value, maxColorValue = 255)

  } else if ("logical" %in% attr_type) { # Handle logicals

    # Creating mapping to recover colors
    cpal <- palette[1:2]
    names(cpal) <- c("FALSE", "TRUE")

    # Color nodes
    value <- cpal[as.integer(value) + 1]
  
  } else if ("integer" %in% attr_type) {
      
      # Find min and max
      attr_min <- min(value, na.rm = TRUE)
      attr_max <- max(value, na.rm = TRUE)
  
      # Create color scale
      cpal <- grDevices::colorRampPalette(palette)(
        length(attr_min:attr_max)
      )
  
      names(cpal) <- as.character(c(attr_min:attr_max))
  
      # Color nodes based on attribute value
      value <- cpal[as.character(value)]

  }

  # Handle other types (characters, dates)
  else {
    stop("Attribute type not supported")
  }

  value[na_idx] <- na_color

  structure(
    value,
    class     = "netplot_color_nodes",
    attr_type = attr_type,
    palette   = palette,
    na_color  = na_color,
    cpal      = cpal,
    value     = value_orig,
    attr_name = attribute
  )

}

#' @noRd 
#' @importFrom stats quantile
#' @keywords internal
color_nodes_legend <- function(object) {

  # Extracting the fill legend
  x <- object$.legend_vertex_fill

  if (!length(x))
    return(invisible(NULL))

  if (!inherits(x, "netplot_color_nodes")) {
    stop("Object is not of class netplot_color_nodes")
  }

  # Continuous attributes get a color bar rather than a set of discrete keys
  if (legend_is_continuous(x))
    return(color_nodes_legend_continuous(object))

  # Discrete attributes (factor/logical/small integer): categorical legend
  values <- attr(x, "cpal")

  print(nplot_legend(
    object,
    labels = names(values),
    pch    = 21,
    gp     = grid::gpar(fill = values)
  ))

}

#' Decide whether a color mapping should use a continuous (color bar) legend
#'
#' Factors and logicals are always discrete. Numeric attributes are continuous.
#' Integer attributes are treated as continuous only when they take many
#' distinct values (otherwise a categorical legend is clearer).
#' @param x A `netplot_color_nodes` object.
#' @noRd
legend_is_continuous <- function(x) {

  attr_type <- attr(x, "attr_type")

  if (attr_type == "numeric")
    return(TRUE)

  if (attr_type == "integer")
    return(length(unique(attr(x, "value"))) > 15L)

  FALSE

}

#' Draw a `netplot` object together with a continuous color-bar legend
#'
#' Called by [print.netplot()] when `vertex.color` was mapped from a continuous
#' attribute. The network is drawn first and the color bar is overlaid in the
#' top-right corner of the device.
#' @param object A `netplot` object with a continuous `.legend_vertex_fill`.
#' @noRd
color_nodes_legend_continuous <- function(object) {

  x       <- object$.legend_vertex_fill
  values  <- attr(x, "value")
  palette <- attr(x, "palette")
  main    <- attr(x, "attr_name")

  rng  <- range(values, na.rm = TRUE)
  cols <- grDevices::colorRampPalette(palette)(100) # low -> high

  # Tick marks within the observed range
  ticks <- pretty(rng, n = 4L)
  ticks <- ticks[ticks >= rng[1] & ticks <= rng[2]]
  if (length(ticks) < 2L)
    ticks <- rng

  # Draw the network first (without recursing into the legend logic)
  print(object, legend = FALSE, newpage = TRUE)

  # Overlay the color bar in the top-right corner of the device
  vp <- grid::viewport(
    x      = grid::unit(1, "npc") - grid::unit(1.5, "lines"),
    y      = grid::unit(0.78, "npc"),
    width  = grid::unit(0.6, "lines"),
    height = grid::unit(0.30, "npc"),
    just   = c("right", "center"),
    yscale = rng,
    name   = "netplot-colorkey"
  )

  grid::pushViewport(vp)
  on.exit(grid::upViewport(), add = TRUE)

  # The gradient (top = high value)
  grid::grid.raster(
    grDevices::as.raster(matrix(rev(cols), ncol = 1L)),
    width       = grid::unit(1, "npc"),
    height      = grid::unit(1, "npc"),
    interpolate = TRUE
  )

  # A thin frame around the bar
  grid::grid.rect(gp = grid::gpar(fill = NA, col = "gray40", lwd = .5))

  # Tick labels to the right of the bar
  grid::grid.text(
    label = format(ticks, trim = TRUE),
    x     = grid::unit(1, "npc") + grid::unit(0.3, "lines"),
    y     = grid::unit(ticks, "native"),
    just  = "left",
    gp    = grid::gpar(fontsize = 8)
  )

  # Title above the bar
  if (length(main) && nzchar(main))
    grid::grid.text(
      label = main,
      x     = grid::unit(0.5, "npc"),
      y     = grid::unit(1, "npc") + grid::unit(0.7, "lines"),
      just  = "bottom",
      gp    = grid::gpar(fontsize = 9, fontface = "bold")
    )

  invisible(object)

}


if (FALSE) {

  # Factor attribute
  g1 <- graph_from_data_frame(d = data.frame(from = c("1", "2", "3"),
                                            to = c("2", "3", "1")),
                              directed = FALSE)

  # Assign unique vertex names
  set_vertex_attr(g1, "name", value = c("v1", "v2", "v3"))

  # Add group attribute to graph as a vertex attribute
  vertex_attr(g1)$group <- c("group1", "group2", "group3")

  # Color nodes by group attribute
  vcolors <- color_nodes(g1, "group")

  # nplot(..., vertex.color = vcolors)
  attr(vcolors, "map")

  color_nodes(g1 ~ group)

  # Here is an example using color_nodes with
  # netplot
  nplot(g1, vertex.color = color_nodes(g1, "group"))
  nplot(
    g1,
    vertex.color = color_nodes(g1, "group"),
    edge.color = "black"
    )

  nplot(
    g1,
    vertex.color = color_nodes(g1, "group"),
    edge.color = "black",
    # The V() function is to access the vertices in
    # igraph. Vertex attributes can then be accessed
    # using the $ operator
    vertex.label = V(g1)$name
    )

  nplot(
    g1,
    vertex.color = color_nodes(g1, "group"),
    edge.color = "black",
    vertex.label = c("A", "B", "C")
    )


  # Numeric attribute
  g2 <- graph_from_data_frame(d = data.frame(from = c(1, 2, 3),
                                            to = c(2, 3, 1)),
                              directed = FALSE)
  V(g2)$value <- c(1, 3, 2)
  color_nodes(g2, "value", "Blues")



  # Logical attribute
  g3 <- graph_from_data_frame(d = data.frame(from = c(1, 2, 3),
                                            to = c(2, 3, 1)),
                              directed = FALSE)
  V(g3)$selected <- c(TRUE, FALSE, TRUE)
  color_nodes(g3, "selected")



  # Invalid palette name
  g4 <- graph_from_data_frame(d = data.frame(from = c(1, 2, 3),
                                            to = c(2, 3, 1)),
                              directed = FALSE)
  V(g4)$group <- factor(c("A", "A", "B"))
  color_nodes(g4, "group", "InvalidPalette")



  # Attribute that does not exist
  color_nodes(g1, "fake_attr")

}

# dots <- radialGradient()
#
# n     <- 100
# n_par <- length(dots)
#
# for (e in list(list(), vector("integer", 2), 1L, "p")) {
#   cat(sprintf(
#     "Mode: %9s; is.atomic: %5s; is.vector: %5s; typeof(): %-9s = %s\n",
#     mode(e),
#     is.atomic(e),
#     is.vector(e),
#     typeof(e),
#     deparse(e)
#   ))
# }


#' Extends the size of the parameters
#'
#' Atomic types (integers, characters, etc.) are extended automatically. In
#' the case of patterns (from the grid package,) if first checks the type. If
#' the ... is a list, then the only valids are size 1 or n.
#'
#' @param x An object of class netplot.
#' @param n Length to recycle the parameters.
#' @param ... Parameters from gpar.
#' @noRd
extend_parameter <- function(x, n, ...) {

  dots <- list(...)
  np_validate$gpar(dots)

  for (i in names(dots)) {

    n_i <- length(dots[[i]])

    # Case 1: Is logical/scalar/int/char ---------------------------------------
    if (is.atomic(dots[[i]])) {

      if (n_i == 1) {

        dots[[i]] <- rep(dots[[i]], times = n)

      } else if (n_i != n) {

        stop(
          sprintf(
            "The value passed to %s: %s, is not valid.",
            i, deparse(dots[[i]])
            )
          )

      }

    }
    # Case 2: Formula ----------------------------------------------------------
    else if (inherits(dots[[i]], "formula")) {

      dots[[i]] <- netplot_edge_formulae(x, dots[[i]])

    # Case 3: A list -----------------------------------------------------------
    } else {

      # Case 3a: A pattern list
      if (inherits(dots[[i]], c("GridRadialGradient", "GridPattern"))) {

        dots[[i]] <- replicate(n, list(dots[[i]]), simplify = FALSE)
        next

      }

      # Case 3a: A list of colors
      if (n_i == n) {
        next
      } else {
        stop("The length of ", i, " is invalid.")
      }

    }

  }

  # Returning
  do.call(Map, c(dots, list(f = function(...) {
    structure(list(...), names = names(list(...)))
  })))

}

#' `nplot` using base graphics
#' @param edge.color.mix Proportion of the mixing.
#' @param edge.color.alpha Either a vector of length 1 or 2, or a matrix of
#' size `ecount(x)*2` with values in `[0,1]`. Alpha (transparency) levels (see
#' details)
#' @inheritParams nplot
#' @export
#' @return `nplot_base` returns a list with the following components:
#'
#' - `vertex.coords` A list of length `N` where each element describes the
#' geomtry of each vertex.
#' - `vertex.color` A vector of colors
#' - `vertex.frame.coords` Similar to `vertex.coords`, but for the frame.
#' - `vertex.frame.color` Similar to `vertex.color`, but for the frame.
#' - `edge.color` Vector of functions used to compute the edge colors.
#' - `edge.coords` Similar to `vertex.coords`, the points that describe each
#' edge.
#' - `edge.arrow.coords` A list of matrices describing the geometry of the
#' tip of the edges.
#' - `edge.width` A numeric vector with edges' widths.
#' - `xlim`, `ylim` Limits of the plot area.
#' @seealso [nplot]
#' @examples
#' # Same example as in nplot
#' library(igraph)
#' library(netplot)
#' set.seed(1)
#' x <- sample_smallworld(1, 200, 5, 0.03)
#'
#' nplot_base(x) # ala netplot (using base)
nplot_base <- function(
  x,
  layout              = igraph::layout_nicely(x),
  vertex.size         = igraph::degree(x, mode="in"),
  bg.col              = "transparent",
  vertex.nsides       = 10,
  vertex.color        = grDevices::hcl.colors(1),
  vertex.size.range   = c(.01, .03, 4),
  vertex.frame.color  = grDevices::adjustcolor(vertex.color, red.f = .75, green.f = .75, blue.f = .75),
  vertex.rot          = 0,
  vertex.frame.prop   = .1,
  edge.width          = NULL,
  edge.width.range    = c(1, 2),
  edge.arrow.size     = NULL,
  edge.color          = NULL,
  edge.color.mix      = .5,
  edge.color.alpha    = c(.1, .5),
  edge.curvature      = pi/3,
  edge.line.lty       = "solid",
  edge.line.breaks    = 5,
  sample.edges        = 1,
  skip.vertex         = FALSE,
  skip.edges          = FALSE,
  skip.arrows         = skip.edges,
  add                 = FALSE,
  zero.margins        = TRUE
) {

  # Checking layout
  if (!inherits(layout, what = c("matrix")))
    stop(
      "-layout- should be of class 'matrix'. It currently is '",
      class(layout),
      "'"
    )

  # Computing colors
  if (!length(vertex.color)) {
    vertex.color <- length(table(igraph::degree(x, mode = "in")))
    vertex.color <- grDevices::hcl.colors(vertex.color)
    vertex.color <- vertex.color[
      as.factor(igraph::degree(x, mode = "in"))
      ]
  }

  # # Creating the window
  if (zero.margins) {
    oldpar <- graphics::par(mai=rep(0, 4))
    on.exit(graphics::par(oldpar))
  }

  if (!add)
    graphics::plot.new()

  # Adjusting layout to fit the device
  layout <- fit_coords_to_dev(layout)

  # Plotting
  xlim <- range(layout[,1])
  ylim <- range(layout[,2])
  graphics::plot.window(xlim, ylim, asp=1, new=FALSE)

  # Adding rectangle
  if (length(bg.col)) {
    usr <- graphics::par("usr")
    rect(
      usr[1], usr[3], usr[2], usr[4],
      col = bg.col,
      border = grDevices::adjustcolor(bg.col, red.f = 1.5, green.f = 1.5, blue.f = 1.5))
  }

  # Rescaling size
  vertex.size <- rescale_size(vertex.size, rel = vertex.size.range)

  # Computing shapes -----------------------------------------------------------
  E <- igraph::as_edgelist(x, names = FALSE)
  M <- nrow(E)

  if (sample.edges < 1) {
    sample.edges <- sample.int(M, floor(M*sample.edges))

    edge_par <- c(
      "edge.width",
      "edge.arrow.size",
      "edge.color.mix",
      "edge.color.alpha",
      "edge.curvature",
      "edge.line.lty",
      "edge.line.breaks"
    )

    for (epar in edge_par) {
      # Force and cache the (possibly lazily-evaluated) argument once, so we
      # don't re-evaluate user-supplied expressions while inspecting/subsetting.
      val <- get(epar)

      if (is.null(val))
        next

      if (is.matrix(val) || is.data.frame(val)) {
        if (nrow(val) == M)
          assign(epar, val[sample.edges, , drop = FALSE])
      } else if (length(val) == M) {
        assign(epar, val[sample.edges])
      }
    }

    E <- E[sample.edges, , drop=FALSE]
  }

  # Weights
  if (!length(edge.width))
    edge.width <- rep(1.0, igraph::ecount(x))

  # Rescaling edges
  if (is.null(edge.width.range))
    edge.width <- rescale_size(edge.width, rel = edge.width.range)
  else
    edge.width <- rescale_size(edge.width/max(edge.width, na.rm=TRUE), rel = edge.width.range)

  if (!length(edge.arrow.size))
    edge.arrow.size <- vertex.size[E[,1]]/1.5
  else if (length(edge.arrow.size) == 1L)
    edge.arrow.size <- rep(edge.arrow.size, nrow(E))

  # Calculating arrow adjustment
  arrow.size.adj <- edge.arrow.size*cos(pi/6)/(
    cos(pi/6) + cos(pi - pi/6 - pi/1.5)
  )/cos(pi/6)

  # Making space for the arrow and edges coordinates
  edge.coords <- vector("list", nrow(E))
  edge.arrow.coords <- vector("list", length(edge.coords))

  if (length(edge.curvature) == 1)
    edge.curvature <- rep(edge.curvature, length(edge.coords))

  if (length(edge.line.breaks) == 1)
    edge.line.breaks <- rep(edge.line.breaks, length(edge.coords))


  for (e in 1:nrow(E)) {

    i <- E[e,1]
    j <- E[e,2]

    # Calculating edges coordinates
    edge.coords[[e]] <- arc(
      layout[i,], layout[j,],
      radii = vertex.size[c(i,j)] + c(0, arrow.size.adj[e]),
      alpha = edge.curvature[e],
      n     = edge.line.breaks[e]
    )

    # Computing arrow
    alpha1 <- attr(edge.coords[[e]], "alpha1")
    edge.arrow.coords[[e]] <- arrow_fancy(
      x = edge.coords[[e]][nrow(edge.coords[[e]]),1:2] +
        arrow.size.adj[e]*c(cos(alpha1), sin(alpha1)),
      alpha = alpha1,
      l     = edge.arrow.size[e]
    )

  }

  # Edges ----------------------------------------------------------------------
  if (!length(edge.color.mix))
    edge.color.mix <- rep(.5, length(edge.coords))
  else if (length(edge.color.mix) == 1)
    edge.color.mix <- rep(edge.color.mix, length(edge.coords))

  if (!length(edge.line.lty))
    edge.line.lty <- rep(1L, length(edge.coords))
  else if (length(edge.line.lty) == 1)
    edge.line.lty <- rep(edge.line.lty, length(edge.coords))
  else if (length(edge.line.lty) != length(edge.coords))
    stop("edge.line.lty must have length 1 or one value per plotted edge.")

  if (!length(edge.color.alpha))
    edge.color.alpha <- matrix(.5, nrow= length(edge.coords), ncol=2)
  else if (length(edge.color.alpha) <= 2)
    edge.color.alpha <- matrix(edge.color.alpha, nrow= length(edge.coords), ncol=2, byrow = TRUE)

  # Nodes ----------------------------------------------------------------------
  if (length(vertex.color) == 1)
    vertex.color <- rep(vertex.color, nrow(layout))

  if (!length(vertex.frame.color))
    vertex.frame.color <- adjustcolor(vertex.color, red.f = .75, blue.f = .75, green.f = .75)

  if (length(vertex.nsides) == 1)
    vertex.nsides <- rep(vertex.nsides, nrow(layout))

  if (length(vertex.rot) == 1)
    vertex.rot <- rep(vertex.rot, nrow(layout))

  if (length(vertex.frame.prop) == 1)
    vertex.frame.prop <- rep(vertex.frame.prop, nrow(layout))

  edge.color <- vector("list", length(edge.coords))

  for (i in seq_along(edge.coords)) {

    if (!length(edge.coords[[i]]))
      next

    # Not plotting self (for now)
    if (E[i,1] == E[i, 2])
      next

    # Computing edge color
    col <- edge_color_mixer(
      i     = E[i, 1],
      j     = E[i, 2],
      vcols = vertex.color,
      p     = edge.color.mix[i],
      alpha = 1
    )

    edge.color[[i]] <- colorRamp2(
      c(
        adjustcolor(col, alpha.f = edge.color.alpha[i,1]),
        adjustcolor(col, alpha.f = edge.color.alpha[i,2])
      ), alpha = TRUE)

    # Drawing lines
    if (!skip.edges) {
      segments_gradient(
        edge.coords[[i]], lwd= edge.width[i],
        col = edge.color[[i]],
        lty = edge.line.lty[i]
      )
    }

    if (!skip.arrows) {
      # Drawing arrows
      graphics::polygon(
        edge.arrow.coords[[i]],
        col    = col,
        border = col,
        lwd    = edge.width[i]
      )
    }

  }

  vertex.frame.prop <- 1 - vertex.frame.prop

  vertex.coords <- vector("list", nrow(layout))
  vertex.frame.coords <- vector("list", nrow(layout))

  if (!skip.vertex)
    for (i in 1:nrow(layout)) {
      # Computing coordinates
      vertex.coords[[i]] <- npolygon(
        layout[i,1], layout[i,2],
        n = vertex.nsides[i],
        r = vertex.size[i]*vertex.frame.prop[i],
        vertex.rot[i]
      )
      vertex.frame.coords[[i]] <- piechart(
        1,
        origin = layout[i,],
        edges  = vertex.nsides[i],
        radius = vertex.size[i],
        doughnut = vertex.size[i]*vertex.frame.prop[i],
        init.angle = vertex.rot[i]*180/pi,
        last.angle = 360 + vertex.rot[i]*180/pi,
        rescale = FALSE,
        add     = TRUE,
        skip.plot.slices = TRUE
      )$slices[[1]]


      # Circle
      graphics::polygon(
        vertex.coords[[i]],
        col    = vertex.color[i],
        border = vertex.color[i],
        lwd=1
      )

      # Border
      graphics::polygon(
        vertex.frame.coords[[i]],
        col    = vertex.frame.color[i],
        border = vertex.frame.color[i],
        lwd=1
      )


    }

  invisible({
    list(
      vertex.coords       = vertex.coords,
      vertex.color        = vertex.color,
      vertex.frame.coords = vertex.frame.coords,
      vertex.frame.color  = vertex.frame.color,
      edge.color         = edge.color,
      edge.coords        = edge.coords,
      edge.arrow.coords  = edge.arrow.coords,
      edge.width         = edge.width,
      xlim                = xlim,
      ylim                = ylim
    )
  })


}

#' Rescale the size of a node to make it relative to the aspectt ratio of the device
#' @param size Numeric vector. Size of the node (radious).
#' @param rel Numeric vector of length 3. Relative size for the minimum and maximum
#' of the plot, and curvature of the scale. The third number is used as `size^rel[3]`.
#' If `NULL`, scaling is suppressed.
#'
#' @details
#' This function is to be called after [plot.new], as it takes the parameter `usr`
#' from the
#' @noRd
rescale_size <- function(size, rel=c(.01, .05, 1)) {

  if (is.null(rel))
    return(size)

  # Checking the rel size
  if (length(rel) == 2)
    rel <- c(rel, 1)
  else if (length(rel) > 3) {
    warning("`rel` has more than 3 elements. Only the first 3 will be used.")
  } else if (length(rel) < 2) {
    stop("`rel` must be at least of length 2 and at most of length 3.")
  }

  # Creating curvature
  size <- size^rel[3]

  # Rescaling to be between range[1], range[2]
  sran <- range(size, na.rm=TRUE)

  if ((sran[2] - sran[1]) > 1e-5)
    size <- (size - sran[1])/(sran[2] - sran[1]) # 0-1
  else
    size <- size/sran[1]

  size * (rel[2] - rel[1]) + rel[1]

}

#' Adjust coordinates to fit aspectt ratio of the device
#' @param coords Two column numeric matrix. Vertices coordinates.
#' @details
#' It first adjusts `coords` to range between `-1,1`, and then, using
#' `graphics::par("pin")`, it rescales the second column of it (`y`) to adjust
#' for the device's aspect ratio.
#' @param adj Numeric vector of length 2.
#' @noRd
# @export
fit_coords_to_dev <- function(coords, adj = dev_size()) {

  # Making it -1 to 1
  yran <- range(coords[,2], na.rm = TRUE)
  xran <- range(coords[,1], na.rm = TRUE)

  coords[,1] <- (coords[,1] - xran[1])/(xran[2] - xran[1])*2 - 1
  coords[,2] <- (coords[,2] - yran[1])/(yran[2] - yran[1])*2 - 1

  # Adjusting aspectt ratio according to the ploting area
  coords[,2] <- coords[,2]*adj[2]/adj[1]

  # Returning new coordinates
  coords

}

#' Rotation of polygon
#' @param mat Two-column numeric matrix. Coordinates of the polygon.
#' @param origin Numeric vector of length two. Origin.
#' @param alpha Numeric scalar. Rotation degree in radians.
#' @noRd
# @export
rotate <- function(mat, origin, alpha) {
  R <- matrix(
    c(cos(alpha), -sin(alpha), sin(alpha), cos(alpha)),
    nrow = 2, byrow = TRUE)

  origin <- matrix(c(origin[1], origin[2]), ncol=2, nrow = nrow(mat), byrow = TRUE)
  t(R %*% t(mat - origin)) + origin
}


#' Arc between two nodes
#'
#' @param p0,p1 Numeric vector of length 2. Center coordinates
#' @param alpha Numeric scalar. Arc angle in radians.
#' @param n Integer scalar. Number of segments to approximate the arc.
#' @param radii Numeric vector of length 2. Radious
#' @noRd
# @export
#'
arc <- function(
  p0,
  p1,
  alpha = pi/3,
  n     = 10L,
  radii = c(0, 0)
) {

  # If no curve, nothing to do (old fashioned straight line)
  if (alpha == 0 | n == 1) {
    alpha <- 1e-5
  }

  elevation <- atan2(p1[2]-p0[2], p1[1] - p0[1])

  # Constants
  d <- sqrt(sum((p0 - p1)^2))

  # If overlapping, then fix the radius to be the average
  if ((d - sum(radii)) < 0) {
    r <- mean(radii)
    alpha <- 2*pi - asin(d/2/r)*2
  } else {
    r <- d/2/(sin(alpha/2))
  }

  # Angles
  alpha0 <- asin(radii[1]/2/r)*2
  alpha1 <- asin(radii[2]/2/r)*2

  # Angle range
  alpha_i <- seq(
    pi/2 + (alpha/2 - alpha0) ,
    pi/2 - (alpha/2 - alpha1),
    length.out = n + 1
  )

  # Middle point
  M <- c(
    p0[1] + d/2,
    p0[2] - cos(alpha/2)*r
  )

  ans <- cbind(
    M[1] + cos(alpha_i)*r,
    M[2] + sin(alpha_i)*r
  )


  # Rotation and return
  ans <- rotate(ans, p0, elevation)

  # Separating the segments
  ans <- cbind(
    as.vector(t(cbind(ans[-(n + 1),1], ans[-1,1]))),
    as.vector(t(cbind(ans[-(n + 1),2], ans[-1,2])))
  )

  structure(
    ans,
    alpha0 = atan2(ans[1,2] - p0[2], ans[1,1] - p0[1]),
    alpha1 = atan2(p1[2] - ans[n*2,2], p1[1] - ans[n*2,1]),
    midpoint = ans[ceiling(n/2),]
  )

}


#' Arrow polygon.
#'
#' @param x Numeric vector of length 2. Coordinates of the tip
#' @param alpha,l,a,b Numeric scalars
#' @noRd
# @export
arrow_fancy <- function(x, alpha = 0, l=.25, a=pi/6, b = pi/1.5) {


  p_left  <- x + c(-cos(a), sin(a))*l

  base <- l*sin(a)
  base2 <- base * cos(pi - b)/sin(pi - b)
  p_mid   <- p_left + c(base2, -base)

  p_right <- x - c(cos(a), sin(a))*l

  ans <- rbind(x, p_left, p_mid, p_right)

  # Rotation

  rotate(ans, x, alpha = alpha)


}

#' Takes a graph attribute and maps it to a shape
#' @param x Is (any) vector to be mapped.
#' @return 
#' A vector of shapes (encoded as integers) from 2 to 8.
#' @noRd
map_attribute_to_shape <- function(x) {

  # Get the unique values of x, if it is more than 9
  # then we will throw an error.
  nunique <- length(unique(x))
  if (nunique > length(sides_lookup)) {
    stop(
      "The number of unique values of the attribute is greater than the number of shapes available. ",
      "Please, use a different attribute or a different shape."
    )
  }

  # If the variable already contains the values
  if (all(x %in% names(sides_lookup)))
    return(x)

  # We can now turn this x into a factor, and then
  # numbers to be used as indices
  names(sides_lookup)[as.numeric(factor(x))]

}

# Placeholder with simple test
expect_equal(1 + 1, 2)

expect_equal(
  netplot:::rescale_size(c(.1, .2, .3), rel = NULL),
  c(.1, .2, .3),
  info = "NULL range should suppress size scaling"
)

# Test that edge.width values are reflected in the line-grob lwd gpar
if (requireNamespace("igraph", quietly = TRUE)) {
  set.seed(1)
  x <- igraph::make_ring(4, directed = FALSE)
  igraph::E(x)$weight <- c(1, 2, 3, 4)
  l <- igraph::layout_in_circle(x)

  g <- nplot(x, layout = l, skip.arrows = TRUE,
             edge.width = igraph::E(x)$weight,
             edge.width.range = c(1, 4))

  lwds <- get_edge_gpar(g, element = "line", "lwd")$lwd

  # All lwd values should be within the specified range
  expect_true(all(lwds >= 1 & lwds <= 4),
              info = "edge.width values should be mapped to edge.width.range")

  # Edges with larger weights should have larger (or equal) lwd
  expect_true(all(diff(lwds) >= 0),
              info = "lwd values should be non-decreasing with increasing weight")

  g_raw <- nplot(x, layout = l, skip.arrows = TRUE,
                 vertex.size = rep(.05, 4),
                 vertex.size.range = NULL,
                 edge.width = igraph::E(x)$weight,
                 edge.width.range = NULL)

  raw_lwds <- get_edge_gpar(g_raw, element = "line", "lwd")$lwd
  expect_equal(
    raw_lwds,
    igraph::E(x)$weight,
    info = "edge.width.range = NULL should use edge.width values as is"
  )

  edge_ltys <- c("solid", "dashed", "dotted", "dotdash")
  g_lty <- nplot(x, layout = l, skip.arrows = TRUE,
                 edge.line.lty = edge_ltys)
  expect_equal(
    get_edge_gpar(g_lty, element = "line", "lty")$lty,
    edge_ltys,
    info = "edge.line.lty should set one line type per grid edge"
  )

  g_lty_scalar <- nplot(x, layout = l, skip.arrows = TRUE,
                        edge.line.lty = "dashed")
  expect_equal(
    get_edge_gpar(g_lty_scalar, element = "line", "lty")$lty,
    rep("dashed", igraph::ecount(x)),
    info = "scalar edge.line.lty should be recycled across grid edges"
  )

  expect_error(
    nplot(x, layout = l, skip.arrows = TRUE,
          edge.line.lty = edge_ltys[seq_len(2)]),
    "edge.line.lty",
    info = "short non-scalar edge.line.lty vectors should be rejected"
  )

  frame <- g_raw$children$graph$children$vertex.1$children$frame
  frame_xy <- cbind(as.numeric(frame$x), as.numeric(frame$y))
  vertex_radius <- max(sqrt(rowSums(
    (frame_xy - matrix(g_raw$.layout[1, ], nrow(frame_xy), 2, byrow = TRUE))^2
  )))

  expect_equal(
    vertex_radius,
    .05,
    tolerance = 1e-8,
    info = "vertex.size.range = NULL should use vertex.size values as is"
  )

  g_rot <- nplot(x, layout = l, skip.edges = TRUE, skip.arrows = TRUE,
                 vertex.size = rep(.05, 4),
                 vertex.size.range = NULL,
                 vertex.frame.prop = 0,
                 vertex.nsides = rep(3, 4),
                 vertex.rot = pi/4)
  core <- g_rot$children$graph$children$vertex.1$children$core
  core_xy <- unname(cbind(as.numeric(core$x), as.numeric(core$y)))
  expected_core_xy <- unname(npolygon(
    g_rot$.layout[1, 1], g_rot$.layout[1, 2],
    n = 3, r = .05, d = pi/4
  ))

  expect_equal(
    core_xy,
    expected_core_xy,
    tolerance = 1e-8,
    info = "vertex.rot should rotate grid vertex polygons using radians"
  )

  g_shape_rot <- nplot(x, layout = l, skip.edges = TRUE, skip.arrows = TRUE,
                       vertex.size = rep(.05, 4),
                       vertex.size.range = NULL,
                       vertex.frame.prop = 0,
                       vertex.nsides = "triangle",
                       vertex.rot = pi/4)
  shape_core <- g_shape_rot$children$graph$children$vertex.1$children$core
  shape_core_xy <- unname(cbind(
    as.numeric(shape_core$x),
    as.numeric(shape_core$y)
  ))
  expected_shape_core_xy <- unname(npolygon(
    g_shape_rot$.layout[1, 1], g_shape_rot$.layout[1, 2],
    n = 3, r = .05, d = pi/4
  ))

  expect_equal(
    shape_core_xy,
    expected_shape_core_xy,
    tolerance = 1e-8,
    info = "vertex.rot should add to named-shape vertex rotations"
  )

  g_square <- nplot(x, layout = l, skip.edges = TRUE, skip.arrows = TRUE,
                    vertex.size = rep(.05, 4),
                    vertex.size.range = NULL,
                    vertex.frame.prop = 0,
                    vertex.nsides = "square")
  square_core <- g_square$children$graph$children$vertex.1$children$core
  square_core_xy <- unname(cbind(
    as.numeric(square_core$x),
    as.numeric(square_core$y)
  ))
  expected_square_core_xy <- unname(npolygon(
    g_square$.layout[1, 1], g_square$.layout[1, 2],
    n = 4, r = .05, d = pi/4
  ))

  expect_equal(
    square_core_xy,
    expected_square_core_xy,
    tolerance = 1e-8,
    info = "the named square shape should use a square orientation"
  )

  g_label <- nplot(x, layout = l, skip.arrows = TRUE,
                   vertex.label = letters[seq_len(igraph::vcount(x))],
                   vertex.label.range = NULL)

  label_fontsizes <- get_vertex_gpar(g_label, element = "label", "fontsize")$fontsize
  expect_true(
    all(label_fontsizes >= 5 & label_fontsizes <= 15),
    info = "vertex.label.range = NULL should fall back to visible default font sizes"
  )

  base_raw <- nplot_base(x, layout = l, skip.arrows = TRUE,
                         vertex.size = rep(.05, 4),
                         vertex.size.range = NULL,
                         edge.width = igraph::E(x)$weight,
                         edge.width.range = NULL)

  expect_equal(
    base_raw$edge.width,
    igraph::E(x)$weight,
    info = "nplot_base should also suppress edge-width scaling for NULL range"
  )

  x_sample <- igraph::make_ring(5, directed = FALSE)
  l_sample <- igraph::layout_in_circle(x_sample)
  sampled_ltys <- c("solid", "not-a-line-type", "dotted", "dotdash", "longdash")

  set.seed(1)
  g_sample_lty <- nplot(x_sample, layout = l_sample, skip.arrows = TRUE,
                        sample.edges = .4, edge.line.lty = sampled_ltys)
  expect_equal(
    get_edge_gpar(g_sample_lty, element = "line", "lty")$lty,
    sampled_ltys[c(1, 4)],
    info = "nplot should keep edge.line.lty aligned when sample.edges drops edges"
  )

  # Attribute formulas -------------------------------------------------------
  x_fm <- igraph::make_ring(5, directed = FALSE)
  igraph::E(x_fm)$w   <- c(1, 2, 3, 4, 5)
  igraph::V(x_fm)$deg <- igraph::degree(x_fm)
  l_fm <- igraph::layout_in_circle(x_fm)

  grDevices::pdf(tempfile(fileext = ".pdf"))

  # Bare attribute name and an equivalent explicit vector should match
  g_fm_name <- nplot(x_fm, layout = l_fm, skip.arrows = TRUE,
                     edge.width = ~ w, edge.width.range = c(1, 5))
  g_fm_vec  <- nplot(x_fm, layout = l_fm, skip.arrows = TRUE,
                     edge.width = igraph::E(x_fm)$w, edge.width.range = c(1, 5))
  expect_equal(
    get_edge_gpar(g_fm_name, element = "line", "lwd")$lwd,
    get_edge_gpar(g_fm_vec, element = "line", "lwd")$lwd,
    info = "edge.width formula ~ w should match the explicit weight vector"
  )

  # Expression on the RHS must be evaluated (regression: used to error with
  # 'the condition has length > 1')
  g_fm_expr <- nplot(x_fm, layout = l_fm, skip.arrows = TRUE,
                     edge.width = ~ w * 2, edge.width.range = c(1, 5))
  expect_equal(
    get_edge_gpar(g_fm_expr, element = "line", "lwd")$lwd,
    get_edge_gpar(g_fm_name, element = "line", "lwd")$lwd,
    info = "edge.width = ~ w * 2 should be evaluated (monotone rescaling unchanged)"
  )

  # Formulas also work for vertex.size and vertex.nsides
  g_fm_vertex <- tryCatch(
    nplot(x_fm, layout = l_fm, skip.arrows = TRUE,
          vertex.size = ~ log(deg + 1), vertex.nsides = ~ deg),
    error = function(e) e
  )
  expect_false(
    inherits(g_fm_vertex, "error"),
    info = "vertex.size/vertex.nsides expression formulas should evaluate"
  )

  # A helpful error when the referenced attribute does not exist
  expect_error(
    nplot(x_fm, layout = l_fm, skip.arrows = TRUE, edge.width = ~ nope),
    "does not|exist|evaluate",
    info = "missing attribute in an edge.width formula should raise a clear error"
  )

  grDevices::dev.off()

  grDevices::pdf(tempfile(fileext = ".pdf"))
  set.seed(1)
  base_sample_lty <- tryCatch(
    nplot_base(x_sample, layout = l_sample, skip.arrows = TRUE,
               sample.edges = .4, edge.line.lty = sampled_ltys),
    error = function(e) e
  )
  grDevices::dev.off()

  expect_false(
    inherits(base_sample_lty, "error"),
    info = "nplot_base should subset edge.line.lty when sample.edges drops edges"
  )

  grDevices::pdf(tempfile(fileext = ".pdf"))
  expect_error(
    nplot_base(x, layout = l, skip.arrows = TRUE,
               edge.line.lty = edge_ltys[seq_len(2)]),
    "edge.line.lty",
    info = "nplot_base should reject short non-scalar edge.line.lty vectors"
  )
  grDevices::dev.off()

  base_rot <- nplot_base(x, layout = l, skip.edges = TRUE, skip.arrows = TRUE,
                         vertex.size = rep(.05, 4),
                         vertex.size.range = NULL,
                         vertex.frame.prop = .2,
                         vertex.nsides = rep(3, 4),
                         vertex.rot = pi/4)
  layout_fit <- netplot:::fit_coords_to_dev(l)
  frame_outer_xy <- unname(as.matrix(
    base_rot$vertex.frame.coords[[1]][seq_len(3), ]
  ))
  expected_frame_outer_xy <- unname(npolygon(
    layout_fit[1, 1], layout_fit[1, 2],
    n = 3, r = .05, d = pi/4
  ))

  expect_equal(
    frame_outer_xy,
    expected_frame_outer_xy,
    tolerance = 1e-8,
    info = "nplot_base should rotate vertex frames with vertex.rot"
  )
}

if (requireNamespace("network", quietly = TRUE)) {
  x_network <- network::network(
    matrix(c(
      0, 1, 0, 1,
      1, 0, 1, 0,
      0, 1, 0, 1,
      1, 0, 1, 0
    ), nrow = 4, byrow = TRUE),
    directed = FALSE,
    matrix.type = "adjacency"
  )

  l_network <- cbind(
    cos(seq(0, 2*pi, length.out = 5)[-5]),
    sin(seq(0, 2*pi, length.out = 5)[-5])
  )

  g_network <- nplot(x_network, layout = l_network, skip.arrows = TRUE,
                     edge.width.range = NULL)

  network_lwds <- get_edge_gpar(g_network, element = "line", "lwd")$lwd
  expect_equal(
    network_lwds,
    rep(1, g_network$.M),
    info = "nplot.network should use width 1 when no edge weight is present"
  )
}

#' Computes edges
#' @noRd
grob_edge <- function(netenv, e) {

  # Obtaining positions of ego and alter
  i       <- netenv$edgelist[e, 1]
  j       <- netenv$edgelist[e, 2]
  nbreaks <- netenv$edge.line.breaks[e]

  if (nbreaks >= 0L) {

    # Computing coordinates
    coords <- arc(
      p0    = netenv$layout[i,],
      p1    = netenv$layout[j,],
      radii = netenv$vertex.size[c(i,j)] + c(0, netenv$arrow.size.adj[e]),
      alpha = netenv$edge.curvature[e],
      n     = nbreaks
    )

    # Generating grob
    if (!netenv$skip.edges) {

      ans <- grid::polylineGrob(
        x          = coords[,1],
        y          = coords[,2],
        id.lengths = rep(2, nbreaks),
        default.units = "native",
        gp         = do.call(
          grid::gpar,
          list(
            lwd     = netenv$edge.width[e],
            lineend = "butt",
            lty     = netenv$edge.line.lty[e]
            )
        ),
        name = "line"
      )

    } else
      ans <- NULL

    # Arrow
    # Computing arrow
    if (!netenv$skip.arrows) {

      alpha1 <- attr(coords, "alpha1")
      arrow  <- arrow_fancy(
        x     = coords[nbreaks*2, 1:2] +
          netenv$arrow.size.adj[e]*c(cos(alpha1), sin(alpha1)),
        alpha = alpha1,
        l     = netenv$edge.arrow.size[e]
      )

      ans <- grid::grobTree(
        ans,
        grid::polygonGrob(
          arrow[,1],
          arrow[,2],
          default.units = "native",
          name = "arrow",
          gp   = grid::gpar(
            lwd  = netenv$edge.width[e]
          )
        ),
        name = netplot_name$make(c(i, j))
      )

    } else
      ans <- grid::grobTree(ans, name = netplot_name$make(c(i, j)))



  } else {

    stop("Line breaks should be positive.", call. = FALSE)

    # The lines that follow should be implemented in the future!
    # Right now these are not working

    # Generating grob
    ans <- grid::curveGrob(
      x1          = netenv$layout[i,1],
      y1          = netenv$layout[i,2],
      x2          = netenv$layout[j,1],
      y2          = netenv$layout[j,2],
      ncp         = 6L,
      curvature   = .5,
      default.units = "native",
      gp         = do.call(
        grid::gpar,
        list(
          lwd     = netenv$edge.width[e],
          lineend = "butt",
          lty     = netenv$edge.line.lty[e]
        )
      ), arrow = grid::arrow(),
      name = "line"
    )

    ans <- grid::grobTree(
      ans,
      name = netplot_name$make(c(i, j))
    )

  }

  ans

}

#' Create a vector of colors for vertices and edges
#'
#' Using `vertex`/`edge` attributes, these functions return vectors of colors
#' that can be used either during the creation of the [nplot] object, or
#' afterwards when changing `gpar` (graphical parameter) values with [set_gpar].
#'
#' @param x A graph of class `network` or `igraph`.
#' @param dat A vector of data to generate the color from.
#' @param vattr,eattr Character. Names of either vertex or edge variables to be
#' used for generating the colors.
#' @param categorical Logical. When `TRUE` sets the colors as categories.
#' @param color_map A function to generate a palette.
#' @param ... Further arguments passed to `make_colors`.
#' @examples
#'
#' data(UKfaculty, package="igraphdata")
#' col <- make_vertex_colors(UKfaculty, "Group")
#'
#' if (require(magrittr)) {
#'
#'   nplot(UKfaculty) %>%
#'     set_vertex_gpar("core", fill = col, col=col) %>%
#'     set_vertex_gpar("frame", fill = col, col=col, alpha=.7) %>%
#'     set_edge_gpar(col="gray50", fill="gray50", alpha=.5)
#' }
#' @export
#' @return
#' A vector of colors with the attribute `color_map`. The color map used
#' to generate the colors.
make_colors <- function(
  dat,
  categorical = FALSE,
  color_map   = grDevices::hcl.colors
  ) {

  # What type of attribute?
  if (!categorical) {

    # Is is numeric?
    if (!is.numeric(dat))
      stop("The selected variable is not numeric.", call. = FALSE)

    # Moving to 0-1 scale
    dat     <- (dat - min(dat))/(diff(range(dat, na.rm = TRUE)) + 1e-15)
    color_map <- grDevices::colorRamp(color_map(20))
    color     <- grDevices::rgb(color_map(dat), maxColorValue = 255)

  } else {

    # Turning the data into factor
    dat <- as.factor(dat)

    # Generating as many colors as needed
    color_map <- structure(color_map(nlevels(dat)), names= levels(dat))
    color     <- color_map[as.integer(dat)]

  }

  structure(
    .Data = color,
    color_map = color_map
  )
}

#' @rdname make_colors
#' @export
make_edges_colors <- function(
  x,
  eattr,
  ...
  ) {


  make_colors(dat = get_edge_attr(x, eattr), ...)

}

#' @rdname make_colors
#' @details
#'
#' If no attribute is provided, then by defaul the colors are set according to
#' indegree.
#'
#' `x` can be either a graph of class `igraph` or `network`.
#'
#' @export
make_vertex_colors <- function(
  x,
  vattr,
  ...
) {

  if (!missing(vattr)) {
    vattr <- get_vertex_attr(x, vattr)
  } else
    vattr <- calc_degree(x, mode. = "in")

  make_colors(dat = vattr, ...)

}

#' Generate functions to create colors for edges
#' @noRd
new_edge_coloring <- function(
  vertex_color_i = NULL,
  vertex_alpha_i = NULL,
  vertex_color_j = NULL,
  vertex_alpha_j = NULL,
  edge_color = NULL,
  edge_alpha = NULL,
  mix = NULL
  ) {

  list(
    vertex = function(e, i, j, n) {

      # Adjusting alpha levels
      col_i <- vertex_color_i[i]
      col_j <- vertex_color_j[j]

      # Getting alpha levels
      alpha_i <- vertex_alpha_i[i]
      alpha_j <- vertex_alpha_j[j]

      # [OPTIONAL] ---------------------------------------------------------------
      # Linear combination between i and j
      col_i <- colorRamp2(x=c(col_i, col_j))(mix[e])
      col_i <- grDevices::rgb(col_i, alpha=col_i[,4], maxColorValue = 255)
      col_j <- col_i
      # --------------------------------------------------------------------------

      # Applying alpha levels and getting mix
      col <- colorRamp2(
        x = c(
          alphacolor(col = col_i, alpha.f = alpha_i),
          alphacolor(col = col_j, alpha.f = alpha_j)
        )
      )(seq(0, 1, length.out = n))

      # Returning
      grDevices::rgb(col, alpha = col[,4], maxColorValue = 255)

    },
    edge = function(e, i, j, n) {

      # Getting alpha levels
      alpha_i <- vertex_alpha_i[i]
      alpha_j <- vertex_alpha_j[j]

      # Edge level params
      col_ij   <- alphacolor(col = edge_color[e], alpha.f = edge_alpha[e])

      # Applying alpha levels and getting mix
      col <- colorRamp2(x=
        c(
          alphacolor(col = col_ij, alpha.f = alpha_i),
          alphacolor(col = col_ij, alpha.f = alpha_j)
        )
      )(seq(0, 1, length.out = n))

      # Returning
      grDevices::rgb(col, alpha = col[,4], maxColorValue = 255)

    }
  )



}


#' @noRd
#' @importFrom stats terms
netplot_edge_formulae <- function(x, fm) {

  # Basic validation
  np_validate$is_netplot(x)

  if (!inherits(fm, "formula"))
    stop("Not a formula", call. = FALSE)

  tm  <- stats::terms(fm)
  mat <- attr(tm, "factors")
  nam <- unique(gsub("\\(.+", "", rownames(mat)))

  # 1. Checking no repeated
  if (length(nam) > nrow(mat))
    stop("One or more terms are repated.", call. = FALSE)

  # 2. edge only goes alone
  if (("edge" %in% nam) & length(nam) > 1)
    stop("`edge` must be used alone.", call. = FALSE)

  # 3. Only one of the
  if (any(!(nam %in% c("edge", "alter", "ego"))))
    stop("Invalid term.", call. = FALSE)

  # 4. Checking if it is linear or an interaction
  linear <- ifelse(ncol(mat) == 1, TRUE, FALSE)

  # Applying formulas
  # Mofiying the likelihood function and the parameters for the mcmc
  val <- attr(tm, "variables")

  # Space where we will save the parameters
  par <- new.env()

  for (i in 2:length(val))
    if (!is.call(val[[i]])) {
      eval(
        call(as.character(val[[i]]), x = x, env = par)
      )
    } else {
      val[[i]]$x   <- bquote(x)
      val[[i]]$env <- bquote(par)
      eval(val[[i]])
    }

  # Creating the coloring
  col <- do.call(new_edge_coloring, as.list(par))

  lapply(seq_len(x$.M), function(e) {

    nam <- netplot_name$make(x$.edgelist[e,])

    col$vertex(
      e = e,
      i = x$.edgelist[e, 1],
      j = x$.edgelist[e, 2],
      n = length(x$children$graph$children[[nam]]$children$line$id.lengths)
    )

  })
}

#' Edge-color formulas in `netplot`
#'
#' Edge colors in both [nplot()] and [set_edge_gpar()] can be specified using
#' a formula based on `ego()` and `alter()` (source and target). This way the
#' user can set various types of combination varying the mixing of the colors,
#' the alpha levels, and the actual mixing colors to create edge colors.
#'
#' @details
#' The formula is one-sided and combines the special terms `ego()` (the source
#' vertex) and `alter()` (the target vertex) with `+`. Each term draws its color
#' from the corresponding endpoint's `vertex.color` unless a `col` is supplied,
#' and accepts:
#'
#' - `col`: the base color to use for that endpoint.
#' - `alpha`: transparency, from 0 (fully transparent) to 1 (opaque).
#' - `mix`: how much weight that endpoint's color receives when the two are
#'   blended along the edge.
#'
#' For instance, `~ ego(alpha = .1, col = "gray") + alter` (the default) fades
#' each edge from a faint gray at the source to the target's color, producing the
#' characteristic netplot look.
#'
#' This grammar is specific to *edge colors*. To map other aesthetics (vertex
#' color, shape, size, or edge width) from a graph attribute, pass a formula
#' naming the attribute directly to [nplot()] (e.g. `vertex.color = ~ group`);
#' see the "Mapping attributes with formulas" section of [nplot()].
#'
#' @seealso [nplot()] for the attribute-mapping formulas, and [set_edge_gpar()].
#'
#' @param col Any valid color. Can be a single color or a vector.
#' @param alpha Number. Alpha levels
#' @param mix Number. For mixing colors between `ego` and `alter`
#' @param env,type,postfix For internal use only.
#' @param ... Passed to `color_formula`.
#' @param x An object of class [netplot].
#' @examples
#' if (require(gridExtra) & require(magrittr)) {
#'   library(igraph)
#'   net <- make_ring(4)
#'
#'   set.seed(1)
#'   np <- nplot(net, vertex.color = grDevices::hcl.colors(4), vertex.size.range=c(.1, .1))
#'   np %<>% set_edge_gpar(lwd = 4)
#'
#'   grid.arrange(
#'     np,
#'     np %>% set_edge_gpar(col =~ego + alter),
#'     np %>% set_edge_gpar(col =~ego(alpha=0) + alter),
#'     np %>% set_edge_gpar(col =~ego + alter(alpha=0)),
#'     np %>% set_edge_gpar(col =~ego(mix=0) + alter(mix=1)),
#'     np %>% set_edge_gpar(col =~ego(mix=1) + alter(mix=0))
#'   )
#' }
#' @name netplot-formulae
#' @export
#' @return Nothing. These functions are called internally when using
#' formulas. `color_formula` modifies the environment `env`.
color_formula <- function(x, col, alpha, env, type, mix = 1, postfix = NULL) {

  n <- switch(type, vertex = x$.N, edge = x$.M)

  # Checking color
  col <- if (missing(col))
    get_vertex_gpar(x, "core", "fill")$fill
  else if (length(col) == 1)
    rep(col, x$.N)
  else if (length(col) != n)
    stop("`col` has the wrong length (", length(col), "). When passing a ",
         "vector it should be of length ", n, ".", call. = FALSE)

  # If all are NA, then it is replaced
  if (all(is.na(col)))
    col[] <- grDevices::hcl.colors(1)

  # Checking alpha
  alpha <- if (missing(alpha))
    rep(.8, n)
  else if (length(alpha) == 1)
    rep(alpha, n)
  else if (length(alpha) != n)
    stop("`alpha` has the wrong length (", length(alpha), "). When passing a ",
         "vector it should be of length ", n, ".", call. = FALSE)

  # Checking mixing levels
  mix <- if (missing(mix))
    rep(1, x$.M)
  else if (length(mix) == 1)
    rep(mix, x$.M)
  else if (length(mix) != x$.M)
    stop("`mix` has the wrong length (", length(mix), "). When passing a ",
         "vector it should be of length ", x$.M, ".", call. = FALSE)

  # Assigning values
  if (type == "vertex") {
    env[[sprintf("vertex_color_%s", postfix)]] <- col
    env[[sprintf("vertex_alpha_%s", postfix)]] <- alpha
  } else {
    env[["edge_color"]] <- col
    env[["edge_alpha"]] <- alpha
  }

  if (!length(env$mix))
    env$mix <- mix
  else {
    env$mix <- 1 - env$mix/(env$mix + mix)
  }

  invisible()
}

#' @export
#' @rdname netplot-formulae
ego   <- function(...) {

  dots        <- list(...)
  dots$type   <- "vertex"
  dots$postfix <- "i"

  do.call(color_formula, dots)
}

#' @export
#' @rdname netplot-formulae
alter <- function(...) {

  dots        <- list(...)
  dots$type   <- "vertex"
  dots$postfix <- "j"

  do.call(color_formula, dots)

}

edge <- function(x, col, alpha = .8) {

  # obtain default
  col <- if (missing(col))
    get_edge_gpar(x, "line", "col")$col
  else if (length(col) == 1)
    rep(col, x$.M)
  else if (length(col) != x$.M)
    stop("`col` has the wrong length (", length(col), "). When passing a ",
         "vector it should be of length ", x$.M, ".", call. = FALSE)

  # Checking alpha
  alpha <- if (missing(alpha))
    rep(.8, x$.M)
  else if (length(alpha) == 1)
    rep(alpha, x$.M)
  else if (length(alpha) != x$.M)
    stop("`alpha` has the wrong length (", length(alpha), "). When passing a ",
         "vector it should be of length ", x$.M, ".", call. = FALSE)


  list(
    col   = col,
    alpha = alpha
  )
}

# netplot_edge_formulae(~ego:alter)

#' Creates a nested list of parameters to be passed to [set_gpar]
#' @noRd
parse_gpar <- function(...) {

  dots <- list(...)
  ans <- new.env(hash = FALSE)

  for (p in names(dots)) {

    # What is hidding there
    par <- strsplit(p, "\\.")[[1]]

    # Setting it up

    # If only length 2, then it sets global attributes for the component
    if (length(par) == 2) {

      # Is it a parameter?
      if (!(par[[2]] %in% .grid_par_names))
        next

      nam <- par[1]
      ans[[nam]] <- c(
        ans[[nam]],
        structure(
          list(dots[[p]], par[1]),
          names = c(par[2], "type")
          )
        )

    } else if (length(par) == 3) {

      # Is it a parameter?
      if (!(par[[3]] %in% .grid_par_names))
        next

      nam <- paste(par[1:2], collapse="_")


      ans[[nam]] <- c(
        ans[[nam]],
        structure(dots[[p]], names = par[3]),
        if (!length(ans[[nam]]))
          structure(
            list(par[1], par[2]),
            names = c("type", "element")
            )
        else
          NULL
      )

    }


  }

  as.list(ans)

}

#' Set/retrieve graphical parameters of a `netplot` object
#'
#' @param x An object of class `netplot`.
#' @param type Character. Either `"edge"` or `"vertex"`.
#' @param element Character. If `"edge"`, then it can be either `"line"` or
#' `"arrow"`, otherwise it can be either `"core"` or `"frame"`.
#' @param idx (optional) Integer vector. Indices of the elements to be modified.
#' When missing, all elements are modified.
#' @param ... Parameters to be modified/retrieved. This is passed to [grid::editGrob]
#' via [grid::gpar].
#'
#' @return An object of class `netplot` with modified parameters.
#'
#' @examples
#' library(igraph)
#' library(netplot)
#'
#' x <- make_ring(5)
#'
#' g <- nplot(x)
#'
#' # Updating edge color
#' g <- set_edge_gpar(g, col = "gray80")
#'
#' # Retrieving the color of the vertices (core)
#' get_vertex_gpar(g, element = "core", "fill", "lwd")
#'
#' @export
set_gpar <- function(x, type, element, idx, ...) {

  # Validations
  np_validate$is_netplot(x)
  np_validate$type(type)
  np_validate$elements(element, type)

  # If elements is more than one
  if (!missing(element) && length(element) > 1) {

    for (e in element)
      x <- do.call(
        set_gpar,
        c(list(x, type, e, idx), list(...))
        )

    return(x)

  }

  # If no specific ids are provided
  if (missing(idx))
    idx <- seq_len(ifelse(type == "vertex", x$.N, x$.M))

  # Creating loop sequence
  idx <- if (type == "vertex")
    as.list(idx)
  else
    unname(split(x$.edgelist[idx,], idx)) #lapply(idx, function(e) x$.edgelist[e, ])

  n <- ifelse(type == "edge", x$.M, x$.N)
  dots <- extend_parameter(x = x, n = n, ...)

  # Updating the
  for (i in seq_along(idx)) {

    # Fabricating the name
    iname <- netplot_name$make(idx[[i]])

    ele <- x$children$graph$children[[iname]]$children[[element]]

    if (!missing(element) & !is.null(ele)) {

      x$children$graph$children[[iname]]$children[[element]] <-
        grid::editGrob(
            grob = ele,
            gp = do.call(grid::gpar, dots[[i]])
          )

    } else {

      x$children$graph$children[[iname]] <- grid::editGrob(
        grob = x$children$graph$children[[iname]],
        gp = do.call(grid::gpar, dots[[i]])
      )

    }

  }

  # Returning the grob
  x

}

#' @rdname set_gpar
#' @details
#' `set_edge_gpar` and `set_vertex_gpar` are shorthands for
#' `set_gpar(type = "edge", ...)` and `set_gpar(type = "vertex", ...)`
#' respectively.
#'
#' @export
set_edge_gpar <- function(x, element, idx, ...) {

  if (missing(element))
    element <- c("line", "arrow")

  if (missing(idx))
    idx <- seq_len(x$.M)

  # dots <- list(...)
  #
  # dots$x       <- x
  # dots$type    <- "edge"
  # dots$element <- element
  # dots$idx     <- idx
  set_gpar(x = x, type = "edge", element = element, idx = idx, ...)

}

#' @rdname set_gpar
#' @export
set_vertex_gpar <- function(x, element, idx, ...) {

  if (missing(element))
    element <- c("core", "frame")

  if (missing(idx))
    idx <- seq_len(x$.N)

  # Step 0: Check whether col or fill was passed through ...
  dots <- list(...)

  # Step 1: Check if col/fill is a formula
  if(inherits(dots$col, "formula") || inherits(dots$fill, "formula")) {

    # Step 2: Extract RHS of formula using terms()
    if (inherits(dots$col, "formula")) {
      color_var <- attr(terms(dots$col), "term.labels")
    }
    if (inherits(dots$fill, "formula")) {
      fill_var <- attr(terms(dots$fill), "term.labels")
    }

    # Step 3: Call color_nodes() with graph and attribute
    if (inherits(dots$col, "formula")) {
      dots$col <- color_nodes(x$.graph, color_var)
    }
    if (inherits(dots$fill, "formula")) {
      dots$fill <- color_nodes(x$.graph, fill_var)
    }

    # Adding information for the legend
    x$.legend_vertex_col <- dots$col
    x$.legend_vertex_fill <- dots$fill

  }

  do.call(
    set_gpar,
    c(
      list(x = x, type = "vertex", element = element, idx = idx),
      dots
    )
  )

}

#' @rdname set_gpar
#' @details
#' `get_edge_gpar` and `get_vertex_gpar` are shorthands for
#' `get_gpar(type = "edge", ...)` and `get_gpar(type = "vertex", ...)`
#' respectively.
#' @export
get_vertex_gpar <- function(x, element, ..., idx) {

  if (missing(idx))
    idx <- seq_len(x$.N)

  get_gpar(x = x, type = "vertex", element = element, ..., idx = idx)

}

#' @rdname set_gpar
#' @export
get_edge_gpar <- function(x, element, ..., idx) {

  if (missing(idx))
    idx <- seq_len(x$.M)

  get_gpar(x = x, type = "edge", element = element, ..., idx = idx)

}

#' @rdname set_gpar
#' @param simplify Logical. When `TRUE` it tries to simplify the result.
#' Otherwise it returns a nested list.
#' @export
get_gpar <- function(x, type, element, ..., idx, simplify=TRUE) {

  # Checking type and element
  np_validate$is_netplot(x)
  np_validate$type(type)
  np_validate$elements(element, type)

  # If no id provided
  if (missing(idx))
    idx <- seq_len(ifelse(type == "vertex", x$.N, x$.M))

  # Creating loop sequence
  idx <- if (type == "vertex")
    as.list(idx)
  else
    unname(split(x$.edgelist[idx,], idx))#lapply(idx, function(e) x$.edgelist[e, ])

  # Basic cheks
  dots <- c(...)
  np_validate$gpar(dots)

  n   <- length(idx)
  ans <- lapply(seq_along(dots), function(i) vector("list", n))
  names(ans) <- dots

  for (p in dots)
    for (i in seq_along(idx)) {

      # Getting the component
      v <- x$children$graph$children[[netplot_name$make(idx[[i]])]]

      # If it exists
      if (!length(v) || !length(v$children[[element]])) {
        ans[[p]][[i]] <- NA
        next
       }

      # And it has the element
      v <- v$children[[element]]$gp[[p]]

      if (length(v))
        ans[[p]][[i]] <- v
      else
        ans[[p]][[i]] <- NA

    }

  if (simplify)
    ans <- lapply(ans, base::simplify2array)

  ans

}

pieslice <- function(a0, a1, r, d, x0, y0, edges, off) {

  # Intermideate points
  mid <- seq(a0, a1, by = 2*pi/edges*sign(a1 - a0))

  # In case that the points are not sufficient
  if (length(mid) < 3)
    mid <- seq(a0, a1, length.out = 3)
  if (utils::tail(mid, 1) != a1)
    mid <- seq(a0, a1, length.out = length(mid))

  # Computing midpoints
  mid <- cbind(cos(mid), sin(mid))
  m <- nrow(mid)

  if (d != 0) {
    pbase <- (mid*d)[m:1,]
  } else pbase <- c(0,0)

  ans <- rbind.data.frame(
    mid*r,
    pbase,
    make.row.names=FALSE
  )

  # Translating to the origin
  ans[,1] <- x0 + ans[,1]
  ans[,2] <- y0 + ans[,2]

  # Setting off
  ans[,1] <- ans[,1] + cos((a1 + a0)/2)*off
  ans[,2] <- ans[,2] + sin((a1 + a0)/2)*off

  colnames(ans) <- c("x", "y")
  ans


}

#' Computes the coordinates to plot a circle
#'
#' It actually does it by creating a polygon with 100 edges.
#' @param x0,y0 Numeric scalars. Center of the circle
#' @param r Numeric scalar. Radius of the circle.
#' @param rescale Logical scalar. When [TRUE], rescales the polygon
#' coordinates such that it preservers its aspectt ratio once plotted.
#'
#'@noRd
# @export
circle <- function(x0, y0, r, rescale=TRUE) {
  ans <-  pieslice(0, 2*pi, r=r, d=0, x0, y0, edges=100, off=0)
  ans <- ans[-nrow(ans), ]

  if (rescale)
    ans <- rescale_polygon(ans, y0)

  ans
}

#' A flexible piechart.
#'
#' While similar to [graphics::pie()], this function is much more
#' flexible as it allows providing different parameters for each slice of the pie.
#' Furthermore, it allows adding the plot to the current device, making it possible
#' to create compound piecharts.
#'
#' @param x Numeric vector. Values that specify the area of the slices.
#' @param add Logical scalar. When `TRUE` it is added to the current device.
#' @param radius Numeric vector. Radious of each slice (can be a scalar).
#' @param doughnut Numeric scalar. Radious of each inner circle (doughnut) (can be a scalar).
#' @param origin Numeric vector of length 2. Coordinates of the origin.
#' @param edges Numeric scalar. Smoothness of the slices curve (can be a vector).
#' @param slice.off Numeric vector. When `!=0`, specifies how much to
#' move the slice away from the origin. When scalar is recycled.
#' @param labels Character vector of length `length(x)`. Passed to
#' [graphics::text()].
#' @param tick.len Numeric scalar. Size of the tick marks as proportion of the radius.
#' @param text.args List. Further arguments passed to [graphics::text()].
#' @param segments.args List. Further arguments passed to [graphics::segments()]
#' when drawing the tickmarks.
#' @param init.angle Numeric scalar. Angle from where to start drawing in degrees.
#' @param last.angle Numeric scalar. Angle where to finish drawing in degrees.
#' @param skip.plot.slices Logical scalar. When `FALSE`, slices are not drawn.
#' This can be useful if, for example, the user only wants to draw the labels.
#' @param rescale Logical scalar. When `TRUE` (default), the y-coordinates of
#' the polygons (slices), text and tickmarks will be rescaled such that the
#' aspectt ratio is preserved, i.e. looks like a circle.
#' @param ... Further arguments passed to [graphics::polygon()]
#' (see details).
#'
#' @return
#' A list with the following elements:
#' \item{slices}{A list of length `length(x)` with the coordinates of each
#'   slice.}
#' \item{textcoords}{A numeric matrix of size `length(x)*2` with
#'   coordinates where the labels can be put at.}
#' \item{alpha0}{A numeric vector of size `length(x)` with the starting
#' degree in radians of the slice.}
#' \item{alpha1}{A numeric vector of size `length(x)` with the ending
#' degree in radians of the slice.}
#'
#' @details The function is a wrapper of [graphics::polygon()],
#' so all parameters such as color, density, border, etc. are passed directly
#' by [mapply()] so that are specified one per slice. The coordinates
#' of the slices are computed internally.
#'
#'
#' @seealso https://commons.wikimedia.org/wiki/File:Nightingale-mortality.jpg
#' @export
#' @examples
#'
#' # Example 1 -----------------------------------------------------------------
#' # A set of 3 nested rings rings starting at 315 deg. and ending at 270 deg.
#'
#' # Values to plot
#' vals <- c(1,2,3,10)
#'
#' # Outer (includes labels)
#' piechart(vals, col=grDevices::blues9[5:8], border=NA, doughnut = .5,
#'     radius=.75, labels=vals, init.angle = 315, last.angle = 270)
#'
#' # Middle
#' piechart(vals, col=grDevices::blues9[3:6], border=NA, doughnut = .3,
#'     radius=.5, add=TRUE, init.angle = 315, last.angle = 270)
#'
#' # Inner
#' piechart(vals, col=grDevices::blues9[1:4], border="gray", doughnut = .1,
#'     radius=.3, add=TRUE, init.angle = 315, last.angle = 270)
#'
#' # Example 2 -----------------------------------------------------------------
#' # Passing values to polygon and playing with the radius and slice.off
#'
#' piechart(1:10, density=(1:10)^2/2, slice.off = (1:10)/30, doughnut = .5,
#'   radius = sqrt(10:1),
#'   # Here we are setting random labels...
#'   labels=sapply(1:10, function(x) paste(sample(letters, x, TRUE), collapse=""))
#'   )
#'
piechart <- function(
  x,
  labels = names(x),
  radius = 1,
  doughnut = 0,
  origin = c(0,0),
  edges = 200,
  slice.off = 0,
  init.angle = 0,
  last.angle = 360,
  tick.len = .1,
  text.args = list(),
  segments.args = list(),
  skip.plot.slices=FALSE,
  add = FALSE,
  rescale = TRUE,
  ...) {

  # X must be numeric
  if (!is.numeric(x))
    stop("-x- must be numeric")
  x <- tryCatch(as.vector(x), error = function(e) e)
  if (inherits(x, "error"))
    stop("Coercion of -x- as vector failed:\n", x)

  # Assigning alpha
  init.angle <- init.angle/360*2.0*pi # as radians
  last.angle <- last.angle/360*2.0*pi

  alpha1 <- cumsum(x/sum(x)*ifelse(
    init.angle >= last.angle, 2*pi - init.angle + last.angle,
    last.angle - init.angle
  )) + init.angle
  alpha0 <- c(init.angle, alpha1[-length(x)])

  ans <- mapply(
    pieslice,
    a0 = alpha0,
    a1 = alpha1,
    r = radius, d=doughnut, x0=origin[1], y0=origin[2],
    edges = edges,
    off = slice.off,
    SIMPLIFY = FALSE
  )

  # Fetching size
  cex <- if (length(labels) && ("cex" %in% names(text.args)))
    text.args[["cex"]]
  else 1

  # Creating the device
  maxradius <- max(radius)
  if (!add) {
    graphics::plot.new()

    # Adjusting so that we get nice circles
    adj <- graphics::par()$pin
    adj <- adj[1]/adj[2]

    # Adjusting the size... including the labels
    ran <- if (length(labels))
      max(
        c(
          graphics::strwidth(labels, units="user", cex=cex),
          graphics::strheight(labels, units="user", cex=cex)
        ),
        na.rm=TRUE)*2
    else 0

    ran <- (ran + maxradius*1.1 + tick.len/2 + max(slice.off))*c(-1,1)

    # Adjustment depending on the asp
    if (adj > 1)
      graphics::plot.window(xlim=ran*adj, ylim = ran)
    else
      graphics::plot.window(xlim=ran, ylim = ran*adj)
  }

  # Adjusting the polygon
  if (rescale)
    ans <- rescale_polygon(ans, origin[2])

  # Adding the slices
  if (!skip.plot.slices)
    mapply(graphics::polygon,
           x = lapply(ans, "[", j=1, i=),
           y = lapply(ans, "[", j=2, i=),
           ..., SIMPLIFY = FALSE
    )

  # Midpoints
  angles     <- (alpha0 + alpha1)/2
  textcoords <- cbind(
    origin[1] + cos(angles)*(radius*1.05 + tick.len/2 + slice.off) ,
    origin[2] + sin(angles)*(radius*1.05 + tick.len/2 + slice.off)
  )

  # If labels are passed
  if (length(labels)) {

    # Adjusting according to string lenght
    textcoords <- textcoords +
      cbind(
        cos(angles)*graphics::strwidth(labels, cex=cex)/2,
        sin(angles)*graphics::strwidth(labels, cex=cex)/2
      )

    if (rescale)
      textcoords <- rescale_polygon(textcoords, origin[2])

    # Drawing the text
    do.call(mapply,
            c(list(FUN = graphics::text,
                   x=textcoords[,1],
                   y=textcoords[,2],
                   labels=labels,SIMPLIFY = FALSE),
              text.args
            ))


    # Here should go the tick marks...
    x0 <- origin[1] + cos(angles)*(radius - tick.len/2 + slice.off)
    x1 <- origin[1] + cos(angles)*(radius + tick.len/2 + slice.off)
    y0 <- origin[2] + sin(angles)*(radius - tick.len/2 + slice.off)
    y1 <- origin[2] + sin(angles)*(radius + tick.len/2 + slice.off)

    if (rescale) {
      y0 <- rescale_polygon(y0, origin[2])
      y1 <- rescale_polygon(y1, origin[2])
    }


    toplot <- which(!is.na(labels))
    do.call(graphics::segments,
            c(
              list(
                x0 = x0[toplot],
                y0 = y0[toplot],
                x1 = x1[toplot],
                y1 = y1[toplot]
              ),
              segments.args
            ))

  }


  # Returning
  invisible(
    list(
      slices     = ans,
      textcoords = textcoords,
      alpha0     = alpha0,
      alpha1     = alpha1
    )
  )

}


#' Function to create a color key
#'
#' @param x0,x1,y0,y1 Numeric scalars. Coordinates of the lower left and
#' upper right points where the color key will be drawn as proportion of
#' the plotting region.
#' @param label.from,label.to Character scalar. Labels of the lower and
#' upper values of the color key.
#' @param tick.range,tick.marks Numeric vectors specifying the range and
#' the tickmarks respectively.
#' @param cols Character scalar. Colors specifications to create the
#' color palette.
#' @param nlevels Integer scalar. Number of levels to extrapolate.
#' @param main Character scalar. Title of the colorkey.
#' @param relative Logical scalar. When `TRUE` the color key is drawn relative
#' to the plotting region area taking `x0, x1, y0, y1` as relative location.
#' @param tick.args,label.args,main.args Lists of arguments passed to
#' [graphics::text] for drawing ticks, labels and main respectively.
#' @export
#' @return NULL.
#' @examples
#' set.seed(22231)
#'
#' # A random figure
#' dat <- matrix(runif(100*3), ncol = 3)
#' col <- colorRamp2(c("blue", "white", "red"))
#'
#' plot(
#'   dat[,1], dat[,2],
#'   col = rgb(col(dat[,3]), maxColorValue=255),
#'   cex=2, pch=20
#'   )
#'
#' # Pretty color key
#' colorkey(
#'   x0 = .60, y0 = .80,
#'   x1 = .95, y1 = .95,
#'   cols = c("blue", "white", "red"),
#'   main = "Some color scale"
#' )
colorkey <- function(
  x0,y0,x1,y1,
  cols       = c("white", "steelblue"),
  tick.range = c(0, 1),
  tick.marks = seq(tick.range[1], tick.range[2], length.out = 5L),
  label.from = NULL,
  label.to   = NULL,
  nlevels    = 100,
  main       = NULL,
  relative   = TRUE,
  tick.args  = list(),
  label.args = list(),
  main.args  = list()
) {

  # If relative (the default, will be added to )
  if (relative) {
    usr <- graphics::par("usr")
    xpd <- graphics::par("xpd")
    on.exit({
      graphics::plot.window(
        usr[1:2],
        usr[3:4],
        new=FALSE,
        xpd=xpd,
        xaxs="i",
        yaxs="i"
      )
    })
    graphics::plot.window(c(0,1), c(0,1), new=FALSE, xpd=NA)

  }

  # Adjusting to textsize
  x0 <- x0 + graphics::strwidth(label.from)
  x1 <- x1 - graphics::strwidth(label.to)
  y0 <- y0 + max(graphics::strheight(tick.marks))
  y1 <- y1 - graphics::strheight(main)

  # Writing labels
  do.call(
    graphics::text,
    c(list(
      x      = c(
        x0 - graphics::strwidth(label.from)/2,
        x1 + graphics::strwidth(label.to)/2
      ),
      y      = rep((y1+y0)/2, 2),
      labels = c(label.from, label.to)
    ), label.args)
  )

  # Readjusting for more space
  x0 <- x0 + (x1-x0)/40
  x1 <- x1 - (x1-x0)/40

  # Computing coordinates
  cols <- grDevices::colorRampPalette(cols)(nlevels)
  n  <- length(cols)
  xsize <- (x1 - x0)/n

  xcoords <- seq(x0 + xsize/2, x1 - xsize/2, length.out = n)
  ycoords <- rep((y1+y0)/2, n)

  # xcoords[1] <- xcoords[1] + xsize

  # Drawing rectangles
  graphics::symbols(
    x          = xcoords,
    y          = ycoords,
    inches     =FALSE,
    bg         = cols,
    fg         = "transparent",
    rectangles = cbind(rep(xsize, n), y1-y0),
    add        = TRUE
  )

  # Bouding box
  graphics::symbols(
    x = (x1 + x0)/2,
    y = (y1 + y0)/2,
    rectangles = cbind((x1 - x0), y1 - y0),
    add=TRUE, inches = FALSE
  )

  # Top tickmarks
  tick.pos <- (tick.marks - tick.range[1])/(tick.range[2] - tick.range[1])*(x1-x0) + x0
  graphics::segments(
    x0 = tick.pos,
    y0 = y0 - (y1 - y0)/5 ,
    x1 = tick.pos,
    y1 = y0 + (y1 - y0)/5
  )

  do.call(
    graphics::text,
    c(
      list(
        x = tick.pos,
        y = y0 - (y1 - y0)/5 - graphics::strheight(max(tick.marks, na.rm=TRUE))/1.5,
        labels = tick.marks
      ), tick.args))

  if (length(main))
    do.call(
      graphics::text,
      c(list(
        x = (x1 + x0)/2,
        y = y1,
        pos = 3,
        labels = main
      ), main.args)
    )

  invisible()
}

#' A wrapper of `colorRamp2`
#' @param i,j Integer scalar. Indices of ego and alter from 1 through n.
#' @param p Numeric scalar from 0 to 1. Proportion of mixing.
#' @param vcols Vector of colors.
#' @param alpha Numeric scalar from 0 to 1. Passed to [colorRamp2]
#' @return A color.
#' @noRd
edge_color_mixer <- function(i, j, vcols, p = .5, alpha = .15) {

  alphacolor(
    sprintf(
      "%sFF", grDevices::rgb(
        colorRamp2(vcols[c(i,j)], alpha = FALSE)(p),
        maxColorValue = 255
        )
      ), alpha.f = alpha
    )

}

#' Plot a network
#'
#' `nplot()` is the main function of the **netplot** package. It draws a network
#' using the **grid** graphics system (the same engine that powers **ggplot2**),
#' emphasizing aesthetics and providing sensible defaults that yield
#' out-of-the-box nice visualizations. Compared with base `igraph`/`network`
#' plots, `nplot()` auto-scales vertices and edges relative to the plotting
#' device, draws truly curved edges, mixes edge colors from their endpoints,
#' and fills the device efficiently.
#'
#' Vertex and edge aesthetics can be set directly (passing a vector) or,
#' conveniently, mapped from graph attributes using a *formula* interface (see
#' the "Mapping attributes with formulas" section below). The returned object is
#' a **grid** `grob`, so it can be further edited with [set_vertex_gpar()] /
#' [set_edge_gpar()], combined with other grid graphics (e.g. via
#' `gridExtra::grid.arrange()`), or annotated with a legend through
#' [nplot_legend()].
#'
#' @param x A graph. It supports networks stored as `igraph`, `network`, and
#' matrices objects (see details).
#' @param bg.col Color of the background.
#' @param layout Numeric two-column matrix with the graph layout in x/y positions of the vertices.
#' @param vertex.size Numeric vector of length `vcount(x)`. Absolute size of the
#' vertex from 0 to 1. Can also be a one-sided formula (e.g. `~ degree`) naming a
#' numeric vertex attribute to map sizes from (see "Mapping attributes with
#' formulas").
#' @param vertex.nsides Numeric vector of length `vcount(x)`. Number of sides of
#' the vertex. E.g. three is a triangle, and 100 approximates a circle. Can also
#' be a one-sided formula (e.g. `~ group`) naming a vertex attribute; each unique
#' value is then mapped to a distinct shape (see "Mapping attributes with
#' formulas").
#' @param vertex.color Vector of length `vcount(x)`. Vertex HEX or built in
#' colors. Can also be a one-sided formula (e.g. `~ group`) naming a vertex
#' attribute to color vertices by (see "Mapping attributes with formulas").
#' @param vertex.size.range Numeric vector of length 2 or 3, or `NULL`. Relative size for the
#' minimum and maximum of the plot, and curvature of the scale. The third number
#' is used as `size^rel[3]`. If `NULL`, scaling is suppressed and `vertex.size`
#' is used as is.
#' @param vertex.frame.color Vector of length `vcount(x)`. Border of vertex in
#' HEX or built in colors.
#' @param vertex.frame.prop Vector of length `vcount(x)`. What proportion of the
#' vertex does the frame occupy (values between 0 and 1).
#' @param vertex.rot Vector of length `vcount(x)` in Radians. Passed to [npolygon],
#' elevation degree from which the polygon is drawn.
#' @param vertex.label Character vector of length `vcount(x)`. Labels.
#' @param vertex.label.fontsize Numeric vector.
#' @param vertex.label.color Vector of colors of length `vcount(x)`.
#' @param vertex.label.fontfamily Character vector of length `vcount(x)`.
#' @param vertex.label.fontface See [grid::gpar]
#' @param vertex.label.show Numeric scalar. Proportion of labels to show as the
#' top ranking according to `vertex.size`.
#' @param vertex.label.range Numeric vector of size 2 or 3. Relative scale of
#' `vertex.label.fontsize` in points (see [grid::gpar]).
#' @param edge.color A vector of length `ecount(x)`. In HEX or built in colors.
#' Can be `NULL` in which case
#' the color is picked as a mixture between ego and alters' `vertex.color` values.
#' @param edge.width Numeric vector of length `ecount(x)`. Relative edge widths.
#' Values are normalized and then mapped to the range specified by `edge.width.range`,
#' unless `edge.width.range` is `NULL`.
#' For `nplot.igraph` and `nplot.network`, defaults to the "weight" edge
#' attribute if present; otherwise all edges use width 1. Can also be a one-sided
#' formula (e.g. `~ weight`) naming a numeric edge attribute (see "Mapping
#' attributes with formulas").
#' @param edge.width.range Numeric vector of length 2, or `NULL`. The minimum and maximum line
#' widths (in points) to use when mapping `edge.width` values. For example,
#' `c(1, 4)` maps the smallest edge weight to 1pt and the largest to 4pt.
#' If `NULL`, scaling is suppressed and `edge.width` is used as is.
#' @param edge.arrow.size Vector of length `ecount(x)` from 0 to 1.
#' @param edge.curvature Numeric vector of length `ecount(x)`. Curvature of edges
#' in terms of radians.
#' @param edge.line.lty Vector of length `ecount(x)`. Line types in R
#' (e.g.- 1 = Solid, 2 = Dashed, etc).
#' @param edge.line.breaks Vector of length `ecount(x)`. Number of vertices to
#' draw (approximate) the arc (edge).
#' @param sample.edges Numeric scalar between 0 and 1. Proportion of edges to
#' sample.
#' @param skip.vertex,skip.edges,skip.arrows Logical scalar. When `TRUE` the
#' object is not plotted.
#' @param add Logical scalar.
#' @param zero.margins Logical scalar.
#' @importFrom igraph layout_with_fr degree vcount ecount
#' @importFrom grDevices adjustcolor rgb
#' @importFrom graphics lines par plot polygon rect segments plot.new plot.window
#' @import grid
#'
#' @details
#' When `x` is of class [matrix], it will be passed to [igraph::graph_from_adjacency_matrix()].
#'
#' @section Mapping attributes with formulas:
#'
#' Several aesthetics can be mapped directly from graph attributes by passing a
#' one-sided formula naming the attribute, instead of building the vector by
#' hand. The mapping depends on the aesthetic:
#'
#' - `vertex.color = ~ attr` colors vertices by the vertex attribute `attr`.
#'   Character/factor attributes are mapped to a categorical palette, numeric
#'   attributes to a continuous gradient, and logical attributes to two colors.
#'   When used this way, `print()`-ing the resulting plot also draws a matching
#'   legend: a categorical key for discrete attributes and a continuous color
#'   bar for continuous ones.
#' - `vertex.nsides = ~ attr` maps each unique value of `attr` to a distinct
#'   vertex shape (triangle, square, pentagon, ...).
#' - `vertex.size = ~ attr` and `edge.width = ~ attr` scale sizes/widths from a
#'   numeric vertex/edge attribute.
#' - `edge.color` uses a different, richer formula grammar based on `ego()` and
#'   `alter()` to mix the endpoints' colors; see [netplot-formulae].
#'
#' For `vertex.nsides`, `vertex.size`, and `edge.width` the right-hand side of
#' the formula is *evaluated* with the graph's attributes in scope, so besides
#' bare names you can use expressions, e.g. `edge.width = ~ log1p(weight)` or
#' `vertex.size = ~ degree ^ 2`.
#'
#' For example, `nplot(x, vertex.color = ~ gender, vertex.size = ~ degree)`
#' colors vertices by the `gender` attribute and sizes them by `degree`. The
#' same attribute-mapping formulas also work in [set_vertex_gpar()].
#' @return An object of class `c("netplot", "gTree", "grob", "gDesc")`. The object
#' has an additional set of attributes:
#' * `.xlim, .ylim` vector of size two with the x-asis/y-axis limits.
#' * `.layout` A numeric matrix of size `vcount(x) * 2` with the vertices positions
#' * `.edgelist` A numeric matrix, The edgelist.
#'
#' @examples
#' library(igraph)
#' library(netplot)
#' set.seed(1)
#' x <- sample_smallworld(1, 200, 5, 0.03)
#'
#' plot(x) # ala igraph
#' nplot(x) # ala netplot
#'
#' # Mapping aesthetics from vertex attributes using formulas
#' V(x)$grp <- sample(letters[1:3], vcount(x), replace = TRUE)
#' V(x)$deg <- degree(x)
#' nplot(
#'   x,
#'   vertex.color  = ~ grp, # color by the categorical attribute
#'   vertex.nsides = ~ grp, # and give each group a distinct shape
#'   vertex.size   = ~ deg  # size by a numeric attribute
#' )
#' @name nplot
#' @aliases netplot
NULL



#' @export
#' @rdname nplot
nplot <- function(
  x,
  layout,
  vertex.size             = 1,
  bg.col                  = "transparent",
  vertex.nsides           = 10,
  vertex.color            = grDevices::hcl.colors(1),
  vertex.size.range       = c(.01, .03, 4),
  vertex.frame.color      = NULL,
  vertex.rot              = 0,
  vertex.frame.prop       = .2,
  vertex.label            = NULL,
  vertex.label.fontsize   = NULL,
  vertex.label.color      = adjustcolor("black", alpha.f = .80),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface   = "plain",
  vertex.label.show       = .3,
  vertex.label.range      = c(5, 15),
  edge.width              = 1,
  edge.width.range        = c(1, 2),
  edge.arrow.size         = NULL,
  edge.color              = ~ ego(alpha = .1, col = "gray") + alter,
  edge.curvature          = pi/3,
  edge.line.lty           = "solid",
  edge.line.breaks        = 5,
  sample.edges            = 1,
  skip.vertex             = FALSE,
  skip.edges              = FALSE,
  skip.arrows             = skip.edges,
  add                     = FALSE,
  zero.margins            = TRUE,
  edgelist
  ) {

  UseMethod("nplot")

}

#' @export
#' @rdname nplot
nplot.igraph <- function(
  x,
  layout                  = igraph::layout_nicely(x),
  vertex.size             = igraph::degree(x, mode="in"),
  bg.col                  = "transparent",
  vertex.nsides           = 10,
  vertex.color            = grDevices::hcl.colors(1),
  vertex.size.range       = c(.01, .03, 4),
  vertex.frame.color      = NULL,
  vertex.rot              = 0,
  vertex.frame.prop       = .2,
  vertex.label            = igraph::vertex_attr(x, "name"),
  vertex.label.fontsize   = NULL,
  vertex.label.color      = adjustcolor("black", alpha.f = .80),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface   = "plain",
  vertex.label.show       = .3,
  vertex.label.range      = c(5, 15),
  edge.width              = igraph::edge_attr(x, "weight"),
  edge.width.range        = c(1, 2),
  edge.arrow.size         = NULL,
  edge.color              = ~ ego(alpha = .1, col = "gray") + alter,
  edge.curvature          = pi/3,
  edge.line.lty           = "solid",
  edge.line.breaks        = 5,
  sample.edges            = 1,
  skip.vertex             = FALSE,
  skip.edges              = FALSE,
  skip.arrows             = !igraph::is_directed(x),
  add                     = FALSE,
  zero.margins            = TRUE,
  edgelist
  ) {

  if (!length(edge.width))
    edge.width <- 1L

  nplot.default(
    x = x,
    layout = layout,
    vertex.size = vertex.size,
    bg.col = bg.col,
    vertex.nsides = vertex.nsides,
    vertex.color = vertex.color,
    vertex.size.range = vertex.size.range,
    vertex.frame.color = vertex.frame.color,
    vertex.rot = vertex.rot,
    vertex.frame.prop = vertex.frame.prop,
    vertex.label = vertex.label,
    vertex.label.fontsize = vertex.label.fontsize,
    vertex.label.color = vertex.label.color,
    vertex.label.fontfamily = vertex.label.fontfamily,
    vertex.label.fontface = vertex.label.fontface,
    vertex.label.show = vertex.label.show,
    vertex.label.range = vertex.label.range,
    edge.width = edge.width,
    edge.width.range = edge.width.range,
    edge.arrow.size = edge.arrow.size,
    edge.color = edge.color,
    edge.curvature = edge.curvature,
    edge.line.lty = edge.line.lty,
    edge.line.breaks = edge.line.breaks,
    sample.edges = sample.edges,
    skip.vertex = skip.vertex,
    skip.edges = skip.edges,
    skip.arrows = skip.arrows,
    add = add,
    zero.margins = zero.margins,
    edgelist = igraph::as_edgelist(x, names = FALSE)
  )

}

#' @export
#' @rdname nplot
#' @importFrom network as.edgelist
#' @importFrom sna gplot.layout.kamadakawai
nplot.network <- function(
  x,
  layout                  = sna::gplot.layout.kamadakawai(x, NULL),
  vertex.size             =  sna::degree(x, cmode="indegree"),
  bg.col                  = "transparent",
  vertex.nsides           = 10,
  vertex.color            = grDevices::hcl.colors(1),
  vertex.size.range       = c(.01, .03, 4),
  vertex.frame.color      = NULL,
  vertex.rot              = 0,
  vertex.frame.prop       = .2,
  vertex.label            =  network::get.vertex.attribute(x, "vertex.names"),
  vertex.label.fontsize   = NULL,
  vertex.label.color      = adjustcolor("black", alpha.f = .80),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface   = "plain",
  vertex.label.show       = .3,
  vertex.label.range      = c(5, 15),
  edge.width              = network::get.edge.attribute(x, "weight"),
  edge.width.range        = c(1, 2),
  edge.arrow.size         = NULL,
  edge.color              = ~ ego(alpha = .1, col = "gray") + alter,
  edge.curvature          = pi/3,
  edge.line.lty           = "solid",
  edge.line.breaks        = 5,
  sample.edges            = 1,
  skip.vertex             = FALSE,
  skip.edges              = FALSE,
  skip.arrows             =  !network::is.directed(x),
  add                     = FALSE,
  zero.margins            = TRUE,
  edgelist
) {

  if (!length(edge.width))
    edge.width <- 1L

  nplot.default(
    x = x,
    layout = layout,
    vertex.size = vertex.size,
    bg.col = bg.col,
    vertex.nsides = vertex.nsides,
    vertex.color = vertex.color,
    vertex.size.range = vertex.size.range,
    vertex.frame.color = vertex.frame.color,
    vertex.rot = vertex.rot,
    vertex.frame.prop = vertex.frame.prop,
    vertex.label = vertex.label,
    vertex.label.fontsize = vertex.label.fontsize,
    vertex.label.color = vertex.label.color,
    vertex.label.fontfamily = vertex.label.fontfamily,
    vertex.label.fontface = vertex.label.fontface,
    vertex.label.show = vertex.label.show,
    vertex.label.range = vertex.label.range,
    edge.width = edge.width,
    edge.width.range = edge.width.range,
    edge.arrow.size = edge.arrow.size,
    edge.color = edge.color,
    edge.curvature = edge.curvature,
    edge.line.lty = edge.line.lty,
    edge.line.breaks = edge.line.breaks,
    sample.edges = sample.edges,
    skip.vertex = skip.vertex,
    skip.edges = skip.edges,
    skip.arrows = skip.arrows,
    add = add,
    zero.margins = zero.margins,
    edgelist = network::as.edgelist(x)
  )

}

#' @export
#' @rdname nplot
#' @importFrom igraph graph_from_adjacency_matrix
nplot.matrix <- function(
  x,
  layout,
  vertex.size             = 1,
  bg.col                  = "transparent",
  vertex.nsides           = 10,
  vertex.color            = grDevices::hcl.colors(1),
  vertex.size.range       = c(.01, .03, 4),
  vertex.frame.color      = NULL,
  vertex.rot              = 0,
  vertex.frame.prop       = .2,
  vertex.label            = NULL,
  vertex.label.fontsize   = NULL,
  vertex.label.color      = adjustcolor("black", alpha.f = .80),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface   = "plain",
  vertex.label.show       = .3,
  vertex.label.range      = c(5, 15),
  edge.width              = 1,
  edge.width.range        = c(1, 2),
  edge.arrow.size         = NULL,
  edge.color              = ~ ego(alpha = .1, col = "gray") + alter,
  edge.curvature          = pi/3,
  edge.line.lty           = "solid",
  edge.line.breaks        = 5,
  sample.edges            = 1,
  skip.vertex             = FALSE,
  skip.edges              = FALSE,
  skip.arrows             = skip.edges,
  add                     = FALSE,
  zero.margins            = TRUE,
  edgelist
  ) {

  x <- igraph::graph_from_adjacency_matrix(x)
  if (missing(layout))
    layout <- igraph::layout_nicely(x)

  nplot.igraph(
    x = x,
    layout = layout,
    vertex.size = vertex.size,
    bg.col = bg.col,
    vertex.nsides = vertex.nsides,
    vertex.color = vertex.color,
    vertex.size.range = vertex.size.range,
    vertex.frame.color = vertex.frame.color,
    vertex.rot = vertex.rot,
    vertex.frame.prop = vertex.frame.prop,
    vertex.label = vertex.label,
    vertex.label.fontsize = vertex.label.fontsize,
    vertex.label.color = vertex.label.color,
    vertex.label.fontfamily = vertex.label.fontfamily,
    vertex.label.fontface = vertex.label.fontface,
    vertex.label.show = vertex.label.show,
    vertex.label.range = vertex.label.range,
    edge.width = edge.width,
    edge.width.range = edge.width.range,
    edge.arrow.size = edge.arrow.size,
    edge.color = edge.color,
    edge.curvature = edge.curvature,
    edge.line.lty = edge.line.lty,
    edge.line.breaks = edge.line.breaks,
    sample.edges = sample.edges,
    skip.vertex = skip.vertex,
    skip.edges = skip.edges,
    skip.arrows = skip.arrows,
    add = add,
    zero.margins = zero.margins,
    edgelist = NULL
  )

}

netplot_theme <- (function() {

  v <- grDevices::hcl.colors(1)
  darkv <- adjustcolor(
    v,
    red.f   = .5,
    green.f = .5,
    blue.f  = .5
  )

  default <- list(
    vertex.core.col   = v,
    vertex.core.fill  = v,
    vertex.frame.col  = darkv,
    vertex.frame.fill = darkv,
    edge.col = ~ego(alpha=0) + alter(alpha = .7),
    name      = "default"
  )

  current <- default

  list(
    set = function(...) {
      dots <- list(...)
      if (!length(dots$name))
        dots$name <- "user"

      current <<- rev(dots)
    },
    get = function() current,
    reset = function() {
      message("Restablishing the default theme")
      current <<- default
    },
    vertex_defaults = function() {
      str(current[grepl("^vertex\\.", names(current))])
    },
    edge_defaults = function() {
      str(current[grepl("^edge\\.", names(current))])
    }
  )

})()

# Function to retrieve the dev.size only if there's an active device
dev_size <- function(...) {

  if (length(grDevices::dev.list())) {

    grDevices::dev.size(...)

  } else
    c(7, 7)

}

#' @export
#' @rdname nplot
#' @param edgelist An edgelist.
#' @return
#' In the case of `nplot.default`, an object of class `netplot` and `grob` (see
#' [grid::grob]) with the following slots:
#'
#' - `children` The main `grob` of the object.
#' - `name` Character scalar. The name of the plot
#' - `.xlim` and `.ylim` Two vectors indicating the limits of the plot
#' - `.layout` A two-column matrix with the location of the vertices.
#' - `.edgelist` A two-column matrix, an edgelist.
#' - `.N` Integer. The number of vertices.
#' - `.M` Integer. The number of edges.
#'
#' The `children` `grob` contains the following two objects:
#'
#' - `background` a `grob` rectangule.
#' - `graph` a `gTree` that contains each vertex and each edge
#' of the figure.
#' @seealso [nplot_base]
nplot.default <- function(
  x,
  layout,
  vertex.size             = 1,
  bg.col                  = "transparent",
  vertex.nsides           = 10,
  vertex.color            = grDevices::hcl.colors(1),
  vertex.size.range       = c(.01, .03, 4),
  vertex.frame.color      = NULL,
  vertex.rot              = 0,
  vertex.frame.prop       = .2,
  vertex.label            = NULL,
  vertex.label.fontsize   = NULL,
  vertex.label.color      = adjustcolor("black", alpha.f = .80),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface   = "plain",
  vertex.label.show       = .3,
  vertex.label.range      = c(5, 15),
  edge.width              = 1,
  edge.width.range        = c(1, 2),
  edge.arrow.size         = NULL,
  edge.color              = ~ ego(alpha = .1, col = "gray") + alter,
  edge.curvature          = pi/3,
  edge.line.lty           = "solid",
  edge.line.breaks        = 5,
  sample.edges            = 1,
  skip.vertex             = FALSE,
  skip.edges              = FALSE,
  skip.arrows             = skip.edges,
  add                     = FALSE,
  zero.margins            = TRUE,
  ...,
  edgelist
  ) {

  # # We turn off the device if not need
  # if (length(grDevices::dev.list()) == 0L) {
  #   on.exit(
  #     grDevices::dev.off(grDevices::dev.cur())
  #   )
  # }

  # listing objects
  netenv <- environment()

  # Checking layout
  if (!inherits(layout, what = c("matrix")))
    stop(
      "-layout- should be of class 'matrix'. It currently is '",
      class(layout),
      "'"
      )

  N <- nrow(layout)
  M <- nrow(edgelist)
  graph_class <- class(x)

  # Mapping attributes ---------------------------------------------------------

  # Nsides. The formula RHS is evaluated against the graph's vertex attributes,
  # so both bare names (~ group) and expressions (~ cut(age, 3)) work.
  if (length(vertex.nsides) && inherits(vertex.nsides, "formula")) {

    vertex.nsides <- map_attribute_to_shape(
      eval_attribute_formula(graph = x, formula = vertex.nsides, type = "vertex")
    )

  }

  # And size
  if (length(vertex.size) && inherits(vertex.size, "formula")) {

    vertex.size <- eval_attribute_formula(
      graph = x, formula = vertex.size, type = "vertex"
      )

    # Now check if it is numeric. If not, it should return an error
    if (!is.numeric(vertex.size)) {
      stop("vertex.size must be numeric")
    }

  }

  # Edges width
  if (length(edge.width) && inherits(edge.width, "formula")) {

    edge.width <- eval_attribute_formula(
      graph = x, formula = edge.width, type = "edge"
      )

    # Now check if it is numeric. If not, it should return an error
    if (!is.numeric(edge.width)) {
      stop("edge.width must be numeric")
    }

  }

  # Sampling edges -------------------------------------------------------------
  if (sample.edges < 1) {

    sample.edges <- sample.int(
      netenv$M, floor(netenv$M * sample.edges),
      replace = FALSE
      )

    sample.edges <- sort(sample.edges)

    edgelist <- edgelist[sample.edges, , drop=FALSE]

    # Do we need to resize the edgepars?
    epars <- c(
      "edge.width",
      "edge.width.range",
      "edge.arrow.size",
      "edge.color",
      "edge.curvature",
      "edge.line.lty",
      "edge.line.breaks"
      )

    for (epar in epars) {

      if (length(netenv[[epar]]) == netenv$M)
        netenv[[epar]] <- netenv[[epar]][sample.edges]

    }

    netenv$M <- length(sample.edges)

  }

  # end ------------------------------------------------------------------------

  if (
    length(edge.line.lty) &&
    length(edge.line.lty) != 1L &&
    length(edge.line.lty) != netenv$M
  )
    stop("edge.line.lty must have length 1 or one value per plotted edge.")

  # This function will repeat a patter taking into account the number of columns
  .rep <- function(x, .times) {
    if (grepl("range$", p) | inherits(x, "formula"))
      return(x)
    matrix(rep(x, .times), ncol = length(x), byrow = TRUE)
  }

  # Checking defaults for vertex
  vertex_par <- ls(pattern = "^vertex\\.", envir = netenv)
  for (p in vertex_par)
    if (length(netenv[[p]]) > 0 && length(netenv[[p]]) < netenv$N)
      netenv[[p]] <- .rep(netenv[[p]], netenv$N)

  # Checking defaults for edges
  edge_par <- ls(pattern = "^edge\\.", envir = netenv)
  for (p in edge_par)
    if (length(netenv[[p]]) > 0 && length(netenv[[p]]) < netenv$M)
      netenv[[p]] <- .rep(netenv[[p]], netenv$M)

  # Adjusting size -------------------------------------------------------------

  # Adjusting layout to fit the device
  netenv$layout <- fit_coords_to_dev(netenv$layout)

  # Rescaling size
  if (!skip.vertex) {
    netenv$vertex.size <- rescale_size(
      netenv$vertex.size,
      rel = netenv$vertex.size.range
      )
  } else
    netenv$vertex.size <- rep(0, netenv$N)

  # Rescaling edges
  if (is.null(netenv$edge.width.range)) {
    netenv$edge.width <- rescale_size(
      netenv$edge.width,
      rel = netenv$edge.width.range
      )
  } else {
    netenv$edge.width <- rescale_size(
      netenv$edge.width/max(netenv$edge.width, na.rm=TRUE),
      rel = netenv$edge.width.range
      )
  }

  # Rescaling arrows
  if (!length(netenv$edge.arrow.size))
    netenv$edge.arrow.size <- netenv$vertex.size[edgelist[,1]]/1.5

  if (netenv$skip.arrows)
    netenv$edge.arrow.size <- rep(0.0, length(netenv$edge.arrow.size))

  # Rescaling text
  if (!length(netenv$vertex.label.fontsize)) {
    if (is.null(netenv$vertex.label.range))
      netenv$vertex.label.range <- c(5, 15)

    netenv$vertex.label.fontsize <- rescale_size(
      netenv$vertex.size,
      rel = netenv$vertex.label.range
      )
  }

  # Computing label threshold
  netenv$label_threshold <- stats::quantile(
    netenv$vertex.size, 1 - netenv$vertex.label.show[1])

  # Calculating arrow adjustment
  netenv$arrow.size.adj <- netenv$edge.arrow.size*cos(pi/6)/(
    cos(pi/6) + cos(pi - pi/6 - pi/1.5)
  )/cos(pi/6)


  # Creating viewport with fixed aspectt ratio ----------------------------------
  netenv$xlim <- range(netenv$layout[,1], na.rm=TRUE)
  netenv$ylim <- range(netenv$layout[,2], na.rm=TRUE)

  # Creating layout
  # Solution from this answer https://stackoverflow.com/a/48084527
  asp <- dev_size()

  lo  <- grid::grid.layout(
    widths  = grid::unit(1, "null"),
    heights = grid::unit(asp[2]/asp[1], "null"),
    respect = TRUE # This forcces it to the aspectt ratio
    )

  vp_graph <- grid::viewport(
    xscale         = netenv$xlim + .04*diff(netenv$xlim)*c(-1,1),
    yscale         = netenv$ylim + .04*diff(netenv$ylim)*c(-1,1),
    clip           = "off",
    layout.pos.col = 1,
    layout.pos.row = 1,
    name           = "graph-vp"
  )

  # Generating grobs -----------------------------------------------------------
  if (!skip.vertex) {
    grob.vertex <- vector("list", netenv$N)
    for (v in 1:netenv$N)
      grob.vertex[[v]] <- grob_vertex(netenv, v)
  } else
    grob.vertex <- NULL

  if (!skip.edges || !skip.arrows) {

    grob.edge <- vector("list", netenv$M)
    for (e in 1:netenv$M)
      grob.edge[[e]] <- grob_edge(netenv, e)

  } else
    grob.edge <- NULL
  # Agregated grob -------------------------------------------------------------
  ans <- do.call(
    grid::gTree,
    list(
      children = do.call(grid::gList, c(grob.edge, grob.vertex)),
      name     = "graph",
      vp       = grid::vpTree(
        parent   = grid::viewport(layout = lo, name = "frame-vp"),
        children = grid::vpList(vp_graph)
      )
    )
  )

  # Changing the plotting order
  ans$childrenOrder <- with(
    ans,
    c(
      childrenOrder[grepl("^edge.[0-9]+[-][0-9]+$", childrenOrder)],
      childrenOrder[grepl("^vertex[.][0-9]+$", childrenOrder)]
    ))

  # Adding background
  background <- grid::rectGrob(
    gp = grid::gpar(
      fill = bg.col,
      col  = if (inherits(bg.col, c("GridRadialGradient", "GridPattern"))) {
        grDevices::adjustcolor("gray", red.f = .75, green.f = .75, blue.f = .75)
      } else {
        grDevices::adjustcolor(bg.col, red.f = .75, green.f = .75, blue.f = .75)
      }
    ),
    name = "background",
    vp   = grid::viewport(name = "background-vp")
  )

  ans <- grid::gTree(
    children     = grid::gList(background, ans),
    name         = netplot_name$new(),
    .xlim        = netenv$xlim,
    .ylim        = netenv$ylim,
    .layout      = netenv$layout,
    .edgelist    = netenv$edgelist,
    .N           = netenv$N,
    .M           = netenv$M,
    .graph       = netenv$x,
    .graph_class = netenv$graph_class
  )

  class(ans) <- c("netplot", class(ans))

  # Passing edge color
  if (!skip.vertex && length(vertex.color)) {

    ans <- set_vertex_gpar(
      x       = ans,
      element = "core",
      fill    = vertex.color,
      col     = vertex.color
    )

    vertex.color <- get_vertex_gpar(
      x = ans, element = "core", "fill"
      )$fill

  }

  if (!skip.vertex && !length(vertex.frame.color)) {

    if (inherits(vertex.color, "character")) {

      vertex.frame.color <- grDevices::adjustcolor(
        vertex.color,
        red.f = .75, green.f = .75, blue.f = .75
        )

    } else
      vertex.frame.color <- .rep("darkgray", N)

  }

  if (!skip.vertex)
    ans <- set_vertex_gpar(
      x       = ans,
      element = "frame",
      fill    = vertex.frame.color,
      col     = vertex.frame.color
    )

  if (!skip.edges && length(edge.color)) {

    ans <- set_edge_gpar(x = ans, element = "line", col = edge.color)

    if (!skip.arrows) {

      gp <- get_edge_gpar(x = ans, element = "line", "col", simplify=FALSE)$col
      gp <- sapply(seq_along(gp), function(i) gp[[i]][netenv$edge.line.breaks[i]])
      ans <- set_edge_gpar(x = ans, "arrow", fill = gp, col=gp)

    }
  }

  # Explicitly set edge line widths to ensure edge.width is applied
  if (!skip.edges) {
    ans <- set_edge_gpar(
      x       = ans,
      element = "line",
      lwd     = as.vector(netenv$edge.width)
    )
  }

  if (!skip.arrows) {
    ans <- set_edge_gpar(
      x       = ans,
      element = "arrow",
      lwd     = as.vector(netenv$edge.width)
    )
  }

  ans

}

#' Tracker of last printed (plotted) values of `netplot`
#' @noRd
.Last.netplot <- eval({

  env       <- new.env(parent = emptyenv())
  env$value <- NULL

  list(
    get = function() {
      env$value
    },
    set = function(x) {
      env$value <- x
      invisible()
    }
  )

})


#' @rdname nplot
#' @param legend Logical scalar. When `TRUE` it adds a legend.
#' @export
#' @param newpage Logical scalar. When `TRUE` calls [grid::grid.newpage].
#' @param y,... Ignored
print.netplot <- function(x, y = NULL, newpage = TRUE, legend = TRUE, ...) {

  # If legend, then we avoid drawing twice
  if (length(x$.legend_vertex_fill) && legend) {

    color_nodes_legend(x)
    return(invisible(x))

  }

  # Drawing
  if (newpage) {
    grid::grid.newpage()
  }

  grid::grid.draw(x)

  # Storing the value
  .Last.netplot$set(x)

  # Returning
  invisible(x)

}

#' Find a vertex in the current plot
#'
#' This function is a wrapper of [grid::grid.locator()], and provides a way to
#' find the coordinates of a vertex in the current plot. It is useful to
#' identify the vertex that is being clicked in a plot.
#'
#' @param x An object of class `netplot`
#' @return A list with the name of the vertex, the x and y coordinates and the
#'  viewport where it is located.
#' @details This function only works in interactive mode. Once it is called,
#' the user can click on a vertex in the plot. The function will return the
#' name of the vertex, the x and y coordinates and the viewport where it is
#' located. If `x` is not specified, the last plotted `netplot` object will be
#' used.
#'
#' @export
#' @examples
#' library(igraph)
#' library(netplot)
#' set.seed(1)
#' x <- sample_smallworld(1, 200, 5, 0.03)
#'
#' # Plotting
#' nplot(x)
#'
#' # Clicking (only works in interactive mode)
#' if (interactive()) {
#'  res <- locate_vertex()
#'  print(res)
#' }
#'
locate_vertex <- function(x = NULL) {

  if (is.null(x))
    x <- .Last.netplot$get()

  # Trying to fecth the
  on.exit(grid::upViewport(0L))
  res <- tryCatch(grid::downViewport("graph-vp"), error = function(e) e)

  if (inherits(res, "error"))
    stop("Couldn't find the network region. Perhaps there's none?", call. = FALSE)

  loc <- grid::grid.locator()
  loc <- as.vector(unlist(loc))
  loc <- matrix(loc, ncol = 2, nrow = x$.N, byrow = TRUE)

  # Which is the closests one
  v <- which.min(abs(x$.layout - loc))

  list(
    name = sprintf("vertex.%i", v),
    x    = loc[1,1],
    y    = loc[1,2],
    vp   = grid::current.viewport()
    )

}


# Look at `chull` from `grDevices`

#' n-sided polygons
#' Calculate the coordinates for an nsided polygon
#' @param x,y Numeric scalar. Origin of the polygon.
#' @param n Integer scalar. Number of sides.
#' @param r Numeric scalar. Radious of the polygon.
#' @param d Numeric scalar. Starting degree in radians.
#' @return
#' A two column matrix with the coordinates to draw a n sided polygon.
#' @examples
#' graphics.off()
#' oldpar <- par(no.readonly = TRUE)
#'
#' par(xpd = NA, mfrow = c(3, 3), mai = rep(0, 4))
#' for (n in c(2, 3, 4, 5, 6, 8, 12, 20, 50)) {
#'
#'   plot.new()
#'   plot.window(c(-1.25,1.25), c(-1.25,1.25))
#'
#'   for (i in seq(1, .0005, length.out = 200)) {
#'     col <- adjustcolor("tomato", alpha.f = i)
#'     polygon(npolygon(x=(i-1)/4, y = (i-1)/4, r = i, d = i-1, n = n),
#'             col = NA, border=col)
#'   }
#'
#'   mtext(sprintf("n = %i", n), side = 1, line = -3)
#' }
#'
#' par(oldpar)
#' @export
npolygon <- function(
  x = 0,
  y = 0,
  n = 6L,
  r = 1.0,
  d = 2.0*pi/(n)/2
) {

  deg <- seq(d, 2.0*pi + d, length.out = n + 1)[-(n+1)]

  coords <- cbind(
    x = cos(deg)*r + x,
    y = sin(deg)*r + y
  )

}


#' Function to rescale a polygon such that it keeps the "original" aspect ratio.
#' @param coordinates A two-column matrix or data frame with coordinates (xy).
#' @param yorigin Reference point in the y-axis.
#' @param adj Rescale factor (scalar). By default the adjustment is computed
#' using `graphics::par()` `usr` and `pin`.
# @export
#' @noRd
rescale_polygon <- function(
  coordinates,
  yorigin = mean(coordinates[,2]),
  adj = NULL
) {

  # Adjustment value
  if (!length(adj)) {
    usr_adj <- with(graphics::par(), (usr[2] - usr[1])/(usr[4] - usr[3]))
    dev_adj <- with(graphics::par(), pin[2]/pin[1])
    adj <- 1/dev_adj/usr_adj
  }

  # If it is multiple polygons (adj is passed by scoping)
  if (!is.data.frame(coordinates) && is.list(coordinates))
    return(mapply(rescale_polygon, coordinates=coordinates, yorigin=yorigin,
                  SIMPLIFY = FALSE))

  # Adjusting
  if ((is.matrix(coordinates) | is.data.frame(coordinates)) && ncol(coordinates) > 1)
    coordinates[,2] <- yorigin + (coordinates[,2] - yorigin)*adj
  else
    return(yorigin + (coordinates - yorigin)*adj)

  coordinates
}

#' Add legend to a netplot object
#'
#' Legends in [grid] graphics is a bit more complicated than in base graphics.
#' The function `nplot_legend` is a wrapper of [grid::legendGrob()] that makes
#' the process easier. Besides `labels`, the main visual arguments for the
#' figure ar passed through the `gp` argument (see examples).
#'
#' @param g An object of class [netplot].
#' @param labels Character vector of labels.
#' @param pch See [graphics::points()].
#' @param gp An object of class [grid::gpar()]
#' @param ... Further arguments passed to [grid::legendGrob()].
#' @param packgrob.args List of arguments passed to [grid::packGrob()].
#' @param newpage Logical scalar. When `TRUE` it calls [grid::grid.newpage()].
#' @return A frame grob.
#' @export
#' @examples
#' library(igraph)
#' library(netplot)
#' set.seed(1)
#' x <- sample_smallworld(1, 200, 5, 0.03)
#' V(x)$nsides <- sample(c(10, 4), 200, replace = TRUE)
#'
#' g <- nplot(
#'   x,
#'   vertex.nsides = V(x)$nsides,
#'   vertex.color  = ifelse(V(x)$nsides == 4, "red", "steelblue"),
#'   edge.line.breaks = 5
#'   )
#'
#' nplot_legend(
#'   g,
#'   labels = c("circle", "diamond", "edge"),
#'   pch    = c(21, 23, NA),
#'   gp     = gpar(
#'     fill = c("steelblue", "red", NA),
#'     lwd  = c(NA, NA, 1),
#'     col  = c(NA, NA, "purple")
#'     )
#'   )
#' grid.text("Legend to the left (default)", y = unit(.95, "npc"), just = "bottom")
#'
#' nplot_legend(
#'   g,
#'   labels = c("circle", "diamond", "edge"),
#'   pch    = c(21, 23, NA),
#'   gp     = gpar(
#'     fill = c("steelblue", "red", NA),
#'     lwd  = c(NA, NA, 1),
#'     col  = c(NA, NA, "purple")
#'     ),
#'   # These two extra options set the legend to the bottom
#'   packgrob.args = list(side = "bottom"),
#'   ncol = 3
#'   )
#' grid.text("Legend bottom", y = unit(.95, "npc"), just = "bottom")
#'
#'
nplot_legend <- function(
    g,
    labels,
    pch,
    gp = grid::gpar(),
    ...,
    packgrob.args = list(side = "left")
    ) {

  # Creating the new frame
  gf <- grid::packGrob(grid::frameGrob(), g)

  # Adding the legend
  legend.args <- do.call(
    grid::legendGrob,
    c(list(labels = labels, pch = pch, gp = gp), list(...))
    )

  packgrob.args <- c(list(frame = gf, grob = legend.args), packgrob.args)

  gf <- do.call(grid::packGrob, packgrob.args)

  # Setting the right names
  gf$children[[1]] <- editGrob(
    gf$children[[1]],
    name = grid::grobName(gf$children[[1]], "graph")
    )

  gf$children[[2]] <- editGrob(
    gf$children[[2]],
    name = grid::grobName(gf$children[[2]], "legend")
  )

  structure(
    gf,
    class = c("netplot_legend", class(gf))
  )

}

#' @export
#' @param x An object of class `netplot_legend`.
#' @param y Ignored.
#' @rdname nplot_legend
print.netplot_legend <- function(x, y = NULL, newpage = TRUE, ...) {

  if (newpage) {
    grid::grid.newpage()
  }

  grid::grid.draw(x)

  invisible(x)

}

#' Extract a graph attribute
#' @param graph A graph object of class igraph or network.
#' @param attribute A character string specifying the name of the attribute.
#' @return A vector of the attribute values. If the attribute does not exist, an error is thrown.
#' @keywords internal
#' @noRd
get_vertex_attribute <- function(graph, attribute) UseMethod("get_vertex_attribute")

#' @keywords internal
get_vertex_attribute.igraph <- function(graph, attribute) {

  # Check if the attribute exists
  if (!(attribute %in% igraph::vertex_attr_names(graph))) {
    stop("Attribute does not exist in graph")
  }

  # Extract the attribute
  igraph::vertex_attr(graph, name = attribute)

}

#' @keywords internal
get_vertex_attribute.network <- function(graph, attribute) {

  # Check if the attribute exists
  if (!(attribute %in% network::list.vertex.attributes(graph))) {
    stop("Attribute does not exist in graph")
  }

  # Extract the attribute
  network::get.vertex.attribute(graph, attribute)

}

#' @keywords internal
get_vertex_attribute.default <- function(graph, attribute) {

  stop("Graph type not supported")

}

#' @keywords internal
get_edge_attribute <- function(graph, attribute) UseMethod("get_edge_attribute")

#' @keywords internal
get_edge_attribute.igraph <- function(graph, attribute) {

  # Check if the attribute exists
  if (!(attribute %in% igraph::edge_attr_names(graph))) {
    stop("Attribute does not exist in graph")
  }

  # Extract the attribute
  igraph::edge_attr(graph, name = attribute)

}

#' @keywords internal
get_edge_attribute.network <- function(graph, attribute) {

  # Check if the attribute exists
  if (!(attribute %in% network::list.edge.attributes(graph))) {
    stop("Attribute does not exist in graph")
  }

  # Extract the attribute
  network::get.edge.attribute(graph, attribute)

}

#' Retrieve all vertex/edge attributes of a graph as a named list
#'
#' Used as the data environment when evaluating attribute formulas (e.g.
#' `~ log(weight)`).
#' @param graph A graph object of class `igraph` or `network`.
#' @param type Either `"vertex"` or `"edge"`.
#' @return A named list, one element per attribute.
#' @noRd
all_graph_attributes <- function(graph, type = c("vertex", "edge")) {

  type <- match.arg(type)

  if (inherits(graph, "igraph")) {

    if (type == "vertex")
      igraph::vertex_attr(graph)
    else
      igraph::edge_attr(graph)

  } else if (inherits(graph, "network")) {

    nms <- if (type == "vertex")
      network::list.vertex.attributes(graph)
    else
      network::list.edge.attributes(graph)

    getter <- if (type == "vertex")
      network::get.vertex.attribute
    else
      network::get.edge.attribute

    stats::setNames(lapply(nms, function(a) getter(graph, a)), nms)

  } else {

    stop(
      "Attribute formulas are only supported for 'igraph' and 'network' ",
      "objects (got '", class(graph)[1], "').",
      call. = FALSE
    )

  }

}

#' Evaluate a one-sided formula against a graph's vertex/edge attributes
#'
#' The right-hand side of the formula is evaluated with the graph's attributes
#' available as variables, so both bare attribute names (`~ weight`) and
#' expressions (`~ log(weight)`, `~ 2 * weight`) are supported.
#' @param graph A graph object of class `igraph` or `network`.
#' @param formula A one-sided formula.
#' @param type Either `"vertex"` or `"edge"`.
#' @return The evaluated vector.
#' @noRd
eval_attribute_formula <- function(graph, formula, type = c("vertex", "edge")) {

  type   <- match.arg(type)
  attrs  <- all_graph_attributes(graph, type)
  rhs    <- formula[[length(formula)]]
  enclos <- environment(formula)
  if (is.null(enclos))
    enclos <- parent.frame()

  tryCatch(
    eval(rhs, envir = attrs, enclos = enclos),
    error = function(e) stop(
      "Could not evaluate the ", type, " formula `", deparse(rhs), "`. ",
      "Make sure the ", type, " attribute(s) it references exist in the graph. ",
      "Available ", type, " attributes: ",
      if (length(attrs)) paste(names(attrs), collapse = ", ") else "(none)",
      ". Original error: ", conditionMessage(e),
      call. = FALSE
    )
  )

}

#' @keywords internal
get_edge_attribute.default <- function(graph, attribute) {

  stop("Graph type not supported")

}

#' A faster implementation of [grDevices::colorRamp] for linear interpolation.
#'
#' @param x A vector of colors.
#' @param alpha Logical scalar. When `TRUE`
#' This implementation of `colorRamp` can be 2 or more times faster than the `grDevices`
#' version. It is intended for consecutive calls (i.e. in a loop) to improve
#' performance. It is equivalent to the linear interpolation of the function
#' `colorRamp`.
#' @param thresholds A numeric vector of length `length(x)`. Optional threshold
#' levels so that the mixing can be different that even.
#' @export
#' @return A function as in [grDevices::colorRamp].
#' @examples
#'
#' # Creating a function for 2 colors
#' myf <- colorRamp2(c("black", "steelblue"))
#' f   <- colorRamp(c("black", "steelblue"))
#'
#' plot.new()
#' plot.window(xlim = c(0,2), ylim = c(1, 11))
#'
#' # These should be the same colors
#' rect(
#'   xleft   = 0,
#'   xright  = 1,
#'   ybottom = 1:10,
#'   ytop    = 2:11,
#'   col = rgb(myf((1:10)/10), maxColorValue = 255)
#'   )
#' rect(
#'   xleft   = 1,
#'   xright  = 2,
#'   ybottom = 1:10,
#'   ytop    = 2:11,
#'   col = rgb(f((1:10)/10), maxColorValue = 255)
#' )
#'
#' # Another example setting different thresholds
#' myf  <- colorRamp2(c("black", "steelblue"))
#' myf2 <- colorRamp2(c("black", "steelblue"), thresholds=c(0, .7))
#'
#' plot.new()
#' plot.window(xlim = c(0,2), ylim = c(1, 11))
#'
#' # These should be the same colors
#' rect(
#'   xleft   = 0,
#'   xright  = 1,
#'   ybottom = 1:10,
#'   ytop    = 2:11,
#'   col = rgb(myf((1:10)/10), maxColorValue = 255)
#'   )
#' rect(
#'   xleft   = 1,
#'   xright  = 2,
#'   ybottom = 1:10,
#'   ytop    = 2:11,
#'   col = rgb(myf2((1:10)/10), maxColorValue = 255)
#' )
#'
#'
#'
colorRamp2 <- function(x, alpha = TRUE, thresholds=NULL) {

  # Converting to RGB
  col <- t(grDevices::col2rgb(col = x, alpha = alpha))

  # Defining thresholds for the linear combinations
  n <- length(x)

  if (!length(thresholds))
    thresholds <- seq(0, 1, length.out = n)
  else if (length(thresholds) != n)
    stop("The length of `thresholds` should be the same as `length(n)`.",
         call. = FALSE)

  if (n == 2L) {
    function(x) {
      N    <- length(x)
      col0 <- rep(1, N)
      x    <- (x - thresholds[1])/(thresholds[2] - thresholds[1])
      x[x<0] <- 0
      x[x>1] <- 1

      col[col0, ,drop=FALSE] * (1-x) + col[col0+1, ,drop=FALSE] * x
    }
  } else
    function(x) {

      # Which colors

      col0 <- findInterval(x, thresholds)
      col1 <- col0 + 1*(col0 < n)

      # Adjusting levels
      x <- (x - thresholds[col0])/(thresholds[col1] - thresholds[col0] + 1e-20)

      col[col0, , drop=FALSE] * (1-x) +
        col[col1, , drop=FALSE] * x

    }

}

alphacolor <- function(col, alpha.f) {
  val <- strtoi(substring(col, 8L), base = 16L)
  substring(col, 8) <- sprintf("%02X", floor(val * alpha.f))
  return(col)
}

# microbenchmark::microbenchmark(
#   alphacolor("#0000FF80", .5),
#   adjustcolor("#0000FF80", .5), times = 1e3, check = "identical"
# )

#' Draw segments colored by gradients
#' @param x,y Coordinates passed to [grDevices::xy.coords].
#' @param col Color ramp function (see [grDevices::colorRamp]).
#' @param lend Passed to [graphics::segments].
#' @param ... Further arguments passed to `segments`.
#' @export
#' @examples
#'
#' set.seed(1)
#' x <- cbind(cumsum(rnorm(1e3, sd=.1)), cumsum(rnorm(1e3, sd=.4)))
#' plot(x, type="n")
#' segments_gradient(x)
#' @return See [graphics::segments].
segments_gradient <- function(
  x, y = NULL,
  col  = colorRamp2(c("transparent", "black"), TRUE),
  lend = 1,
  ...) {

  # Getting new coords
  x   <- grDevices::xy.coords(x, y)
  n   <- length(x$x) - 1L
  col <- col(0:(n-1)/(n-1))
  col <- rgb(col, alpha=col[,4], maxColorValue = 255)

  graphics::segments(
    x0 = x$x[1:n],
    y0 = x$y[1:n],
    x1 = x$x[2:(n+1)],
    y1 = x$y[2:(n+1)],
    col = col,
    lend = lend,
    ...
  )

}

  sides_lookup <- list(
    line     = list(sides = 2, rot = 0),    # 1
    triangle = list(sides = 3, rot = 0),    # 2
    square   = list(sides = 4, rot = pi/4), # 3
    diamond  = list(sides = 4, rot = 0),    # 4
    pentagon = list(sides = 5, rot = 0),    # 5
    hexagon  = list(sides = 6, rot = 0),    # 6
    heptagon = list(sides = 7, rot = 0),    # 7
    octagon  = list(sides = 8, rot = 0),    # 8
    circle   = list(sides = 25, rot = 0)    # 9
  )

resolve_vertex_shape <- function(nsides, rot) {

  if (!is.numeric(nsides) & !is.character(nsides)) {
    stop("vertex.nsides must be numeric or character")
  }

  rot <- as.numeric(rot)

  if (is.character(nsides)) {

    shape <- nsides

    if (shape %in% names(sides_lookup)) {
      info <- sides_lookup[[shape]]
      return(list(sides = info$sides, rot = info$rot + rot))
    } else {
      stop("Invalid shape name: ", shape)
    }

  }

  list(sides = nsides, rot = rot)

}

#' Functions to calculate graph polygons coordinates
#' @param netenv An object of class network environment.
#' @param v,e Integer scalars. vertex or edge index.
#'
#' @return A grob
#' Generating coordinates
#' @noRd
grob_vertex <- function(netenv, v) {

  # # Add formula handling
  # if(inherits(netenv$vertex.nsides, "formula")) {
  #   var_name <- all.vars(netenv$vertex.nsides)[1]
  #   netenv$vertex.nsides <- eval(netenv$vertex.nsides, envir = data)
  # }

  vertex_shape <- resolve_vertex_shape(
    nsides = netenv$vertex.nsides[v],
    rot    = netenv$vertex.rot[v]
  )

  if (netenv$skip.vertex)
    return(
      grid::gTree(
        children = grid::gList(
          grid::grob(name = "frame"),
          grid::grob(name = "core"),
          grid::grob(name = "label")
        ),
        name = netplot_name$make(v)
      )
    )

  # Computing coordinates
  coords <- npolygon(
    x = netenv$layout[v, 1],
    y = netenv$layout[v, 2],
    n = as.integer(vertex_shape$sides),
    r = netenv$vertex.size[v]*(1 - netenv$vertex.frame.prop[v]),
    d = vertex_shape$rot
  )

  # Frame coordinates
  framecoords <- npolygon(
    x = netenv$layout[v, 1],
    y = netenv$layout[v, 2],
    n = as.integer(vertex_shape$sides),
    r = netenv$vertex.size[v],
    d = vertex_shape$rot
  )

  # Create color palette
  nsides <- unique(netenv$vertex.nsides)
  ncolors <- length(nsides)
  colors <- grDevices::hsv(h = seq(0, 1, length.out = ncolors), v = 1, a = 1)
  pal <- stats::setNames(colors, nsides)

  # Lookup color for this vertex based on number of sides
  col <- pal[as.character(netenv$vertex.nsides[v])]

  # Returning
  ans <- grid::polygonGrob(
    x = c(coords[,1]),
    y = c(coords[,2]),
    gp = grid::gpar(
      fill = col,
      col = col
    ),
    default.units = "native",
    name = "core"
  )

  ans <- grid::grobTree(
    grid::polygonGrob(
      x = framecoords[,1],
      y = framecoords[,2],
      gp = grid::gpar(
        fill = col,
        col = col
      ),
      default.units = "native",
      name = "frame"
    ),
    ans,
    name = netplot_name$make(v)
  )
# If the users is drawing text
  if (length(netenv$vertex.label) && !is.na(netenv$vertex.label[v])) {

    # Only if it is big enough
    if (netenv$label_threshold <= netenv$vertex.size[v]) {

      ans <- grid::addGrob(
        ans,
        grid::textGrob(
          label = netenv$vertex.label[v],
          x     = netenv$layout[v, 1],
          y     = netenv$layout[v, 2] + netenv$vertex.size[v]*1.1,
          gp    = grid::gpar(
            fontsize   = netenv$vertex.label.fontsize[v],
            col        = netenv$vertex.label.color[v],
            fontfamily = netenv$vertex.label.fontfamily[v],
            fontface   = netenv$vertex.label.fontface[v]
          ),
          vjust         = 0,
          default.units = "native",
          name          = "label",
          check.overlap = TRUE
        )
      )

    }
  }
  ans
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----pkgs-and-data, message=FALSE, warning=FALSE------------------------------
library(netplot)
library(igraph)
library(sna)
library(ggraph)

# We will use the UKfaculty data from the igraphdata package
data("UKfaculty", package = "igraphdata")

# With a fixed layout
set.seed(225)
l_ukf <- layout_with_fr(UKfaculty)

## -----------------------------------------------------------------------------
# Function to map base to grid
map_base_to_grid <- function(fun) {
  gridGraphics::grid.echo(fun)
  grid::grid.grab()
}

## ----igraph-sna, fig.keep='none'----------------------------------------------
# Mapping base to grid
ig <- map_base_to_grid(function() plot(UKfaculty, main = "igraph"))
nw <- map_base_to_grid(function() gplot(as.matrix(as_adj(UKfaculty)), coord = l_ukf, main = "sna"))

## ----nplot, fig.keep='none'---------------------------------------------------
nplot(UKfaculty, layout = l_ukf)
grid::grid.text("netplot", x = .05, y=.9)
np <- grid::grid.grab()

## ----fig.width=7, fig.height=5, fig.keep='none'-------------------------------
ggraph::ggraph(UKfaculty, layout = l_ukf) +
  ggraph::geom_edge_arc() +
  ggraph::geom_node_point() +
  ggplot2::ggtitle("ggraph")

gg <- grid::grid.grab()

## ----fig.width=7, fig.height=7, fig.cap="Comparison of default `igraph`, `sna`, `ggraph`, and `netplot` default call. `nplot` fills completely the plotting area, and adjusts vertex size, edge width, and edge arrows' size accordingly to the plotting area and plotting device."----
# Putting all together
gridExtra::grid.arrange(ig, nw, gg, np, nrow=2, ncol=2)

## -----------------------------------------------------------------------------
x_igraph  <- sample_smallworld(1, 20, 4, .05)
x_network <- intergraph::asNetwork(x_igraph)

l <- layout_nicely(x_igraph)

# Putting two plots in the same page (one using igraph and the other network)
gridExtra::grid.arrange(
  nplot(x_igraph, layout = l),
  nplot(x_network, layout = l), ncol=2, nrow=1
)

## ----node-scaling, fig.width=7, fig.height=7, fig.cap="Modifying `vertex.size.range`: use `NULL` to keep the supplied vertex sizes unchanged."----

gridExtra::grid.arrange(
  nplot(UKfaculty, layout = l_ukf,
        vertex.size = rep(.025, vcount(UKfaculty)),
        vertex.size.range = NULL),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025, 4)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.02, .05, 4)),
  ncol=2, nrow=2
)



## ----node-shapes, fig.width=4, fig.height=4, fig.cap="Number of sides for node drawing."----
nplot(make_ring(5), vertex.nsides = 2:6,
      vertex.size.range = c(.1,.1),
      edge.curvature = 0)

## ----sim-net------------------------------------------------------------------
set.seed(12233)

x <- sample_smallworld(1, size = 10, nei = 2, .1)
l <- layout_with_fr(x)

## ----edge-color, fig.width=7, fig.height=3, fig.cap="Modifying `edge.color.mix`: Each figure shows a different parameter for the edge color mixer. From left to right, (a) colors the edges as alter, (b) mixes ego and alter's colors, and (c) only uses ego"----

gridExtra::grid.arrange(
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=0, alpha = .1, col="black") + alter(mix=1),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=.5, alpha = .1, col="black") + alter(mix=.5),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=1, alpha = .1, col="black") + alter(mix=0),
    vertex.size.range = c(.05,.1)
    ),
  ncol=3, nrow=1
)


## ----edge-curvature, fig.width=7, fig.height=3, fig.cap="Modifying `edge.curvature`: Each figure shows a different parameter for the edge curvature. From left to right, (a) straight edges, (b) the edge between ego and alter is an arc that measures $\\pi/2$ radians (90 degree), and (c) the edge as an arc between ego and alter that measures $\\pi$ radians (180 degrees)."----
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.curvature = 0, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi/2, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi, vertex.size.range = c(.05,.1)),
  ncol = 3, nrow=1
)

## ----edge-line-type, fig.width=6, fig.height=6, fig.cap="Changing the number of breaks in the edge (arc) and the type of line to be drawn."----
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.line.breaks=3),
  nplot(x, layout = l, edge.line.lty = 2, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 3, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 4, edge.line.breaks=10),
  nrow=2, ncol=2
)

## ----edge-width-setup---------------------------------------------------------
set.seed(1)
x_w <- make_ring(8)

# Assign varying edge weights
E(x_w)$weight <- c(1, 3, 1, 5, 2, 4, 1, 6)
l_w <- layout_in_circle(x_w)

## ----edge-width, fig.width=7, fig.height=3, fig.cap="Effect of `edge.width` and `edge.width.range`. Left: uniform width. Middle: weights mapped to 1–2 pt. Right: raw weights with `edge.width.range = NULL`."----
gridExtra::grid.arrange(
  nplot(x_w, layout = l_w, skip.arrows = TRUE,
        vertex.size.range = c(.05, .05),
        edge.width = 1,
        edge.width.range = c(1, 2)),
  nplot(x_w, layout = l_w, skip.arrows = TRUE,
        vertex.size.range = c(.05, .05),
        edge.width = E(x_w)$weight,
        edge.width.range = c(1, 2)),
  nplot(x_w, layout = l_w, skip.arrows = TRUE,
        vertex.size.range = c(.05, .05),
        edge.width = E(x_w)$weight,
        edge.width.range = NULL),
  ncol = 3, nrow = 1
)

## ----edge-width-set-gpar, fig.width=7, fig.height=3, fig.cap="Using `set_edge_gpar()` to change edge widths after plotting. Left: all edges set to 1 pt. Right: all edges set to 4 pt."----
g <- nplot(x_w, layout = l_w, skip.arrows = TRUE,
           vertex.size.range = c(.05, .05))

gridExtra::grid.arrange(
  set_edge_gpar(g, element = "line", lwd = 1),
  set_edge_gpar(g, element = "line", lwd = 4),
  ncol = 2
)

---
title: "Graph drawing with netplot"
author: "George G. Vega Yon"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Graph drawing with netplot}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Some features:

1.  Auto-scaling of vertices using sizes relative to the plotting device.
2.  Embedded edge color mixer.
3.  True curved edges drawing.
4.  User-defined edge curvature.
5.  Nicer vertex frame color.
6.  Better use of space filling the plotting device.


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

```{r pkgs-and-data, message=FALSE, warning=FALSE}
library(netplot)
library(igraph)
library(sna)
library(ggraph)

# We will use the UKfaculty data from the igraphdata package
data("UKfaculty", package = "igraphdata")

# With a fixed layout
set.seed(225)
l_ukf <- layout_with_fr(UKfaculty)
```

Since igraph and statnet do the plotting using base system graphics, we can't put everything in the same device right away. Fortunately, the `gridGraphics` R package allows us to reproduce base graphics using the `grid` system, which, in combination with the function `gridExtra::grid.arrange` will let us put both base and grid graphics in the same page. First me write a func

```{r}
# Function to map base to grid
map_base_to_grid <- function(fun) {
  gridGraphics::grid.echo(fun)
  grid::grid.grab()
}
```


```{r igraph-sna, fig.keep='none'}
# Mapping base to grid
ig <- map_base_to_grid(function() plot(UKfaculty, main = "igraph"))
nw <- map_base_to_grid(function() gplot(as.matrix(as_adj(UKfaculty)), coord = l_ukf, main = "sna"))
```

```{r nplot, fig.keep='none'}
nplot(UKfaculty, layout = l_ukf)
grid::grid.text("netplot", x = .05, y=.9)
np <- grid::grid.grab()
```

Here is an example with [`ggraph`](https://cran.r-project.org/package=ggraph)

```{r, fig.width=7, fig.height=5, fig.keep='none'}
ggraph::ggraph(UKfaculty, layout = l_ukf) +
  ggraph::geom_edge_arc() +
  ggraph::geom_node_point() +
  ggplot2::ggtitle("ggraph")

gg <- grid::grid.grab()
```


```{r, fig.width=7, fig.height=7, fig.cap="Comparison of default `igraph`, `sna`, `ggraph`, and `netplot` default call. `nplot` fills completely the plotting area, and adjusts vertex size, edge width, and edge arrows' size accordingly to the plotting area and plotting device."}
# Putting all together
gridExtra::grid.arrange(ig, nw, gg, np, nrow=2, ncol=2)
```


# Multiple plots

To arrange multiple plots in the same page we can use the `grid.arrange` function from the `gridExtra` package

```{r}
x_igraph  <- sample_smallworld(1, 20, 4, .05)
x_network <- intergraph::asNetwork(x_igraph)

l <- layout_nicely(x_igraph)

# Putting two plots in the same page (one using igraph and the other network)
gridExtra::grid.arrange(
  nplot(x_igraph, layout = l),
  nplot(x_network, layout = l), ncol=2, nrow=1
)
```


# Nodes

```{r node-scaling, fig.width=7, fig.height=7, fig.cap="Modifying `vertex.size.range`: Each figure shows a different parameter for the vertex size range. From left to right, and top down: (a) Has all vertices with the same scale of 2.5%, (b) "}

gridExtra::grid.arrange(
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.025, .025)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025, 4)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.02, .05, 4)),
  ncol=2, nrow=2
)


```

```{r node-shapes, fig.width=4, fig.height=4, fig.cap="Number of sides for node drawing."}
nplot(make_ring(5), vertex.nsides = 2:6,
      vertex.size.range = c(.1,.1),
      edge.curvature = 0)
```

# Edges

```{r sim-net}
set.seed(12233)

x <- sample_smallworld(1, size = 10, nei = 2, .1)
l <- layout_with_fr(x)
```


```{r edge-color, fig.width=7, fig.height=3, fig.cap="Modifying `edge.color.mix`: Each figure shows a different parameter for the edge color mixer. From left to right, (a) colors the edges as alter, (b) mixes ego and alter's colors, and (c) only uses ego"}

gridExtra::grid.arrange(
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=0, alpha = .1, col="black") + alter(mix=1),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=.5, alpha = .1, col="black") + alter(mix=.5),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=1, alpha = .1, col="black") + alter(mix=0),
    vertex.size.range = c(.05,.1)
    ),
  ncol=3, nrow=1
)

```

```{r edge-curvature, fig.width=7, fig.height=3, fig.cap="Modifying `edge.curvature`: Each figure shows a different parameter for the edge curvature. From left to right, (a) straight edges, (b) the edge between ego and alter is an arc that measures $\\pi/2$ radians (90 degree), and (c) the edge as an arc between ego and alter that measures $\\pi$ radians (180 degrees)."}
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.curvature = 0, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi/2, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi, vertex.size.range = c(.05,.1)),
  ncol = 3, nrow=1
)
```

```{r edge-line-type, fig.width=6, fig.height=6, fig.cap="Changing the number of breaks in the edge (arc) and the type of line to be drawn."}
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.line.breaks=3),
  nplot(x, layout = l, edge.line.lty = 2, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 3, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 4, edge.line.breaks=10),
  nrow=2, ncol=2
)
```

# Node labels

```{r}
data(UKfaculty, package = "igraphdata")
# fakenames <- sample
```

---
title: "Mixing netplot with base graphics"
author: "George G. Vega Yon"
date: 2023-05-22
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Example with baseplots}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Working with grid graphics provides flexibility that is harder to get when using base graphics. Nevertheless, a lot of users may still be more comfortable with base graphics, so this document shows a few examples of how to mix both of them using the [`gridBase`](https://cran.r-project.org/package=gridBase) package.

First, we load igraph to simulate a small network and complement that with uninteresting correlated data:

```{r data-sim, warning=FALSE, message=FALSE, fig.width = 6, fig.height = 5}
library(igraph)

# Simulation parameters
set.seed(12)
n <- 100

# Points
x0  <- rnorm(n)
x1  <- x0*.4 + rnorm(n)

# network
net <- sample_smallworld(1, n, 4, .1)
```

And here, we can create the `netplot` object. We use the [`magrittr`](https://cran.r-project.org/package=magrittr) package to make the workflow nicer:

```{r netplot, fig.width = 6, fig.height = 5}
library(netplot)
library(magrittr)

np <- nplot(net, bg.col = "gray70", vertex.nsides = "square") %>%
  set_edge_gpar("line", col = "gray50", alpha = .7) %>%
  set_vertex_gpar("frame", fill = "black", col = "black")
```

Imagine that we would like to include a plot of the network in the following plot

```{r baseplot, fig.width = 6, fig.height = 5}
plot(x0, x1)
```
 
We will use the [`gridBase`](https://cran.r-project.org/package=gridBase) package. `gridBase` allows some interaction between the base and grid graphics systems. In particular, we will use the `baseViewports()` function, which returns a list with `viewports` that can be mapped to match the current base plot
 
```r
library(gridBase)
library(grid)

# Getting the current viewports
vp <- baseViewports() 
```

Once we have the viewports, we can work with these to add the current plot to the base plot. In this case, we will include the network that we just created, `np`, and this will be placed on the top left corner of the plot region and will occupy 1/4 of the area:

```r
# Adding the plot to the current view
pushViewport(vp$plot)

# Sub viewport, top-left corner using 1/4 of the area
pushViewport(viewport(.25, .75, width = .5, height = .5)) 

# Drawing, we could have used `print(np, newpage=FALSE)`
grid.draw(np)
```
Which yields

```{r alltogether, echo=FALSE, fig.width = 6, fig.height = 5}
library(gridBase)
library(grid)

# Base plot and views
plot(x0, x1)
vp <- baseViewports() 

# Adding the plot to the current view
pushViewport(vp$plot)
pushViewport(viewport(.25, .75, width = .5, height = .5))
grid.draw(np)
```

Or suppose that you want to frame it into a rectangle to the top-right corner of the entire device that uses 3/10 x 3/10 of the area, then:

```{r topright, fig.width = 6, fig.height = 5}
plot(x0, x1)
# vp <- baseViewports() 
# pushViewport(vp$figure)
pushViewport(viewport(1, 1, just = c("right", "top"), width = .3, height = .3))
grid.draw(np)
```

Notice that we didn't need to extract the viewports of the base plot. A simple viewport did the job since we are using the entire device. Now, imagine that you are feeling playful and would like to draw it a bit tilted, say -20 degrees:


```{r tilted, fig.width = 6, fig.height = 5}
plot(x0, x1)
pushViewport(
  viewport(x = .8, y = .8, width = .3, height = .3, angle = -20)
  )
grid.draw(np)
```

---
title: "Graph drawing with netplot"
author: "George G. Vega Yon"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Graph drawing with netplot}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Some features:

1.  Auto-scaling of vertices using sizes relative to the plotting device.
2.  Embedded edge color mixer.
3.  True curved edges drawing.
4.  User-defined edge curvature.
5.  Nicer vertex frame color.
6.  Better use of space filling the plotting device.


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

```{r pkgs-and-data, message=FALSE, warning=FALSE}
library(netplot)
library(igraph)
library(sna)
library(ggraph)

# We will use the UKfaculty data from the igraphdata package
data("UKfaculty", package = "igraphdata")

# With a fixed layout
set.seed(225)
l_ukf <- layout_with_fr(UKfaculty)
```

Since igraph and statnet do the plotting using base system graphics, we can't put everything in the same device right away. Fortunately, the `gridGraphics` R package allows us to reproduce base graphics using the `grid` system, which, in combination with the function `gridExtra::grid.arrange` will let us put both base and grid graphics in the same page. First me write a func

```{r}
# Function to map base to grid
map_base_to_grid <- function(fun) {
  gridGraphics::grid.echo(fun)
  grid::grid.grab()
}
```


```{r igraph-sna, fig.keep='none'}
# Mapping base to grid
ig <- map_base_to_grid(function() plot(UKfaculty, main = "igraph"))
nw <- map_base_to_grid(function() gplot(as.matrix(as_adj(UKfaculty)), coord = l_ukf, main = "sna"))
```

```{r nplot, fig.keep='none'}
nplot(UKfaculty, layout = l_ukf)
grid::grid.text("netplot", x = .05, y=.9)
np <- grid::grid.grab()
```

Here is an example with [`ggraph`](https://cran.r-project.org/package=ggraph)

```{r, fig.width=7, fig.height=5, fig.keep='none'}
ggraph::ggraph(UKfaculty, layout = l_ukf) +
  ggraph::geom_edge_arc() +
  ggraph::geom_node_point() +
  ggplot2::ggtitle("ggraph")

gg <- grid::grid.grab()
```


```{r, fig.width=7, fig.height=7, fig.cap="Comparison of default `igraph`, `sna`, `ggraph`, and `netplot` default call. `nplot` fills completely the plotting area, and adjusts vertex size, edge width, and edge arrows' size accordingly to the plotting area and plotting device."}
# Putting all together
gridExtra::grid.arrange(ig, nw, gg, np, nrow=2, ncol=2)
```


# Multiple plots

To arrange multiple plots in the same page we can use the `grid.arrange` function from the `gridExtra` package

```{r}
x_igraph  <- sample_smallworld(1, 20, 4, .05)
x_network <- intergraph::asNetwork(x_igraph)

l <- layout_nicely(x_igraph)

# Putting two plots in the same page (one using igraph and the other network)
gridExtra::grid.arrange(
  nplot(x_igraph, layout = l),
  nplot(x_network, layout = l), ncol=2, nrow=1
)
```


# Nodes

```{r node-scaling, fig.width=7, fig.height=7, fig.cap="Modifying `vertex.size.range`: use `NULL` to keep the supplied vertex sizes unchanged."}

gridExtra::grid.arrange(
  nplot(UKfaculty, layout = l_ukf,
        vertex.size = rep(.025, vcount(UKfaculty)),
        vertex.size.range = NULL),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.01, .025, 4)),
  nplot(UKfaculty, layout = l_ukf, vertex.size.range = c(.02, .05, 4)),
  ncol=2, nrow=2
)


```

```{r node-shapes, fig.width=4, fig.height=4, fig.cap="Number of sides for node drawing."}
nplot(make_ring(5), vertex.nsides = 2:6,
      vertex.size.range = c(.1,.1),
      edge.curvature = 0)
```

# Edges

```{r sim-net}
set.seed(12233)

x <- sample_smallworld(1, size = 10, nei = 2, .1)
l <- layout_with_fr(x)
```


```{r edge-color, fig.width=7, fig.height=3, fig.cap="Modifying `edge.color.mix`: Each figure shows a different parameter for the edge color mixer. From left to right, (a) colors the edges as alter, (b) mixes ego and alter's colors, and (c) only uses ego"}

gridExtra::grid.arrange(
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=0, alpha = .1, col="black") + alter(mix=1),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=.5, alpha = .1, col="black") + alter(mix=.5),
    vertex.size.range = c(.05,.1)
    ),
  nplot(
    x, layout          = l,
    edge.color        = ~ego(mix=1, alpha = .1, col="black") + alter(mix=0),
    vertex.size.range = c(.05,.1)
    ),
  ncol=3, nrow=1
)

```

```{r edge-curvature, fig.width=7, fig.height=3, fig.cap="Modifying `edge.curvature`: Each figure shows a different parameter for the edge curvature. From left to right, (a) straight edges, (b) the edge between ego and alter is an arc that measures $\\pi/2$ radians (90 degree), and (c) the edge as an arc between ego and alter that measures $\\pi$ radians (180 degrees)."}
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.curvature = 0, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi/2, vertex.size.range = c(.05,.1)),
  nplot(x, layout = l, edge.curvature = pi, vertex.size.range = c(.05,.1)),
  ncol = 3, nrow=1
)
```

```{r edge-line-type, fig.width=6, fig.height=6, fig.cap="Changing the number of breaks in the edge (arc) and the type of line to be drawn."}
gridExtra::grid.arrange(
  nplot(x, layout = l, edge.line.breaks=3),
  nplot(x, layout = l, edge.line.lty = 2, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 3, edge.line.breaks=10),
  nplot(x, layout = l, edge.line.lty = 4, edge.line.breaks=10),
  nrow=2, ncol=2
)
```

## Edge width

The `edge.width` parameter controls the relative width of each edge. Values are
normalized and mapped to the range `[min, max]` specified by `edge.width.range`
(in points). For `nplot.igraph`, this defaults to the `"weight"` edge attribute
when present. Use `edge.width.range = NULL` to keep supplied widths unchanged.

```{r edge-width-setup}
set.seed(1)
x_w <- make_ring(8)

# Assign varying edge weights
E(x_w)$weight <- c(1, 3, 1, 5, 2, 4, 1, 6)
l_w <- layout_in_circle(x_w)
```

```{r edge-width, fig.width=7, fig.height=3, fig.cap="Effect of `edge.width` and `edge.width.range`. Left: uniform width. Middle: weights mapped to 1–2 pt. Right: raw weights with `edge.width.range = NULL`."}
gridExtra::grid.arrange(
  nplot(x_w, layout = l_w, skip.arrows = TRUE,
        vertex.size.range = c(.05, .05),
        edge.width = 1,
        edge.width.range = c(1, 2)),
  nplot(x_w, layout = l_w, skip.arrows = TRUE,
        vertex.size.range = c(.05, .05),
        edge.width = E(x_w)$weight,
        edge.width.range = c(1, 2)),
  nplot(x_w, layout = l_w, skip.arrows = TRUE,
        vertex.size.range = c(.05, .05),
        edge.width = E(x_w)$weight,
        edge.width.range = NULL),
  ncol = 3, nrow = 1
)
```

You can also modify edge widths after the plot has been created using
`set_edge_gpar()`:

```{r edge-width-set-gpar, fig.width=7, fig.height=3, fig.cap="Using `set_edge_gpar()` to change edge widths after plotting. Left: all edges set to 1 pt. Right: all edges set to 4 pt."}
g <- nplot(x_w, layout = l_w, skip.arrows = TRUE,
           vertex.size.range = c(.05, .05))

gridExtra::grid.arrange(
  set_edge_gpar(g, element = "line", lwd = 1),
  set_edge_gpar(g, element = "line", lwd = 4),
  ncol = 2
)
```

---
title: "Mixing netplot with base graphics"
author: "George G. Vega Yon"
date: 2023-05-22
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Mixing netplot with base graphics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Working with grid graphics provides flexibility that is harder to get when using base graphics. Nevertheless, a lot of users may still be more comfortable with base graphics, so this document shows a few examples of how to mix both of them using the [`gridBase`](https://cran.r-project.org/package=gridBase) package.

First, we load igraph to simulate a small network and complement that with uninteresting correlated data:

```{r data-sim, warning=FALSE, message=FALSE, fig.width = 6, fig.height = 5}
library(igraph)

# Simulation parameters
set.seed(12)
n <- 100

# Points
x0  <- rnorm(n)
x1  <- x0*.4 + rnorm(n)

# network
net <- sample_smallworld(1, n, 4, .1)
```

And here, we can create the `netplot` object. We use the [`magrittr`](https://cran.r-project.org/package=magrittr) package to make the workflow nicer:

```{r netplot, fig.width = 6, fig.height = 5}
library(netplot)
library(magrittr)

np <- nplot(net, bg.col = "gray70", vertex.nsides = "square") %>%
  set_edge_gpar("line", col = "gray50", alpha = .7) %>%
  set_vertex_gpar("frame", fill = "black", col = "black")
```

Imagine that we would like to include a plot of the network in the following plot

```{r baseplot, fig.width = 6, fig.height = 5}
plot(x0, x1)
```
 
We will use the [`gridBase`](https://cran.r-project.org/package=gridBase) package. `gridBase` allows some interaction between the base and grid graphics systems. In particular, we will use the `baseViewports()` function, which returns a list with `viewports` that can be mapped to match the current base plot
 
```r
library(gridBase)
library(grid)

# Getting the current viewports
vp <- baseViewports() 
```

Once we have the viewports, we can work with these to add the current plot to the base plot. In this case, we will include the network that we just created, `np`, and this will be placed on the top left corner of the plot region and will occupy 1/4 of the area:

```r
# Adding the plot to the current view
pushViewport(vp$plot)

# Sub viewport, top-left corner using 1/4 of the area
pushViewport(viewport(.25, .75, width = .5, height = .5)) 

# Drawing, we could have used `print(np, newpage=FALSE)`
grid.draw(np)
```
Which yields

```{r alltogether, echo=FALSE, fig.width = 6, fig.height = 5}
library(gridBase)
library(grid)

# Base plot and views
plot(x0, x1)
vp <- baseViewports() 

# Adding the plot to the current view
pushViewport(vp$plot)
pushViewport(viewport(.25, .75, width = .5, height = .5))
grid.draw(np)
```

Or suppose that you want to frame it into a rectangle to the top-right corner of the entire device that uses 3/10 x 3/10 of the area, then:

```{r topright, fig.width = 6, fig.height = 5}
plot(x0, x1)
# vp <- baseViewports() 
# pushViewport(vp$figure)
pushViewport(viewport(1, 1, just = c("right", "top"), width = .3, height = .3))
grid.draw(np)
```

Notice that we didn't need to extract the viewports of the base plot. A simple viewport did the job since we are using the entire device. Now, imagine that you are feeling playful and would like to draw it a bit tilted, say -20 degrees:


```{r tilted, fig.width = 6, fig.height = 5}
plot(x0, x1)
pushViewport(
  viewport(x = .8, y = .8, width = .3, height = .3, angle = -20)
  )
grid.draw(np)
```

---
title: "Mapping aesthetics with formulas"
author: "George G. Vega Yon"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Mapping aesthetics with formulas}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 6,
  warning = FALSE,
  message = FALSE
)
```

One of the most convenient features of `nplot()` is that many aesthetics can be
mapped **directly from graph attributes** using a one-sided formula, instead of
building the vector of colors, shapes, or sizes by hand. This vignette walks
through the different formula interfaces available in **netplot**.

At a glance, `nplot()` understands two flavors of formula:

| Aesthetic       | Formula                | What it does                                                        |
|-----------------|------------------------|---------------------------------------------------------------------|
| `vertex.color`  | `~ attr`               | Colors vertices by a vertex attribute (categorical, numeric, logical). |
| `vertex.nsides` | `~ attr`               | Maps each unique value of a vertex attribute to a distinct shape.   |
| `vertex.size`   | `~ attr`               | Scales vertex sizes from a numeric vertex attribute.                |
| `edge.width`    | `~ attr`               | Scales edge widths from a numeric edge attribute.                   |
| `edge.color`    | `~ ego(...) + alter(...)` | Blends edge colors from the two endpoints (see the last section).|

```{r pkgs}
library(netplot)
library(igraph)
```

# A working example

We will use the `UKfaculty` network from the **igraphdata** package, a
friendship network among faculty at a UK university. It already carries a
`Group` vertex attribute (the school/department each person belongs to).

```{r data}
data("UKfaculty", package = "igraphdata")

set.seed(225)
l <- layout_with_fr(UKfaculty)

# A couple of extra attributes to play with. We qualify igraph::degree()
# explicitly because other packages (e.g. sna) also define a degree().
V(UKfaculty)$indeg  <- igraph::degree(UKfaculty, mode = "in")
V(UKfaculty)$is_hub <- V(UKfaculty)$indeg > stats::median(V(UKfaculty)$indeg)
```

# Coloring vertices: `vertex.color = ~ attr`

Passing `vertex.color = ~ Group` colors each vertex according to its `Group`
attribute. netplot detects the type of the attribute and picks a sensible
scale automatically:

* **character / factor** attributes are mapped to a categorical palette,
* **numeric** attributes to a continuous gradient, and
* **logical** attributes to two contrasting colors.

When you *print* a plot built this way, netplot also draws a matching legend.

```{r color-factor, fig.cap="Vertices colored by the categorical `Group` attribute, with an automatic legend."}
nplot(UKfaculty, layout = l, vertex.color = ~ Group)
```

The same syntax works for a **numeric** attribute, in which case a continuous
color gradient is used. netplot detects that the attribute is continuous and
draws a **color bar** (rather than a set of discrete keys) as the legend:

```{r color-numeric, fig.cap="Vertices colored by in-degree (a numeric attribute) using a continuous gradient, with a color-bar legend."}
nplot(UKfaculty, layout = l, vertex.color = ~ indeg)
```

And for a **logical** attribute, mapping the two values to two colors:

```{r color-logical, fig.cap="Vertices colored by a logical attribute (hub vs. non-hub)."}
nplot(UKfaculty, layout = l, vertex.color = ~ is_hub)
```

# Shaping vertices: `vertex.nsides = ~ attr`

`vertex.nsides` controls the number of sides of each vertex polygon (3 = a
triangle, 4 = a square, and larger numbers approximate a circle). Passing a
formula maps every unique value of the attribute to a **distinct shape**, which
is handy for encoding a second categorical variable alongside color:

```{r shape, fig.cap="Vertex color *and* shape mapped from the same `Group` attribute."}
nplot(
  UKfaculty,
  layout        = l,
  vertex.color  = ~ Group,
  vertex.nsides = ~ Group
)
```

Because each group gets both a color and a shape, the figure stays readable even
when printed in grayscale.

# Sizing vertices: `vertex.size = ~ attr`

`vertex.size` accepts a formula naming a **numeric** vertex attribute. Values
are rescaled to the range given by `vertex.size.range`, so more central actors
show up as larger nodes:

```{r size, fig.cap="Vertex size mapped from in-degree."}
nplot(
  UKfaculty,
  layout            = l,
  vertex.size       = ~ indeg,
  vertex.size.range = c(.01, .04, 4)
)
```

# Putting it together

The formula interfaces compose, so a single `nplot()` call can encode several
variables at once — color, shape, and size — with very little code:

```{r combined, fig.cap="Color, shape, and size all mapped from graph attributes in a single call."}
nplot(
  UKfaculty,
  layout            = l,
  vertex.color      = ~ Group,   # color by department
  vertex.nsides     = ~ Group,   # distinct shape per department
  vertex.size       = ~ indeg,   # size by popularity (in-degree)
  vertex.size.range = c(.01, .04, 4)
)
```

# Scaling edges: `edge.width = ~ attr`

Edges have their own numeric attributes too. `edge.width = ~ attr` scales edge
widths from a numeric **edge** attribute; the values are normalized and mapped
to `edge.width.range`. Here we use the friendship `weight`:

```{r edge-width, fig.cap="Edge width mapped from the numeric `weight` edge attribute."}
nplot(
  UKfaculty,
  layout           = l,
  edge.width       = ~ weight,
  edge.width.range = c(1, 4, 4),
  skip.arrows      = TRUE
)
```

For `vertex.nsides`, `vertex.size`, and `edge.width`, the right-hand side of the
formula is *evaluated* with the graph's attributes in scope, so you are not
limited to bare attribute names — expressions work too, e.g.
`edge.width = ~ log1p(weight)` or `vertex.size = ~ degree ^ 2`.

# Coloring edges: `edge.color = ~ ego(...) + alter(...)`

Edge *colors* use a richer, dedicated grammar built from two special terms:

* `ego()` — the **source** vertex of the edge, and
* `alter()` — the **target** vertex.

Each term borrows its color from the corresponding endpoint's `vertex.color`
(unless you pass an explicit `col`), and each accepts three tweaks:

* `col` — the base color,
* `alpha` — transparency, from 0 (transparent) to 1 (opaque), and
* `mix` — how much weight that endpoint contributes when the two colors are
  blended along the edge.

The default, `~ ego(alpha = .1, col = "gray") + alter`, fades each edge from a
faint gray at the source to the target's color. The panels below vary `mix` to
shift the blend from *alter only* to *ego only*:

```{r edge-color, fig.width=7, fig.height=3, fig.cap="Varying the `mix` between `ego` and `alter` in the edge color formula. Left: alter only. Middle: an even blend. Right: ego only."}
gridExtra::grid.arrange(
  nplot(UKfaculty, layout = l, vertex.color = ~ Group,
        edge.color = ~ ego(mix = 0, alpha = .1) + alter(mix = 1)),
  nplot(UKfaculty, layout = l, vertex.color = ~ Group,
        edge.color = ~ ego(mix = .5, alpha = .1) + alter(mix = .5)),
  nplot(UKfaculty, layout = l, vertex.color = ~ Group,
        edge.color = ~ ego(mix = 1, alpha = .1) + alter(mix = 0)),
  ncol = 3
)
```

# Applying formulas after the fact

The attribute-mapping formulas for vertex color also work with
`set_vertex_gpar()`, so you can recolor an existing plot without rebuilding it:

```{r set-gpar, fig.cap="Recoloring an existing plot by attribute using `set_vertex_gpar()`."}
np <- nplot(UKfaculty, layout = l)

set_vertex_gpar(np, element = "core", fill = ~ Group)
```

# Summary

* Use `vertex.color = ~ attr` to color vertices by an attribute (with an
  automatic legend), `vertex.nsides = ~ attr` to give each category a shape, and
  `vertex.size = ~ attr` / `edge.width = ~ attr` to scale sizes and widths from
  numeric attributes.
* Use `edge.color = ~ ego(...) + alter(...)` to blend edge colors from their
  endpoints; see `?\`netplot-formulae\`` for the full grammar.
* All of these compose, letting a single `nplot()` call encode several variables
  at once.
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{nplot}
\alias{nplot}
\alias{netplot}
\alias{nplot.igraph}
\alias{nplot.network}
\alias{nplot.matrix}
\alias{nplot.default}
\alias{print.netplot}
\title{Plot a network}
\usage{
nplot(
  x,
  layout,
  vertex.size = 1,
  bg.col = "transparent",
  vertex.nsides = 10,
  vertex.color = grDevices::hcl.colors(1),
  vertex.size.range = c(0.01, 0.03, 4),
  vertex.frame.color = NULL,
  vertex.rot = 0,
  vertex.frame.prop = 0.2,
  vertex.label = NULL,
  vertex.label.fontsize = NULL,
  vertex.label.color = adjustcolor("black", alpha.f = 0.8),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface = "plain",
  vertex.label.show = 0.3,
  vertex.label.range = c(5, 15),
  edge.width = 1,
  edge.width.range = c(1, 2),
  edge.arrow.size = NULL,
  edge.color = ~ego(alpha = 0.1, col = "gray") + alter,
  edge.curvature = pi/3,
  edge.line.lty = "solid",
  edge.line.breaks = 5,
  sample.edges = 1,
  skip.vertex = FALSE,
  skip.edges = FALSE,
  skip.arrows = skip.edges,
  add = FALSE,
  zero.margins = TRUE,
  edgelist
)

\method{nplot}{igraph}(
  x,
  layout = igraph::layout_nicely(x),
  vertex.size = igraph::degree(x, mode = "in"),
  bg.col = "transparent",
  vertex.nsides = 10,
  vertex.color = grDevices::hcl.colors(1),
  vertex.size.range = c(0.01, 0.03, 4),
  vertex.frame.color = NULL,
  vertex.rot = 0,
  vertex.frame.prop = 0.2,
  vertex.label = igraph::vertex_attr(x, "name"),
  vertex.label.fontsize = NULL,
  vertex.label.color = adjustcolor("black", alpha.f = 0.8),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface = "plain",
  vertex.label.show = 0.3,
  vertex.label.range = c(5, 15),
  edge.width = igraph::edge_attr(x, "weight"),
  edge.width.range = c(1, 2),
  edge.arrow.size = NULL,
  edge.color = ~ego(alpha = 0.1, col = "gray") + alter,
  edge.curvature = pi/3,
  edge.line.lty = "solid",
  edge.line.breaks = 5,
  sample.edges = 1,
  skip.vertex = FALSE,
  skip.edges = FALSE,
  skip.arrows = !igraph::is_directed(x),
  add = FALSE,
  zero.margins = TRUE,
  edgelist
)

\method{nplot}{network}(
  x,
  layout = sna::gplot.layout.kamadakawai(x, NULL),
  vertex.size = sna::degree(x, cmode = "indegree"),
  bg.col = "transparent",
  vertex.nsides = 10,
  vertex.color = grDevices::hcl.colors(1),
  vertex.size.range = c(0.01, 0.03, 4),
  vertex.frame.color = NULL,
  vertex.rot = 0,
  vertex.frame.prop = 0.2,
  vertex.label = network::get.vertex.attribute(x, "vertex.names"),
  vertex.label.fontsize = NULL,
  vertex.label.color = adjustcolor("black", alpha.f = 0.8),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface = "plain",
  vertex.label.show = 0.3,
  vertex.label.range = c(5, 15),
  edge.width = network::get.edge.attribute(x, "weight"),
  edge.width.range = c(1, 2),
  edge.arrow.size = NULL,
  edge.color = ~ego(alpha = 0.1, col = "gray") + alter,
  edge.curvature = pi/3,
  edge.line.lty = "solid",
  edge.line.breaks = 5,
  sample.edges = 1,
  skip.vertex = FALSE,
  skip.edges = FALSE,
  skip.arrows = !network::is.directed(x),
  add = FALSE,
  zero.margins = TRUE,
  edgelist
)

\method{nplot}{matrix}(
  x,
  layout,
  vertex.size = 1,
  bg.col = "transparent",
  vertex.nsides = 10,
  vertex.color = grDevices::hcl.colors(1),
  vertex.size.range = c(0.01, 0.03, 4),
  vertex.frame.color = NULL,
  vertex.rot = 0,
  vertex.frame.prop = 0.2,
  vertex.label = NULL,
  vertex.label.fontsize = NULL,
  vertex.label.color = adjustcolor("black", alpha.f = 0.8),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface = "plain",
  vertex.label.show = 0.3,
  vertex.label.range = c(5, 15),
  edge.width = 1,
  edge.width.range = c(1, 2),
  edge.arrow.size = NULL,
  edge.color = ~ego(alpha = 0.1, col = "gray") + alter,
  edge.curvature = pi/3,
  edge.line.lty = "solid",
  edge.line.breaks = 5,
  sample.edges = 1,
  skip.vertex = FALSE,
  skip.edges = FALSE,
  skip.arrows = skip.edges,
  add = FALSE,
  zero.margins = TRUE,
  edgelist
)

\method{nplot}{default}(
  x,
  layout,
  vertex.size = 1,
  bg.col = "transparent",
  vertex.nsides = 10,
  vertex.color = grDevices::hcl.colors(1),
  vertex.size.range = c(0.01, 0.03, 4),
  vertex.frame.color = NULL,
  vertex.rot = 0,
  vertex.frame.prop = 0.2,
  vertex.label = NULL,
  vertex.label.fontsize = NULL,
  vertex.label.color = adjustcolor("black", alpha.f = 0.8),
  vertex.label.fontfamily = "sans",
  vertex.label.fontface = "plain",
  vertex.label.show = 0.3,
  vertex.label.range = c(5, 15),
  edge.width = 1,
  edge.width.range = c(1, 2),
  edge.arrow.size = NULL,
  edge.color = ~ego(alpha = 0.1, col = "gray") + alter,
  edge.curvature = pi/3,
  edge.line.lty = "solid",
  edge.line.breaks = 5,
  sample.edges = 1,
  skip.vertex = FALSE,
  skip.edges = FALSE,
  skip.arrows = skip.edges,
  add = FALSE,
  zero.margins = TRUE,
  ...,
  edgelist
)

\method{print}{netplot}(x, y = NULL, newpage = TRUE, legend = TRUE, ...)
}
\arguments{
\item{x}{A graph. It supports networks stored as \code{igraph}, \code{network}, and
matrices objects (see details).}

\item{layout}{Numeric two-column matrix with the graph layout in x/y positions of the vertices.}

\item{vertex.size}{Numeric vector of length \code{vcount(x)}. Absolute size of the
vertex from 0 to 1. Can also be a one-sided formula (e.g. \code{~ degree}) naming a
numeric vertex attribute to map sizes from (see "Mapping attributes with
formulas").}

\item{bg.col}{Color of the background.}

\item{vertex.nsides}{Numeric vector of length \code{vcount(x)}. Number of sides of
the vertex. E.g. three is a triangle, and 100 approximates a circle. Can also
be a one-sided formula (e.g. \code{~ group}) naming a vertex attribute; each unique
value is then mapped to a distinct shape (see "Mapping attributes with
formulas").}

\item{vertex.color}{Vector of length \code{vcount(x)}. Vertex HEX or built in
colors. Can also be a one-sided formula (e.g. \code{~ group}) naming a vertex
attribute to color vertices by (see "Mapping attributes with formulas").}

\item{vertex.size.range}{Numeric vector of length 2 or 3, or \code{NULL}. Relative size for the
minimum and maximum of the plot, and curvature of the scale. The third number
is used as \code{size^rel[3]}. If \code{NULL}, scaling is suppressed and \code{vertex.size}
is used as is.}

\item{vertex.frame.color}{Vector of length \code{vcount(x)}. Border of vertex in
HEX or built in colors.}

\item{vertex.rot}{Vector of length \code{vcount(x)} in Radians. Passed to \link{npolygon},
elevation degree from which the polygon is drawn.}

\item{vertex.frame.prop}{Vector of length \code{vcount(x)}. What proportion of the
vertex does the frame occupy (values between 0 and 1).}

\item{vertex.label}{Character vector of length \code{vcount(x)}. Labels.}

\item{vertex.label.fontsize}{Numeric vector.}

\item{vertex.label.color}{Vector of colors of length \code{vcount(x)}.}

\item{vertex.label.fontfamily}{Character vector of length \code{vcount(x)}.}

\item{vertex.label.fontface}{See \link[grid:gpar]{grid::gpar}}

\item{vertex.label.show}{Numeric scalar. Proportion of labels to show as the
top ranking according to \code{vertex.size}.}

\item{vertex.label.range}{Numeric vector of size 2 or 3. Relative scale of
\code{vertex.label.fontsize} in points (see \link[grid:gpar]{grid::gpar}).}

\item{edge.width}{Numeric vector of length \code{ecount(x)}. Relative edge widths.
Values are normalized and then mapped to the range specified by \code{edge.width.range},
unless \code{edge.width.range} is \code{NULL}.
For \code{nplot.igraph} and \code{nplot.network}, defaults to the "weight" edge
attribute if present; otherwise all edges use width 1. Can also be a one-sided
formula (e.g. \code{~ weight}) naming a numeric edge attribute (see "Mapping
attributes with formulas").}

\item{edge.width.range}{Numeric vector of length 2, or \code{NULL}. The minimum and maximum line
widths (in points) to use when mapping \code{edge.width} values. For example,
\code{c(1, 4)} maps the smallest edge weight to 1pt and the largest to 4pt.
If \code{NULL}, scaling is suppressed and \code{edge.width} is used as is.}

\item{edge.arrow.size}{Vector of length \code{ecount(x)} from 0 to 1.}

\item{edge.color}{A vector of length \code{ecount(x)}. In HEX or built in colors.
Can be \code{NULL} in which case
the color is picked as a mixture between ego and alters' \code{vertex.color} values.}

\item{edge.curvature}{Numeric vector of length \code{ecount(x)}. Curvature of edges
in terms of radians.}

\item{edge.line.lty}{Vector of length \code{ecount(x)}. Line types in R
(e.g.- 1 = Solid, 2 = Dashed, etc).}

\item{edge.line.breaks}{Vector of length \code{ecount(x)}. Number of vertices to
draw (approximate) the arc (edge).}

\item{sample.edges}{Numeric scalar between 0 and 1. Proportion of edges to
sample.}

\item{skip.vertex, skip.edges, skip.arrows}{Logical scalar. When \code{TRUE} the
object is not plotted.}

\item{add}{Logical scalar.}

\item{zero.margins}{Logical scalar.}

\item{edgelist}{An edgelist.}

\item{y, ...}{Ignored}

\item{newpage}{Logical scalar. When \code{TRUE} calls \link[grid:grid.newpage]{grid::grid.newpage}.}

\item{legend}{Logical scalar. When \code{TRUE} it adds a legend.}
}
\value{
An object of class \code{c("netplot", "gTree", "grob", "gDesc")}. The object
has an additional set of attributes:
\itemize{
\item \verb{.xlim, .ylim} vector of size two with the x-asis/y-axis limits.
\item \code{.layout} A numeric matrix of size \code{vcount(x) * 2} with the vertices positions
\item \code{.edgelist} A numeric matrix, The edgelist.
}

In the case of \code{nplot.default}, an object of class \code{netplot} and \code{grob} (see
\link[grid:grob]{grid::grob}) with the following slots:
\itemize{
\item \code{children} The main \code{grob} of the object.
\item \code{name} Character scalar. The name of the plot
\item \code{.xlim} and \code{.ylim} Two vectors indicating the limits of the plot
\item \code{.layout} A two-column matrix with the location of the vertices.
\item \code{.edgelist} A two-column matrix, an edgelist.
\item \code{.N} Integer. The number of vertices.
\item \code{.M} Integer. The number of edges.
}

The \code{children} \code{grob} contains the following two objects:
\itemize{
\item \code{background} a \code{grob} rectangule.
\item \code{graph} a \code{gTree} that contains each vertex and each edge
of the figure.
}
}
\description{
\code{nplot()} is the main function of the \strong{netplot} package. It draws a network
using the \strong{grid} graphics system (the same engine that powers \strong{ggplot2}),
emphasizing aesthetics and providing sensible defaults that yield
out-of-the-box nice visualizations. Compared with base \code{igraph}/\code{network}
plots, \code{nplot()} auto-scales vertices and edges relative to the plotting
device, draws truly curved edges, mixes edge colors from their endpoints,
and fills the device efficiently.
}
\details{
Vertex and edge aesthetics can be set directly (passing a vector) or,
conveniently, mapped from graph attributes using a \emph{formula} interface (see
the "Mapping attributes with formulas" section below). The returned object is
a \strong{grid} \code{grob}, so it can be further edited with \code{\link[=set_vertex_gpar]{set_vertex_gpar()}} /
\code{\link[=set_edge_gpar]{set_edge_gpar()}}, combined with other grid graphics (e.g. via
\code{gridExtra::grid.arrange()}), or annotated with a legend through
\code{\link[=nplot_legend]{nplot_legend()}}.

When \code{x} is of class \link{matrix}, it will be passed to \code{\link[igraph:graph_from_adjacency_matrix]{igraph::graph_from_adjacency_matrix()}}.
}
\section{Mapping attributes with formulas}{


Several aesthetics can be mapped directly from graph attributes by passing a
one-sided formula naming the attribute, instead of building the vector by
hand. The mapping depends on the aesthetic:
\itemize{
\item \code{vertex.color = ~ attr} colors vertices by the vertex attribute \code{attr}.
Character/factor attributes are mapped to a categorical palette, numeric
attributes to a continuous gradient, and logical attributes to two colors.
When used this way, \code{print()}-ing the resulting plot also draws a matching
legend: a categorical key for discrete attributes and a continuous color
bar for continuous ones.
\item \code{vertex.nsides = ~ attr} maps each unique value of \code{attr} to a distinct
vertex shape (triangle, square, pentagon, ...).
\item \code{vertex.size = ~ attr} and \code{edge.width = ~ attr} scale sizes/widths from a
numeric vertex/edge attribute.
\item \code{edge.color} uses a different, richer formula grammar based on \code{ego()} and
\code{alter()} to mix the endpoints' colors; see \link{netplot-formulae}.
}

For \code{vertex.nsides}, \code{vertex.size}, and \code{edge.width} the right-hand side of
the formula is \emph{evaluated} with the graph's attributes in scope, so besides
bare names you can use expressions, e.g. \code{edge.width = ~ log1p(weight)} or
\code{vertex.size = ~ degree ^ 2}.

For example, \code{nplot(x, vertex.color = ~ gender, vertex.size = ~ degree)}
colors vertices by the \code{gender} attribute and sizes them by \code{degree}. The
same attribute-mapping formulas also work in \code{\link[=set_vertex_gpar]{set_vertex_gpar()}}.
}

\examples{
library(igraph)
library(netplot)
set.seed(1)
x <- sample_smallworld(1, 200, 5, 0.03)

plot(x) # ala igraph
nplot(x) # ala netplot

# Mapping aesthetics from vertex attributes using formulas
V(x)$grp <- sample(letters[1:3], vcount(x), replace = TRUE)
V(x)$deg <- degree(x)
nplot(
  x,
  vertex.color  = ~ grp, # color by the categorical attribute
  vertex.nsides = ~ grp, # and give each group a distinct shape
  vertex.size   = ~ deg  # size by a numeric attribute
)
}
\seealso{
\link{nplot_base}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coloring.R
\name{netplot-formulae}
\alias{netplot-formulae}
\alias{color_formula}
\alias{ego}
\alias{alter}
\title{Edge-color formulas in \code{netplot}}
\usage{
color_formula(x, col, alpha, env, type, mix = 1, postfix = NULL)

ego(...)

alter(...)
}
\arguments{
\item{x}{An object of class \link{netplot}.}

\item{col}{Any valid color. Can be a single color or a vector.}

\item{alpha}{Number. Alpha levels}

\item{env, type, postfix}{For internal use only.}

\item{mix}{Number. For mixing colors between \code{ego} and \code{alter}}

\item{...}{Passed to \code{color_formula}.}
}
\value{
Nothing. These functions are called internally when using
formulas. \code{color_formula} modifies the environment \code{env}.
}
\description{
Edge colors in both \code{\link[=nplot]{nplot()}} and \code{\link[=set_edge_gpar]{set_edge_gpar()}} can be specified using
a formula based on \code{ego()} and \code{alter()} (source and target). This way the
user can set various types of combination varying the mixing of the colors,
the alpha levels, and the actual mixing colors to create edge colors.
}
\details{
The formula is one-sided and combines the special terms \code{ego()} (the source
vertex) and \code{alter()} (the target vertex) with \code{+}. Each term draws its color
from the corresponding endpoint's \code{vertex.color} unless a \code{col} is supplied,
and accepts:
\itemize{
\item \code{col}: the base color to use for that endpoint.
\item \code{alpha}: transparency, from 0 (fully transparent) to 1 (opaque).
\item \code{mix}: how much weight that endpoint's color receives when the two are
blended along the edge.
}

For instance, \code{~ ego(alpha = .1, col = "gray") + alter} (the default) fades
each edge from a faint gray at the source to the target's color, producing the
characteristic netplot look.

This grammar is specific to \emph{edge colors}. To map other aesthetics (vertex
color, shape, size, or edge width) from a graph attribute, pass a formula
naming the attribute directly to \code{\link[=nplot]{nplot()}} (e.g. \code{vertex.color = ~ group});
see the "Mapping attributes with formulas" section of \code{\link[=nplot]{nplot()}}.
}
\examples{
if (require(gridExtra) & require(magrittr)) {
  library(igraph)
  net <- make_ring(4)

  set.seed(1)
  np <- nplot(net, vertex.color = grDevices::hcl.colors(4), vertex.size.range=c(.1, .1))
  np \%<>\% set_edge_gpar(lwd = 4)

  grid.arrange(
    np,
    np \%>\% set_edge_gpar(col =~ego + alter),
    np \%>\% set_edge_gpar(col =~ego(alpha=0) + alter),
    np \%>\% set_edge_gpar(col =~ego + alter(alpha=0)),
    np \%>\% set_edge_gpar(col =~ego(mix=0) + alter(mix=1)),
    np \%>\% set_edge_gpar(col =~ego(mix=1) + alter(mix=0))
  )
}
}
\seealso{
\code{\link[=nplot]{nplot()}} for the attribute-mapping formulas, and \code{\link[=set_edge_gpar]{set_edge_gpar()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpar.R
\name{set_gpar}
\alias{set_gpar}
\alias{set_edge_gpar}
\alias{set_vertex_gpar}
\alias{get_vertex_gpar}
\alias{get_edge_gpar}
\alias{get_gpar}
\title{Set/retrieve graphical parameters of a \code{netplot} object}
\usage{
set_gpar(x, type, element, idx, ...)

set_edge_gpar(x, element, idx, ...)

set_vertex_gpar(x, element, idx, ...)

get_vertex_gpar(x, element, ..., idx)

get_edge_gpar(x, element, ..., idx)

get_gpar(x, type, element, ..., idx, simplify = TRUE)
}
\arguments{
\item{x}{An object of class \code{netplot}.}

\item{type}{Character. Either \code{"edge"} or \code{"vertex"}.}

\item{element}{Character. If \code{"edge"}, then it can be either \code{"line"} or
\code{"arrow"}, otherwise it can be either \code{"core"} or \code{"frame"}.}

\item{idx}{(optional) Integer vector. Indices of the elements to be modified.
When missing, all elements are modified.}

\item{...}{Parameters to be modified/retrieved. This is passed to \link[grid:editGrob]{grid::editGrob}
via \link[grid:gpar]{grid::gpar}.}

\item{simplify}{Logical. When \code{TRUE} it tries to simplify the result.
Otherwise it returns a nested list.}
}
\value{
An object of class \code{netplot} with modified parameters.
}
\description{
Set/retrieve graphical parameters of a \code{netplot} object
}
\details{
\code{set_edge_gpar} and \code{set_vertex_gpar} are shorthands for
\code{set_gpar(type = "edge", ...)} and \code{set_gpar(type = "vertex", ...)}
respectively.

\code{get_edge_gpar} and \code{get_vertex_gpar} are shorthands for
\code{get_gpar(type = "edge", ...)} and \code{get_gpar(type = "vertex", ...)}
respectively.
}
\examples{
library(igraph)
library(netplot)

x <- make_ring(5)

g <- nplot(x)

# Updating edge color
g <- set_edge_gpar(g, col = "gray80")

# Retrieving the color of the vertices (core)
get_vertex_gpar(g, element = "core", "fill", "lwd")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nplot_legend.R
\name{nplot_legend}
\alias{nplot_legend}
\alias{print.netplot_legend}
\title{Add legend to a netplot object}
\usage{
nplot_legend(
  g,
  labels,
  pch,
  gp = grid::gpar(),
  ...,
  packgrob.args = list(side = "left")
)

\method{print}{netplot_legend}(x, y = NULL, newpage = TRUE, ...)
}
\arguments{
\item{g}{An object of class \link{netplot}.}

\item{labels}{Character vector of labels.}

\item{pch}{See \code{\link[graphics:points]{graphics::points()}}.}

\item{gp}{An object of class \code{\link[grid:gpar]{grid::gpar()}}}

\item{...}{Further arguments passed to \code{\link[grid:legendGrob]{grid::legendGrob()}}.}

\item{packgrob.args}{List of arguments passed to \code{\link[grid:packGrob]{grid::packGrob()}}.}

\item{x}{An object of class \code{netplot_legend}.}

\item{y}{Ignored.}

\item{newpage}{Logical scalar. When \code{TRUE} it calls \code{\link[grid:grid.newpage]{grid::grid.newpage()}}.}
}
\value{
A frame grob.
}
\description{
Legends in \link{grid} graphics is a bit more complicated than in base graphics.
The function \code{nplot_legend} is a wrapper of \code{\link[grid:legendGrob]{grid::legendGrob()}} that makes
the process easier. Besides \code{labels}, the main visual arguments for the
figure ar passed through the \code{gp} argument (see examples).
}
\examples{
library(igraph)
library(netplot)
set.seed(1)
x <- sample_smallworld(1, 200, 5, 0.03)
V(x)$nsides <- sample(c(10, 4), 200, replace = TRUE)

g <- nplot(
  x,
  vertex.nsides = V(x)$nsides,
  vertex.color  = ifelse(V(x)$nsides == 4, "red", "steelblue"),
  edge.line.breaks = 5
  )

nplot_legend(
  g,
  labels = c("circle", "diamond", "edge"),
  pch    = c(21, 23, NA),
  gp     = gpar(
    fill = c("steelblue", "red", NA),
    lwd  = c(NA, NA, 1),
    col  = c(NA, NA, "purple")
    )
  )
grid.text("Legend to the left (default)", y = unit(.95, "npc"), just = "bottom")

nplot_legend(
  g,
  labels = c("circle", "diamond", "edge"),
  pch    = c(21, 23, NA),
  gp     = gpar(
    fill = c("steelblue", "red", NA),
    lwd  = c(NA, NA, 1),
    col  = c(NA, NA, "purple")
    ),
  # These two extra options set the legend to the bottom
  packgrob.args = list(side = "bottom"),
  ncol = 3
  )
grid.text("Legend bottom", y = unit(.95, "npc"), just = "bottom")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot.R
\name{locate_vertex}
\alias{locate_vertex}
\title{Find a vertex in the current plot}
\usage{
locate_vertex(x = NULL)
}
\arguments{
\item{x}{An object of class \code{netplot}}
}
\value{
A list with the name of the vertex, the x and y coordinates and the
viewport where it is located.
}
\description{
This function is a wrapper of \code{\link[grid:grid.locator]{grid::grid.locator()}}, and provides a way to
find the coordinates of a vertex in the current plot. It is useful to
identify the vertex that is being clicked in a plot.
}
\details{
This function only works in interactive mode. Once it is called,
the user can click on a vertex in the plot. The function will return the
name of the vertex, the x and y coordinates and the viewport where it is
located. If \code{x} is not specified, the last plotted \code{netplot} object will be
used.
}
\examples{
library(igraph)
library(netplot)
set.seed(1)
x <- sample_smallworld(1, 200, 5, 0.03)

# Plotting
nplot(x)

# Clicking (only works in interactive mode)
if (interactive()) {
 res <- locate_vertex()
 print(res)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color-paletts.R
\name{colorRamp2}
\alias{colorRamp2}
\title{A faster implementation of \link[grDevices:colorRamp]{grDevices::colorRamp} for linear interpolation.}
\usage{
colorRamp2(x, alpha = TRUE, thresholds = NULL)
}
\arguments{
\item{x}{A vector of colors.}

\item{alpha}{Logical scalar. When \code{TRUE}
This implementation of \code{colorRamp} can be 2 or more times faster than the \code{grDevices}
version. It is intended for consecutive calls (i.e. in a loop) to improve
performance. It is equivalent to the linear interpolation of the function
\code{colorRamp}.}

\item{thresholds}{A numeric vector of length \code{length(x)}. Optional threshold
levels so that the mixing can be different that even.}
}
\value{
A function as in \link[grDevices:colorRamp]{grDevices::colorRamp}.
}
\description{
A faster implementation of \link[grDevices:colorRamp]{grDevices::colorRamp} for linear interpolation.
}
\examples{

# Creating a function for 2 colors
myf <- colorRamp2(c("black", "steelblue"))
f   <- colorRamp(c("black", "steelblue"))

plot.new()
plot.window(xlim = c(0,2), ylim = c(1, 11))

# These should be the same colors
rect(
  xleft   = 0,
  xright  = 1,
  ybottom = 1:10,
  ytop    = 2:11,
  col = rgb(myf((1:10)/10), maxColorValue = 255)
  )
rect(
  xleft   = 1,
  xright  = 2,
  ybottom = 1:10,
  ytop    = 2:11,
  col = rgb(f((1:10)/10), maxColorValue = 255)
)

# Another example setting different thresholds
myf  <- colorRamp2(c("black", "steelblue"))
myf2 <- colorRamp2(c("black", "steelblue"), thresholds=c(0, .7))

plot.new()
plot.window(xlim = c(0,2), ylim = c(1, 11))

# These should be the same colors
rect(
  xleft   = 0,
  xright  = 1,
  ybottom = 1:10,
  ytop    = 2:11,
  col = rgb(myf((1:10)/10), maxColorValue = 255)
  )
rect(
  xleft   = 1,
  xright  = 2,
  ybottom = 1:10,
  ytop    = 2:11,
  col = rgb(myf2((1:10)/10), maxColorValue = 255)
)



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot-package.R
\docType{package}
\name{netplot-package}
\alias{netplot-package}
\title{netplot: Beautiful Graph Drawing}
\description{
netplot is a graph visualization engine that emphasizes aesthetics while
providing default parameters that yield out-of-the-box nice visualizations.
It is built on top of the \strong{grid} graphics system (the same engine behind
\strong{ggplot2}) and works seamlessly with \code{igraph} and \code{network} objects.
}
\details{
The main entry point is \code{\link[=nplot]{nplot()}}. See \code{vignette("examples", package = "netplot")} for a general overview and \code{vignette("formulas", package = "netplot")} for mapping vertex/edge aesthetics from graph attributes.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/USCCANA/netplot}
  \item \url{https://usccana.github.io/netplot/}
  \item Report bugs at \url{https://github.com/USCCANA/netplot/issues}
}

}
\author{
\strong{Maintainer}: George Vega Yon \email{g.vegayon@gmail.com} (\href{https://orcid.org/0000-0002-3171-0844}{ORCID})

Authors:
\itemize{
  \item George Vega Yon \email{g.vegayon@gmail.com} (\href{https://orcid.org/0000-0002-3171-0844}{ORCID})
  \item Porter Bischoff \email{portergbischoff@gmail.com} (\href{https://orcid.org/0009-0004-6742-6281}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piechart.R
\name{colorkey}
\alias{colorkey}
\title{Function to create a color key}
\usage{
colorkey(
  x0,
  y0,
  x1,
  y1,
  cols = c("white", "steelblue"),
  tick.range = c(0, 1),
  tick.marks = seq(tick.range[1], tick.range[2], length.out = 5L),
  label.from = NULL,
  label.to = NULL,
  nlevels = 100,
  main = NULL,
  relative = TRUE,
  tick.args = list(),
  label.args = list(),
  main.args = list()
)
}
\arguments{
\item{x0, x1, y0, y1}{Numeric scalars. Coordinates of the lower left and
upper right points where the color key will be drawn as proportion of
the plotting region.}

\item{cols}{Character scalar. Colors specifications to create the
color palette.}

\item{tick.range, tick.marks}{Numeric vectors specifying the range and
the tickmarks respectively.}

\item{label.from, label.to}{Character scalar. Labels of the lower and
upper values of the color key.}

\item{nlevels}{Integer scalar. Number of levels to extrapolate.}

\item{main}{Character scalar. Title of the colorkey.}

\item{relative}{Logical scalar. When \code{TRUE} the color key is drawn relative
to the plotting region area taking \verb{x0, x1, y0, y1} as relative location.}

\item{tick.args, label.args, main.args}{Lists of arguments passed to
\link[graphics:text]{graphics::text} for drawing ticks, labels and main respectively.}
}
\value{
NULL.
}
\description{
Function to create a color key
}
\examples{
set.seed(22231)

# A random figure
dat <- matrix(runif(100*3), ncol = 3)
col <- colorRamp2(c("blue", "white", "red"))

plot(
  dat[,1], dat[,2],
  col = rgb(col(dat[,3]), maxColorValue=255),
  cex=2, pch=20
  )

# Pretty color key
colorkey(
  x0 = .60, y0 = .80,
  x1 = .95, y1 = .95,
  cols = c("blue", "white", "red"),
  main = "Some color scale"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/netplot_base.R
\name{nplot_base}
\alias{nplot_base}
\title{\code{nplot} using base graphics}
\usage{
nplot_base(
  x,
  layout = igraph::layout_nicely(x),
  vertex.size = igraph::degree(x, mode = "in"),
  bg.col = "transparent",
  vertex.nsides = 10,
  vertex.color = grDevices::hcl.colors(1),
  vertex.size.range = c(0.01, 0.03, 4),
  vertex.frame.color = grDevices::adjustcolor(vertex.color, red.f = 0.75, green.f = 0.75,
    blue.f = 0.75),
  vertex.rot = 0,
  vertex.frame.prop = 0.1,
  edge.width = NULL,
  edge.width.range = c(1, 2),
  edge.arrow.size = NULL,
  edge.color = NULL,
  edge.color.mix = 0.5,
  edge.color.alpha = c(0.1, 0.5),
  edge.curvature = pi/3,
  edge.line.lty = "solid",
  edge.line.breaks = 5,
  sample.edges = 1,
  skip.vertex = FALSE,
  skip.edges = FALSE,
  skip.arrows = skip.edges,
  add = FALSE,
  zero.margins = TRUE
)
}
\arguments{
\item{x}{A graph. It supports networks stored as \code{igraph}, \code{network}, and
matrices objects (see details).}

\item{layout}{Numeric two-column matrix with the graph layout in x/y positions of the vertices.}

\item{vertex.size}{Numeric vector of length \code{vcount(x)}. Absolute size of the
vertex from 0 to 1. Can also be a one-sided formula (e.g. \code{~ degree}) naming a
numeric vertex attribute to map sizes from (see "Mapping attributes with
formulas").}

\item{bg.col}{Color of the background.}

\item{vertex.nsides}{Numeric vector of length \code{vcount(x)}. Number of sides of
the vertex. E.g. three is a triangle, and 100 approximates a circle. Can also
be a one-sided formula (e.g. \code{~ group}) naming a vertex attribute; each unique
value is then mapped to a distinct shape (see "Mapping attributes with
formulas").}

\item{vertex.color}{Vector of length \code{vcount(x)}. Vertex HEX or built in
colors. Can also be a one-sided formula (e.g. \code{~ group}) naming a vertex
attribute to color vertices by (see "Mapping attributes with formulas").}

\item{vertex.size.range}{Numeric vector of length 2 or 3, or \code{NULL}. Relative size for the
minimum and maximum of the plot, and curvature of the scale. The third number
is used as \code{size^rel[3]}. If \code{NULL}, scaling is suppressed and \code{vertex.size}
is used as is.}

\item{vertex.frame.color}{Vector of length \code{vcount(x)}. Border of vertex in
HEX or built in colors.}

\item{vertex.rot}{Vector of length \code{vcount(x)} in Radians. Passed to \link{npolygon},
elevation degree from which the polygon is drawn.}

\item{vertex.frame.prop}{Vector of length \code{vcount(x)}. What proportion of the
vertex does the frame occupy (values between 0 and 1).}

\item{edge.width}{Numeric vector of length \code{ecount(x)}. Relative edge widths.
Values are normalized and then mapped to the range specified by \code{edge.width.range},
unless \code{edge.width.range} is \code{NULL}.
For \code{nplot.igraph} and \code{nplot.network}, defaults to the "weight" edge
attribute if present; otherwise all edges use width 1. Can also be a one-sided
formula (e.g. \code{~ weight}) naming a numeric edge attribute (see "Mapping
attributes with formulas").}

\item{edge.width.range}{Numeric vector of length 2, or \code{NULL}. The minimum and maximum line
widths (in points) to use when mapping \code{edge.width} values. For example,
\code{c(1, 4)} maps the smallest edge weight to 1pt and the largest to 4pt.
If \code{NULL}, scaling is suppressed and \code{edge.width} is used as is.}

\item{edge.arrow.size}{Vector of length \code{ecount(x)} from 0 to 1.}

\item{edge.color}{A vector of length \code{ecount(x)}. In HEX or built in colors.
Can be \code{NULL} in which case
the color is picked as a mixture between ego and alters' \code{vertex.color} values.}

\item{edge.color.mix}{Proportion of the mixing.}

\item{edge.color.alpha}{Either a vector of length 1 or 2, or a matrix of
size \code{ecount(x)*2} with values in \verb{[0,1]}. Alpha (transparency) levels (see
details)}

\item{edge.curvature}{Numeric vector of length \code{ecount(x)}. Curvature of edges
in terms of radians.}

\item{edge.line.lty}{Vector of length \code{ecount(x)}. Line types in R
(e.g.- 1 = Solid, 2 = Dashed, etc).}

\item{edge.line.breaks}{Vector of length \code{ecount(x)}. Number of vertices to
draw (approximate) the arc (edge).}

\item{sample.edges}{Numeric scalar between 0 and 1. Proportion of edges to
sample.}

\item{skip.vertex, skip.edges, skip.arrows}{Logical scalar. When \code{TRUE} the
object is not plotted.}

\item{add}{Logical scalar.}

\item{zero.margins}{Logical scalar.}
}
\value{
\code{nplot_base} returns a list with the following components:
\itemize{
\item \code{vertex.coords} A list of length \code{N} where each element describes the
geomtry of each vertex.
\item \code{vertex.color} A vector of colors
\item \code{vertex.frame.coords} Similar to \code{vertex.coords}, but for the frame.
\item \code{vertex.frame.color} Similar to \code{vertex.color}, but for the frame.
\item \code{edge.color} Vector of functions used to compute the edge colors.
\item \code{edge.coords} Similar to \code{vertex.coords}, the points that describe each
edge.
\item \code{edge.arrow.coords} A list of matrices describing the geometry of the
tip of the edges.
\item \code{edge.width} A numeric vector with edges' widths.
\item \code{xlim}, \code{ylim} Limits of the plot area.
}
}
\description{
\code{nplot} using base graphics
}
\examples{
# Same example as in nplot
library(igraph)
library(netplot)
set.seed(1)
x <- sample_smallworld(1, 200, 5, 0.03)

nplot_base(x) # ala netplot (using base)
}
\seealso{
\link{nplot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piechart.R
\name{piechart}
\alias{piechart}
\title{A flexible piechart.}
\usage{
piechart(
  x,
  labels = names(x),
  radius = 1,
  doughnut = 0,
  origin = c(0, 0),
  edges = 200,
  slice.off = 0,
  init.angle = 0,
  last.angle = 360,
  tick.len = 0.1,
  text.args = list(),
  segments.args = list(),
  skip.plot.slices = FALSE,
  add = FALSE,
  rescale = TRUE,
  ...
)
}
\arguments{
\item{x}{Numeric vector. Values that specify the area of the slices.}

\item{labels}{Character vector of length \code{length(x)}. Passed to
\code{\link[graphics:text]{graphics::text()}}.}

\item{radius}{Numeric vector. Radious of each slice (can be a scalar).}

\item{doughnut}{Numeric scalar. Radious of each inner circle (doughnut) (can be a scalar).}

\item{origin}{Numeric vector of length 2. Coordinates of the origin.}

\item{edges}{Numeric scalar. Smoothness of the slices curve (can be a vector).}

\item{slice.off}{Numeric vector. When \verb{!=0}, specifies how much to
move the slice away from the origin. When scalar is recycled.}

\item{init.angle}{Numeric scalar. Angle from where to start drawing in degrees.}

\item{last.angle}{Numeric scalar. Angle where to finish drawing in degrees.}

\item{tick.len}{Numeric scalar. Size of the tick marks as proportion of the radius.}

\item{text.args}{List. Further arguments passed to \code{\link[graphics:text]{graphics::text()}}.}

\item{segments.args}{List. Further arguments passed to \code{\link[graphics:segments]{graphics::segments()}}
when drawing the tickmarks.}

\item{skip.plot.slices}{Logical scalar. When \code{FALSE}, slices are not drawn.
This can be useful if, for example, the user only wants to draw the labels.}

\item{add}{Logical scalar. When \code{TRUE} it is added to the current device.}

\item{rescale}{Logical scalar. When \code{TRUE} (default), the y-coordinates of
the polygons (slices), text and tickmarks will be rescaled such that the
aspectt ratio is preserved, i.e. looks like a circle.}

\item{...}{Further arguments passed to \code{\link[graphics:polygon]{graphics::polygon()}}
(see details).}
}
\value{
A list with the following elements:
\item{slices}{A list of length \code{length(x)} with the coordinates of each
slice.}
\item{textcoords}{A numeric matrix of size \code{length(x)*2} with
coordinates where the labels can be put at.}
\item{alpha0}{A numeric vector of size \code{length(x)} with the starting
degree in radians of the slice.}
\item{alpha1}{A numeric vector of size \code{length(x)} with the ending
degree in radians of the slice.}
}
\description{
While similar to \code{\link[graphics:pie]{graphics::pie()}}, this function is much more
flexible as it allows providing different parameters for each slice of the pie.
Furthermore, it allows adding the plot to the current device, making it possible
to create compound piecharts.
}
\details{
The function is a wrapper of \code{\link[graphics:polygon]{graphics::polygon()}},
so all parameters such as color, density, border, etc. are passed directly
by \code{\link[=mapply]{mapply()}} so that are specified one per slice. The coordinates
of the slices are computed internally.
}
\examples{

# Example 1 -----------------------------------------------------------------
# A set of 3 nested rings rings starting at 315 deg. and ending at 270 deg.

# Values to plot
vals <- c(1,2,3,10)

# Outer (includes labels)
piechart(vals, col=grDevices::blues9[5:8], border=NA, doughnut = .5,
    radius=.75, labels=vals, init.angle = 315, last.angle = 270)

# Middle
piechart(vals, col=grDevices::blues9[3:6], border=NA, doughnut = .3,
    radius=.5, add=TRUE, init.angle = 315, last.angle = 270)

# Inner
piechart(vals, col=grDevices::blues9[1:4], border="gray", doughnut = .1,
    radius=.3, add=TRUE, init.angle = 315, last.angle = 270)

# Example 2 -----------------------------------------------------------------
# Passing values to polygon and playing with the radius and slice.off

piechart(1:10, density=(1:10)^2/2, slice.off = (1:10)/30, doughnut = .5,
  radius = sqrt(10:1),
  # Here we are setting random labels...
  labels=sapply(1:10, function(x) paste(sample(letters, x, TRUE), collapse=""))
  )

}
\seealso{
https://commons.wikimedia.org/wiki/File:Nightingale-mortality.jpg
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color-paletts.R
\name{segments_gradient}
\alias{segments_gradient}
\title{Draw segments colored by gradients}
\usage{
segments_gradient(
  x,
  y = NULL,
  col = colorRamp2(c("transparent", "black"), TRUE),
  lend = 1,
  ...
)
}
\arguments{
\item{x, y}{Coordinates passed to \link[grDevices:xy.coords]{grDevices::xy.coords}.}

\item{col}{Color ramp function (see \link[grDevices:colorRamp]{grDevices::colorRamp}).}

\item{lend}{Passed to \link[graphics:segments]{graphics::segments}.}

\item{...}{Further arguments passed to \code{segments}.}
}
\value{
See \link[graphics:segments]{graphics::segments}.
}
\description{
Draw segments colored by gradients
}
\examples{

set.seed(1)
x <- cbind(cumsum(rnorm(1e3, sd=.1)), cumsum(rnorm(1e3, sd=.4)))
plot(x, type="n")
segments_gradient(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_mixer.R
\name{make_colors}
\alias{make_colors}
\alias{make_edges_colors}
\alias{make_vertex_colors}
\title{Create a vector of colors for vertices and edges}
\usage{
make_colors(dat, categorical = FALSE, color_map = grDevices::hcl.colors)

make_edges_colors(x, eattr, ...)

make_vertex_colors(x, vattr, ...)
}
\arguments{
\item{dat}{A vector of data to generate the color from.}

\item{categorical}{Logical. When \code{TRUE} sets the colors as categories.}

\item{color_map}{A function to generate a palette.}

\item{x}{A graph of class \code{network} or \code{igraph}.}

\item{...}{Further arguments passed to \code{make_colors}.}

\item{vattr, eattr}{Character. Names of either vertex or edge variables to be
used for generating the colors.}
}
\value{
A vector of colors with the attribute \code{color_map}. The color map used
to generate the colors.
}
\description{
Using \code{vertex}/\code{edge} attributes, these functions return vectors of colors
that can be used either during the creation of the \link{nplot} object, or
afterwards when changing \code{gpar} (graphical parameter) values with \link{set_gpar}.
}
\details{
If no attribute is provided, then by defaul the colors are set according to
indegree.

\code{x} can be either a graph of class \code{igraph} or \code{network}.
}
\examples{

data(UKfaculty, package="igraphdata")
col <- make_vertex_colors(UKfaculty, "Group")

if (require(magrittr)) {

  nplot(UKfaculty) \%>\%
    set_vertex_gpar("core", fill = col, col=col) \%>\%
    set_vertex_gpar("frame", fill = col, col=col, alpha=.7) \%>\%
    set_edge_gpar(col="gray50", fill="gray50", alpha=.5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polygons.R
\name{npolygon}
\alias{npolygon}
\title{n-sided polygons
Calculate the coordinates for an nsided polygon}
\usage{
npolygon(x = 0, y = 0, n = 6L, r = 1, d = 2 * pi/(n)/2)
}
\arguments{
\item{x, y}{Numeric scalar. Origin of the polygon.}

\item{n}{Integer scalar. Number of sides.}

\item{r}{Numeric scalar. Radious of the polygon.}

\item{d}{Numeric scalar. Starting degree in radians.}
}
\value{
A two column matrix with the coordinates to draw a n sided polygon.
}
\description{
n-sided polygons
Calculate the coordinates for an nsided polygon
}
\examples{
graphics.off()
oldpar <- par(no.readonly = TRUE)

par(xpd = NA, mfrow = c(3, 3), mai = rep(0, 4))
for (n in c(2, 3, 4, 5, 6, 8, 12, 20, 50)) {

  plot.new()
  plot.window(c(-1.25,1.25), c(-1.25,1.25))

  for (i in seq(1, .0005, length.out = 200)) {
    col <- adjustcolor("tomato", alpha.f = i)
    polygon(npolygon(x=(i-1)/4, y = (i-1)/4, r = i, d = i-1, n = n),
            col = NA, border=col)
  }

  mtext(sprintf("n = \%i", n), side = 1, line = -3)
}

par(oldpar)
}
